@@ -223,6 +223,7 @@ alcor_kill_arch(int mode)
 {
 	cia_kill_arch(mode);
 
+#ifndef ALPHA_RESTORE_SRM_SETUP
 	switch(mode) {
 	case LINUX_REBOOT_CMD_RESTART:
 		/* Who said DEC engineer's have no sense of humor? ;-)  */
@@ -238,6 +239,29 @@ alcor_kill_arch(int mode)
 	}
 
 	halt();
+#endif
+}
+
+static void __init
+alcor_init_pci(void)
+{
+	struct pci_dev *dev;
+
+	cia_init_pci();
+
+	/*
+	 * Now we can look to see if we are really running on an XLT-type
+	 * motherboard, by looking for a 21040 TULIP in slot 6, which is
+	 * built into XLT and BRET/MAVERICK, but not available on ALCOR.
+	 */
+	dev = pci_find_device(PCI_VENDOR_ID_DEC,
+			      PCI_DEVICE_ID_DEC_TULIP,
+			      NULL);
+	if (dev && dev->devfn == PCI_DEVFN(6,0)) {
+		alpha_mv.sys.cia.gru_int_req_bits = XLT_GRU_INT_REQ_BITS; 
+		printk(KERN_INFO "%s: Detected AS500 or XLT motherboard.\n",
+		       __FUNCTION__);
+	}
 }
 
 
@@ -262,7 +286,7 @@ struct alpha_machine_vector alcor_mv __initmv = {
 	.init_arch		= cia_init_arch,
 	.init_irq		= alcor_init_irq,
 	.init_rtc		= common_init_rtc,
-	.init_pci		= cia_init_pci,
+	.init_pci		= alcor_init_pci,
 	.kill_arch		= alcor_kill_arch,
 	.pci_map_irq		= alcor_map_irq,
 	.pci_swizzle		= common_swizzle,
@@ -290,7 +314,7 @@ struct alpha_machine_vector xlt_mv __initmv = {
 	.init_arch		= cia_init_arch,
 	.init_irq		= alcor_init_irq,
 	.init_rtc		= common_init_rtc,
-	.init_pci		= cia_init_pci,
+	.init_pci		= alcor_init_pci,
 	.kill_arch		= alcor_kill_arch,
 	.pci_map_irq		= alcor_map_irq,
 	.pci_swizzle		= common_swizzle,
