@@ -14,6 +14,7 @@
  * All initialization functions provided here are intended to be called
  * from machine specific code with proper arguments when required.
  */
+#include <linux/config.h>
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/kernel.h>
@@ -24,6 +25,7 @@
 #include <linux/device.h>
 #include <linux/slab.h>
 #include <linux/spinlock.h>
+#include <linux/dma-mapping.h>
 
 #include <asm/hardware.h>
 #include <asm/mach-types.h>
@@ -33,6 +35,8 @@
 
 #include <asm/hardware/sa1111.h>
 
+extern void __init sa1110_mb_enable(void);
+
 /*
  * We keep the following data for the overall SA1111.  Note that the
  * struct device and struct resource are "fake"; they should be supplied
@@ -42,7 +46,7 @@
  */
 struct sa1111 {
 	struct device	*dev;
-	struct resource	res;
+	unsigned long	phys;
 	int		irq;
 	spinlock_t	lock;
 	void		*base;
@@ -54,108 +58,74 @@ struct sa1111 {
  */
 static struct sa1111 *g_sa1111;
 
-static struct sa1111_dev usb_dev = {
-	.dev = {
-		.name	= "Intel Corporation SA1111 [USB Controller]",
-	},
-	.skpcr_mask	= SKPCR_UCLKEN,
-	.devid		= SA1111_DEVID_USB,
-	.irq = {
-		IRQ_USBPWR,
-		IRQ_HCIM,
-		IRQ_HCIBUFFACC,
-		IRQ_HCIRMTWKP,
-		IRQ_NHCIMFCIR,
-		IRQ_USB_PORT_RESUME
-	},
+struct sa1111_dev_info {
+	unsigned long	offset;
+	unsigned long	skpcr_mask;
+	unsigned int	devid;
+	unsigned int	irq[6];
 };
 
-static struct sa1111_dev sac_dev = {
-	.dev = {
-		.name	= "Intel Corporation SA1111 [Audio Controller]",
+static struct sa1111_dev_info sa1111_devices[] = {
+	{
+		.offset		= SA1111_USB,
+		.skpcr_mask	= SKPCR_UCLKEN,
+		.devid		= SA1111_DEVID_USB,
+		.irq = {
+			IRQ_USBPWR,
+			IRQ_HCIM,
+			IRQ_HCIBUFFACC,
+			IRQ_HCIRMTWKP,
+			IRQ_NHCIMFCIR,
+			IRQ_USB_PORT_RESUME
+		},
 	},
-	.skpcr_mask	= SKPCR_I2SCLKEN | SKPCR_L3CLKEN,
-	.devid		= SA1111_DEVID_SAC,
-	.irq = {
-		AUDXMTDMADONEA,
-		AUDXMTDMADONEB,
-		AUDRCVDMADONEA,
-		AUDRCVDMADONEB
+	{
+		.offset		= 0x0600,
+		.skpcr_mask	= SKPCR_I2SCLKEN | SKPCR_L3CLKEN,
+		.devid		= SA1111_DEVID_SAC,
+		.irq = {
+			AUDXMTDMADONEA,
+			AUDXMTDMADONEB,
+			AUDRCVDMADONEA,
+			AUDRCVDMADONEB
+		},
 	},
-};
-
-static struct sa1111_dev ssp_dev = {
-	.dev = {
-		.name	= "Intel Corporation SA1111 [SSP Controller]",
+	{
+		.offset		= 0x0800,
+		.skpcr_mask	= SKPCR_SCLKEN,
+		.devid		= SA1111_DEVID_SSP,
 	},
-	.skpcr_mask	= SKPCR_SCLKEN,
-	.devid		= SA1111_DEVID_SSP,
-};
-
-static struct sa1111_dev kbd_dev = {
-	.dev = {
-		.name	= "Intel Corporation SA1111 [PS2]",
+	{
+		.offset		= SA1111_KBD,
+		.skpcr_mask	= SKPCR_PTCLKEN,
+		.devid		= SA1111_DEVID_PS2,
+		.irq = {
+			IRQ_TPRXINT,
+			IRQ_TPTXINT
+		},
 	},
-	.skpcr_mask	= SKPCR_PTCLKEN,
-	.devid		= SA1111_DEVID_PS2,
-	.irq = {
-		IRQ_TPRXINT,
-		IRQ_TPTXINT
+	{
+		.offset		= SA1111_MSE,
+		.skpcr_mask	= SKPCR_PMCLKEN,
+		.devid		= SA1111_DEVID_PS2,
+		.irq = {
+			IRQ_MSRXINT,
+			IRQ_MSTXINT
+		},
 	},
-};
-
-static struct sa1111_dev mse_dev = {
-	.dev = {
-		.name	= "Intel Corporation SA1111 [PS2]",
+	{
+		.offset		= 0x1800,
+		.skpcr_mask	= 0,
+		.devid		= SA1111_DEVID_PCMCIA,
+		.irq = {
+			IRQ_S0_READY_NINT,
+			IRQ_S0_CD_VALID,
+			IRQ_S0_BVD1_STSCHG,
+			IRQ_S1_READY_NINT,
+			IRQ_S1_CD_VALID,
+			IRQ_S1_BVD1_STSCHG,
+		},
 	},
-	.skpcr_mask	= SKPCR_PMCLKEN,
-	.devid		= SA1111_DEVID_PS2,
-	.irq = {
-		IRQ_MSRXINT,
-		IRQ_MSTXINT
-	},
-};
-
-static struct sa1111_dev int_dev = {
-	.dev = {
-		.name	= "Intel Corporation SA1111 [Interrupt Controller]",
-	},
-	.skpcr_mask	= 0,
-	.devid		= SA1111_DEVID_INT,
-};
-
-static struct sa1111_dev pcmcia_dev = {
-	.dev = {
-		.name	= "Intel Corporation SA1111 [PCMCIA Controller]",
-	},
-	.skpcr_mask	= 0,
-	.devid		= SA1111_DEVID_PCMCIA,
-	.irq = {
-		IRQ_S0_READY_NINT,
-		IRQ_S0_CD_VALID,
-		IRQ_S0_BVD1_STSCHG,
-		IRQ_S1_READY_NINT,
-		IRQ_S1_CD_VALID,
-		IRQ_S1_BVD1_STSCHG,
-	},
-};
-
-static struct sa1111_dev *devs[] = {
-	&usb_dev,
-	&sac_dev,
-	&ssp_dev,
-	&kbd_dev,
-	&mse_dev,
-	&pcmcia_dev,
-};
-
-static unsigned int dev_offset[] = {
-	SA1111_USB,
-	0x0600,
-	0x0800,
-	SA1111_KBD,
-	SA1111_MSE,
-	0x1800,
 };
 
 /*
@@ -168,14 +138,14 @@ sa1111_irq_handler(unsigned int irq, struct irqdesc *desc, struct pt_regs *regs)
 {
 	unsigned int stat0, stat1, i;
 
-	stat0 = INTSTATCLR0;
-	stat1 = INTSTATCLR1;
+	stat0 = sa1111_readl(desc->data + SA1111_INTSTATCLR0);
+	stat1 = sa1111_readl(desc->data + SA1111_INTSTATCLR1);
 
-	INTSTATCLR0 = stat0;
+	sa1111_writel(stat0, desc->data + SA1111_INTSTATCLR0);
 
 	desc->chip->ack(irq);
 
-	INTSTATCLR1 = stat1;
+	sa1111_writel(stat1, desc->data + SA1111_INTSTATCLR1);
 
 	if (stat0 == 0 && stat1 == 0) {
 		do_bad_IRQ(irq, desc, regs);
@@ -203,12 +173,22 @@ static void sa1111_ack_irq(unsigned int irq)
 
 static void sa1111_mask_lowirq(unsigned int irq)
 {
-	INTEN0 &= ~SA1111_IRQMASK_LO(irq);
+	void *mapbase = get_irq_chipdata(irq);
+	unsigned long ie0;
+
+	ie0 = sa1111_readl(mapbase + SA1111_INTEN0);
+	ie0 &= ~SA1111_IRQMASK_LO(irq);
+	writel(ie0, mapbase + SA1111_INTEN0);
 }
 
 static void sa1111_unmask_lowirq(unsigned int irq)
 {
-	INTEN0 |= SA1111_IRQMASK_LO(irq);
+	void *mapbase = get_irq_chipdata(irq);
+	unsigned long ie0;
+
+	ie0 = sa1111_readl(mapbase + SA1111_INTEN0);
+	ie0 |= SA1111_IRQMASK_LO(irq);
+	sa1111_writel(ie0, mapbase + SA1111_INTEN0);
 }
 
 /*
@@ -221,12 +201,15 @@ static void sa1111_unmask_lowirq(unsigned int irq)
 static int sa1111_retrigger_lowirq(unsigned int irq)
 {
 	unsigned int mask = SA1111_IRQMASK_LO(irq);
+	void *mapbase = get_irq_chipdata(irq);
+	unsigned long ip0;
 	int i;
 
+	ip0 = sa1111_readl(mapbase + SA1111_INTPOL0);
 	for (i = 0; i < 8; i++) {
-		INTPOL0 ^= mask;
-		INTPOL0 ^= mask;
-		if (INTSTATCLR1 & mask)
+		sa1111_writel(ip0 ^ mask, mapbase + SA1111_INTPOL0);
+		sa1111_writel(ip0, mapbase + SA1111_INTPOL0);
+		if (sa1111_readl(mapbase + SA1111_INTSTATCLR1) & mask)
 			break;
 	}
 
@@ -239,6 +222,8 @@ static int sa1111_retrigger_lowirq(unsigned int irq)
 static int sa1111_type_lowirq(unsigned int irq, unsigned int flags)
 {
 	unsigned int mask = SA1111_IRQMASK_LO(irq);
+	void *mapbase = get_irq_chipdata(irq);
+	unsigned long ip0;
 
 	if (flags == IRQT_PROBE)
 		return 0;
@@ -246,11 +231,13 @@ static int sa1111_type_lowirq(unsigned int irq, unsigned int flags)
 	if ((!(flags & __IRQT_RISEDGE) ^ !(flags & __IRQT_FALEDGE)) == 0)
 		return -EINVAL;
 
+	ip0 = sa1111_readl(mapbase + SA1111_INTPOL0);
 	if (flags & __IRQT_RISEDGE)
-		INTPOL0 &= ~mask;
+		ip0 &= ~mask;
 	else
-		INTPOL0 |= mask;
-	WAKE_POL0 = INTPOL0;
+		ip0 |= mask;
+	sa1111_writel(ip0, mapbase + SA1111_INTPOL0);
+	sa1111_writel(ip0, mapbase + SA1111_WAKEPOL0);
 
 	return 0;
 }
@@ -258,11 +245,15 @@ static int sa1111_type_lowirq(unsigned int irq, unsigned int flags)
 static int sa1111_wake_lowirq(unsigned int irq, unsigned int on)
 {
 	unsigned int mask = SA1111_IRQMASK_LO(irq);
+	void *mapbase = get_irq_chipdata(irq);
+	unsigned long we0;
 
+	we0 = sa1111_readl(mapbase + SA1111_WAKEEN0);
 	if (on)
-		WAKE_EN0 |= mask;
+		we0 |= mask;
 	else
-		WAKE_EN0 &= ~mask;
+		we0 &= ~mask;
+	sa1111_writel(we0, mapbase + SA1111_WAKEEN0);
 
 	return 0;
 }
@@ -278,12 +269,22 @@ static struct irqchip sa1111_low_chip = {
 
 static void sa1111_mask_highirq(unsigned int irq)
 {
-	INTEN1 &= ~SA1111_IRQMASK_HI(irq);
+	void *mapbase = get_irq_chipdata(irq);
+	unsigned long ie1;
+
+	ie1 = sa1111_readl(mapbase + SA1111_INTEN1);
+	ie1 &= ~SA1111_IRQMASK_HI(irq);
+	sa1111_writel(ie1, mapbase + SA1111_INTEN1);
 }
 
 static void sa1111_unmask_highirq(unsigned int irq)
 {
-	INTEN1 |= SA1111_IRQMASK_HI(irq);
+	void *mapbase = get_irq_chipdata(irq);
+	unsigned long ie1;
+
+	ie1 = sa1111_readl(mapbase + SA1111_INTEN1);
+	ie1 |= SA1111_IRQMASK_HI(irq);
+	sa1111_writel(ie1, mapbase + SA1111_INTEN1);
 }
 
 /*
@@ -296,12 +297,15 @@ static void sa1111_unmask_highirq(unsigned int irq)
 static int sa1111_retrigger_highirq(unsigned int irq)
 {
 	unsigned int mask = SA1111_IRQMASK_HI(irq);
+	void *mapbase = get_irq_chipdata(irq);
+	unsigned long ip1;
 	int i;
 
+	ip1 = sa1111_readl(mapbase + SA1111_INTPOL1);
 	for (i = 0; i < 8; i++) {
-		INTPOL1 ^= mask;
-		INTPOL1 ^= mask;
-		if (INTSTATCLR1 & mask)
+		sa1111_writel(ip1 ^ mask, mapbase + SA1111_INTPOL1);
+		sa1111_writel(ip1, mapbase + SA1111_INTPOL1);
+		if (sa1111_readl(mapbase + SA1111_INTSTATCLR1) & mask)
 			break;
 	}
 
@@ -314,6 +318,8 @@ static int sa1111_retrigger_highirq(unsigned int irq)
 static int sa1111_type_highirq(unsigned int irq, unsigned int flags)
 {
 	unsigned int mask = SA1111_IRQMASK_HI(irq);
+	void *mapbase = get_irq_chipdata(irq);
+	unsigned long ip1;
 
 	if (flags == IRQT_PROBE)
 		return 0;
@@ -321,11 +327,13 @@ static int sa1111_type_highirq(unsigned int irq, unsigned int flags)
 	if ((!(flags & __IRQT_RISEDGE) ^ !(flags & __IRQT_FALEDGE)) == 0)
 		return -EINVAL;
 
+	ip1 = sa1111_readl(mapbase + SA1111_INTPOL1);
 	if (flags & __IRQT_RISEDGE)
-		INTPOL1 &= ~mask;
+		ip1 &= ~mask;
 	else
-		INTPOL1 |= mask;
-	WAKE_POL1 = INTPOL1;
+		ip1 |= mask;
+	sa1111_writel(ip1, mapbase + SA1111_INTPOL1);
+	sa1111_writel(ip1, mapbase + SA1111_WAKEPOL1);
 
 	return 0;
 }
@@ -333,11 +341,15 @@ static int sa1111_type_highirq(unsigned int irq, unsigned int flags)
 static int sa1111_wake_highirq(unsigned int irq, unsigned int on)
 {
 	unsigned int mask = SA1111_IRQMASK_HI(irq);
+	void *mapbase = get_irq_chipdata(irq);
+	unsigned long we1;
 
+	we1 = sa1111_readl(mapbase + SA1111_WAKEEN1);
 	if (on)
-		WAKE_EN1 |= mask;
+		we1 |= mask;
 	else
-		WAKE_EN1 &= ~mask;
+		we1 &= ~mask;
+	sa1111_writel(we1, mapbase + SA1111_WAKEEN1);
 
 	return 0;
 }
@@ -351,42 +363,45 @@ static struct irqchip sa1111_high_chip = {
 	.wake		= sa1111_wake_highirq,
 };
 
-static void __init sa1111_init_irq(struct sa1111_dev *sadev)
+static void sa1111_setup_irq(struct sa1111 *sachip)
 {
+	void *irqbase = sachip->base + SA1111_INTC;
 	unsigned int irq;
 
 	/*
 	 * We're guaranteed that this region hasn't been taken.
 	 */
-	request_mem_region(sadev->res.start, 512, "irqs");
+	request_mem_region(sachip->phys + SA1111_INTC, 512, "irq");
 
 	/* disable all IRQs */
-	sa1111_writel(0, sadev->mapbase + SA1111_INTEN0);
-	sa1111_writel(0, sadev->mapbase + SA1111_INTEN1);
-	sa1111_writel(0, sadev->mapbase + SA1111_WAKEEN0);
-	sa1111_writel(0, sadev->mapbase + SA1111_WAKEEN1);
+	sa1111_writel(0, irqbase + SA1111_INTEN0);
+	sa1111_writel(0, irqbase + SA1111_INTEN1);
+	sa1111_writel(0, irqbase + SA1111_WAKEEN0);
+	sa1111_writel(0, irqbase + SA1111_WAKEEN1);
 
 	/*
 	 * detect on rising edge.  Note: Feb 2001 Errata for SA1111
 	 * specifies that S0ReadyInt and S1ReadyInt should be '1'.
 	 */
-	sa1111_writel(0, sadev->mapbase + SA1111_INTPOL0);
+	sa1111_writel(0, irqbase + SA1111_INTPOL0);
 	sa1111_writel(SA1111_IRQMASK_HI(IRQ_S0_READY_NINT) |
 		      SA1111_IRQMASK_HI(IRQ_S1_READY_NINT),
-		      sadev->mapbase + SA1111_INTPOL1);
+		      irqbase + SA1111_INTPOL1);
 
 	/* clear all IRQs */
-	sa1111_writel(~0, sadev->mapbase + SA1111_INTSTATCLR0);
-	sa1111_writel(~0, sadev->mapbase + SA1111_INTSTATCLR1);
+	sa1111_writel(~0, irqbase + SA1111_INTSTATCLR0);
+	sa1111_writel(~0, irqbase + SA1111_INTSTATCLR1);
 
 	for (irq = IRQ_GPAIN0; irq <= SSPROR; irq++) {
 		set_irq_chip(irq, &sa1111_low_chip);
+		set_irq_chipdata(irq, irqbase);
 		set_irq_handler(irq, do_edge_IRQ);
 		set_irq_flags(irq, IRQF_VALID | IRQF_PROBE);
 	}
 
 	for (irq = AUDXMTDMADONEA; irq <= IRQ_S1_BVD1_STSCHG; irq++) {
 		set_irq_chip(irq, &sa1111_high_chip);
+		set_irq_chipdata(irq, irqbase);
 		set_irq_handler(irq, do_edge_IRQ);
 		set_irq_flags(irq, IRQF_VALID | IRQF_PROBE);
 	}
@@ -394,8 +409,9 @@ static void __init sa1111_init_irq(struct sa1111_dev *sadev)
 	/*
 	 * Register SA1111 interrupt
 	 */
-	set_irq_type(sadev->irq[0], IRQT_RISING);
-	set_irq_chained_handler(sadev->irq[0], sa1111_irq_handler);
+	set_irq_type(sachip->irq, IRQT_RISING);
+	set_irq_data(sachip->irq, irqbase);
+	set_irq_chained_handler(sachip->irq, sa1111_irq_handler);
 }
 
 /*
@@ -418,7 +434,7 @@ static void sa1111_wake(struct sa1111 *sachip)
 
 	spin_lock_irqsave(&sachip->lock, flags);
 
-#if CONFIG_ARCH_SA1100
+#ifdef CONFIG_ARCH_SA1100
 	/*
 	 * First, set up the 3.6864MHz clock on GPIO 27 for the SA-1111:
 	 * (SA-1110 Developer's Manual, section 9.1.2.1)
@@ -469,6 +485,17 @@ static void sa1111_wake(struct sa1111 *sachip)
 
 #ifdef CONFIG_ARCH_SA1100
 
+static u32 sa1111_dma_mask[] = {
+	~0,
+	~(1 << 20),
+	~(1 << 23),
+	~(1 << 24),
+	~(1 << 25),
+	~(1 << 20),
+	~(1 << 20),
+	0,
+};
+
 /*
  * Configure the SA1111 shared memory controller.
  */
@@ -482,31 +509,93 @@ sa1111_configure_smc(struct sa1111 *sachip, int sdram, unsigned int drac,
 		smcr |= SMCR_CLAT;
 
 	sa1111_writel(smcr, sachip->base + SA1111_SMCR);
+
+	/*
+	 * Now clear the bits in the DMA mask to work around the SA1111
+	 * DMA erratum (Intel StrongARM SA-1111 Microprocessor Companion
+	 * Chip Specification Update, June 2000, Erratum #7).
+	 */
+	if (sachip->dev->dma_mask)
+		*sachip->dev->dma_mask &= sa1111_dma_mask[drac >> 2];
+
+	sachip->dev->coherent_dma_mask &= sa1111_dma_mask[drac >> 2];
 }
 
 #endif
 
-static void
-sa1111_init_one_child(struct sa1111 *sachip, struct sa1111_dev *sadev, unsigned int offset)
+static void sa1111_dev_release(struct device *_dev)
 {
-	snprintf(sadev->dev.bus_id, sizeof(sadev->dev.bus_id),
-		 "%4.4x", offset);
+	struct sa1111_dev *dev = SA1111_DEV(_dev);
 
-	sadev->dev.parent = sachip->dev;
-	sadev->dev.bus    = &sa1111_bus_type;
-	sadev->res.start  = sachip->res.start + offset;
-	sadev->res.end    = sadev->res.start + 511;
-	sadev->res.name   = sadev->dev.name;
-	sadev->res.flags  = IORESOURCE_MEM;
-	sadev->mapbase    = sachip->base + offset;
+	release_resource(&dev->res);
+	kfree(dev);
+}
 
-	if (request_resource(&sachip->res, &sadev->res)) {
+static int
+sa1111_init_one_child(struct sa1111 *sachip, struct resource *parent,
+		      struct sa1111_dev_info *info)
+{
+	struct sa1111_dev *dev;
+	int ret;
+
+	dev = kmalloc(sizeof(struct sa1111_dev), GFP_KERNEL);
+	if (!dev) {
+		ret = -ENOMEM;
+		goto out;
+	}
+	memset(dev, 0, sizeof(struct sa1111_dev));
+
+	snprintf(dev->dev.bus_id, sizeof(dev->dev.bus_id),
+		 "%4.4lx", info->offset);
+
+	dev->devid	 = info->devid;
+	dev->dev.parent  = sachip->dev;
+	dev->dev.bus     = &sa1111_bus_type;
+	dev->dev.release = sa1111_dev_release;
+	dev->dev.coherent_dma_mask = sachip->dev->coherent_dma_mask;
+	dev->res.start   = sachip->phys + info->offset;
+	dev->res.end     = dev->res.start + 511;
+	dev->res.name    = dev->dev.bus_id;
+	dev->res.flags   = IORESOURCE_MEM;
+	dev->mapbase     = sachip->base + info->offset;
+	dev->skpcr_mask  = info->skpcr_mask;
+	memmove(dev->irq, info->irq, sizeof(dev->irq));
+
+	ret = request_resource(parent, &dev->res);
+	if (ret) {
 		printk("SA1111: failed to allocate resource for %s\n",
-			sadev->res.name);
-		return;
+			dev->res.name);
+		kfree(dev);
+		goto out;
 	}
 
-	device_register(&sadev->dev);
+
+	ret = device_register(&dev->dev);
+	if (ret) {
+		release_resource(&dev->res);
+		kfree(dev);
+		goto out;
+	}
+
+	/*
+	 * If the parent device has a DMA mask associated with it,
+	 * propagate it down to the children.
+	 */
+	if (sachip->dev->dma_mask) {
+		dev->dma_mask = *sachip->dev->dma_mask;
+		dev->dev.dma_mask = &dev->dma_mask;
+
+		if (dev->dma_mask != 0xffffffffUL) {
+			ret = dmabounce_register_dev(&dev->dev, 1024, 4096);
+			if (ret) {
+				printk("SA1111: Failed to register %s with dmabounce", dev->dev.bus_id);
+				device_unregister(&dev->dev);
+			}
+		}
+	}
+
+out:
+	return ret;
 }
 
 /**
@@ -522,7 +611,7 @@ sa1111_init_one_child(struct sa1111 *sachip, struct sa1111_dev *sadev, unsigned
  *	%0		successful.
  */
 static int __init
-__sa1111_probe(struct device *me, unsigned long phys_addr, int irq)
+__sa1111_probe(struct device *me, struct resource *mem, int irq)
 {
 	struct sa1111 *sachip;
 	unsigned long id;
@@ -540,24 +629,17 @@ __sa1111_probe(struct device *me, unsigned long phys_addr, int irq)
 	sachip->dev = me;
 	dev_set_drvdata(sachip->dev, sachip);
 
-	sachip->res.name  = me->name;
-	sachip->res.start = phys_addr;
-	sachip->res.end   = phys_addr + 0x2000;
+	sachip->phys = mem->start;
 	sachip->irq = irq;
 
-	if (request_resource(&iomem_resource, &sachip->res)) {
-		ret = -EBUSY;
-		goto out;
-	}
-
 	/*
 	 * Map the whole region.  This also maps the
 	 * registers for our children.
 	 */
-	sachip->base = ioremap(phys_addr, PAGE_SIZE * 2);
+	sachip->base = ioremap(mem->start, PAGE_SIZE * 2);
 	if (!sachip->base) {
 		ret = -ENOMEM;
-		goto release;
+		goto out;
 	}
 
 	/*
@@ -609,9 +691,8 @@ __sa1111_probe(struct device *me, unsigned long phys_addr, int irq)
 	 * The interrupt controller must be initialised before any
 	 * other device to ensure that the interrupts are available.
 	 */
-	int_dev.irq[0] = irq;
-	sa1111_init_one_child(sachip, &int_dev, SA1111_INTC);
-	sa1111_init_irq(&int_dev);
+	if (sachip->irq != NO_IRQ)
+		sa1111_setup_irq(sachip);
 
 	g_sa1111 = sachip;
 
@@ -622,16 +703,14 @@ __sa1111_probe(struct device *me, unsigned long phys_addr, int irq)
 	else
 		has_devs &= ~(1 << 1);
 
-	for (i = 0; i < ARRAY_SIZE(devs); i++)
+	for (i = 0; i < ARRAY_SIZE(sa1111_devices); i++)
 		if (has_devs & (1 << i))
-			sa1111_init_one_child(sachip, devs[i], dev_offset[i]);
+			sa1111_init_one_child(sachip, mem, &sa1111_devices[i]);
 
 	return 0;
 
  unmap:
 	iounmap(sachip->base);
- release:
-	release_resource(&sachip->res);
  out:
 	kfree(sachip);
 	return ret;
@@ -639,15 +718,29 @@ __sa1111_probe(struct device *me, unsigned long phys_addr, int irq)
 
 static void __sa1111_remove(struct sa1111 *sachip)
 {
-	int i;
+	struct list_head *l, *n;
+	void *irqbase = sachip->base + SA1111_INTC;
 
-	for (i = 0; i < ARRAY_SIZE(devs); i++) {
-		put_device(&devs[i]->dev);
-		release_resource(&devs[i]->res);
+	list_for_each_safe(l, n, &sachip->dev->children) {
+		struct device *d = list_to_dev(l);
+
+		device_unregister(d);
+	}
+
+	/* disable all IRQs */
+	sa1111_writel(0, irqbase + SA1111_INTEN0);
+	sa1111_writel(0, irqbase + SA1111_INTEN1);
+	sa1111_writel(0, irqbase + SA1111_WAKEEN0);
+	sa1111_writel(0, irqbase + SA1111_WAKEEN1);
+
+	if (sachip->irq != NO_IRQ) {
+		set_irq_chained_handler(sachip->irq, NULL);
+		set_irq_data(sachip->irq, NULL);
+
+		release_mem_region(sachip->phys + SA1111_INTC, 512);
 	}
 
 	iounmap(sachip->base);
-	release_resource(&sachip->res);
 	kfree(sachip);
 }
 
@@ -662,61 +755,30 @@ static void __sa1111_remove(struct sa1111 *sachip)
  *
  * This routine only identifies whether or not a given DMA address
  * is susceptible to the bug.
+ *
+ * This should only get called for sa1111_device types due to the
+ * way we configure our device dma_masks.
  */
-int sa1111_check_dma_bug(dma_addr_t addr)
+int dma_needs_bounce(struct device *dev, dma_addr_t addr, size_t size)
 {
-	struct sa1111 *sachip = g_sa1111;
 	unsigned int physaddr = SA1111_DMA_ADDR((unsigned int)addr);
-	unsigned int smcr;
+	u32 dma_mask = *dev->dma_mask;
 
-	/* Section 4.6 of the "Intel StrongARM SA-1111 Development Module
+	/*
+	 * Section 4.6 of the "Intel StrongARM SA-1111 Development Module
 	 * User's Guide" mentions that jumpers R51 and R52 control the
 	 * target of SA-1111 DMA (either SDRAM bank 0 on Assabet, or
 	 * SDRAM bank 1 on Neponset). The default configuration selects
 	 * Assabet, so any address in bank 1 is necessarily invalid.
 	 */
-	if ((machine_is_assabet() || machine_is_pfs168()) && addr >= 0xc8000000)
-	  	return -1;
+	if ((machine_is_assabet() || machine_is_pfs168()) &&
+		(addr >= 0xc8000000 || (addr + size) >= 0xc8000000))
+	  	return 1;
 
-	/* The bug only applies to buffers located more than one megabyte
-	 * above the start of the target bank:
+	/*
+	 * Check to see if either the start or end are illegal.
 	 */
-	if (physaddr<(1<<20))
-		return 0;
-
-	smcr = sa1111_readl(sachip->base + SA1111_SMCR);
-	switch (FExtr(smcr, SMCR_DRAC)) {
-	case 01: /* 10 row + bank address bits, A<20> must not be set */
-	  	if (physaddr & (1<<20))
-		  	return -1;
-		break;
-	case 02: /* 11 row + bank address bits, A<23> must not be set */
-	  	if (physaddr & (1<<23))
-		  	return -1;
-		break;
-	case 03: /* 12 row + bank address bits, A<24> must not be set */
-	  	if (physaddr & (1<<24))
-		  	return -1;
-		break;
-	case 04: /* 13 row + bank address bits, A<25> must not be set */
-	  	if (physaddr & (1<<25))
-		  	return -1;
-		break;
-	case 05: /* 14 row + bank address bits, A<20> must not be set */
-	  	if (physaddr & (1<<20))
-		  	return -1;
-		break;
-	case 06: /* 15 row + bank address bits, A<20> must not be set */
-	  	if (physaddr & (1<<20))
-		  	return -1;
-		break;
-	default:
-	  	printk(KERN_ERR "%s(): invalid SMCR DRAC value 0%lo\n",
-		       __FUNCTION__, FExtr(smcr, SMCR_DRAC));
-		return -1;
-	}
-
-	return 0;
+	return ((addr & ~dma_mask)) || ((addr + size - 1) & ~dma_mask);
 }
 
 struct sa1111_save_data {
@@ -743,61 +805,52 @@ struct sa1111_save_data {
 static int sa1111_suspend(struct device *dev, u32 state, u32 level)
 {
 	struct sa1111 *sachip = dev_get_drvdata(dev);
+	struct sa1111_save_data *save;
 	unsigned long flags;
+	unsigned int val;
 	char *base;
 
+	if (level != SUSPEND_DISABLE)
+		return 0;
+
+	dev->saved_state = kmalloc(sizeof(struct sa1111_save_data), GFP_KERNEL);
+	if (!dev->saved_state)
+		return -ENOMEM;
+
+	save = (struct sa1111_save_data *)dev->saved_state;
+
+	spin_lock_irqsave(&sachip->lock, flags);
+
 	/*
 	 * Save state.
 	 */
-	if (level == SUSPEND_SAVE_STATE ||
-	    level == SUSPEND_DISABLE ||
-	    level == SUSPEND_POWER_DOWN) {
-		struct sa1111_save_data *save;
+	base = sachip->base;
+	save->skcr     = sa1111_readl(base + SA1111_SKCR);
+	save->skpcr    = sa1111_readl(base + SA1111_SKPCR);
+	save->skcdr    = sa1111_readl(base + SA1111_SKCDR);
+	save->skaud    = sa1111_readl(base + SA1111_SKAUD);
+	save->skpwm0   = sa1111_readl(base + SA1111_SKPWM0);
+	save->skpwm1   = sa1111_readl(base + SA1111_SKPWM1);
 
-		if (!dev->saved_state)
-			dev->saved_state = kmalloc(sizeof(struct sa1111_save_data), GFP_KERNEL);
-		if (!dev->saved_state)
-			return -ENOMEM;
-
-		save = (struct sa1111_save_data *)dev->saved_state;
-
-		spin_lock_irqsave(&sachip->lock, flags);
-		base = sachip->base;
-		save->skcr     = sa1111_readl(base + SA1111_SKCR);
-		save->skpcr    = sa1111_readl(base + SA1111_SKPCR);
-		save->skcdr    = sa1111_readl(base + SA1111_SKCDR);
-		save->skaud    = sa1111_readl(base + SA1111_SKAUD);
-		save->skpwm0   = sa1111_readl(base + SA1111_SKPWM0);
-		save->skpwm1   = sa1111_readl(base + SA1111_SKPWM1);
-
-		base = sachip->base + SA1111_INTC;
-		save->intpol0  = sa1111_readl(base + SA1111_INTPOL0);
-		save->intpol1  = sa1111_readl(base + SA1111_INTPOL1);
-		save->inten0   = sa1111_readl(base + SA1111_INTEN0);
-		save->inten1   = sa1111_readl(base + SA1111_INTEN1);
-		save->wakepol0 = sa1111_readl(base + SA1111_WAKEPOL0);
-		save->wakepol1 = sa1111_readl(base + SA1111_WAKEPOL1);
-		save->wakeen0  = sa1111_readl(base + SA1111_WAKEEN0);
-		save->wakeen1  = sa1111_readl(base + SA1111_WAKEEN1);
-		spin_unlock_irqrestore(&sachip->lock, flags);
-	}
+	base = sachip->base + SA1111_INTC;
+	save->intpol0  = sa1111_readl(base + SA1111_INTPOL0);
+	save->intpol1  = sa1111_readl(base + SA1111_INTPOL1);
+	save->inten0   = sa1111_readl(base + SA1111_INTEN0);
+	save->inten1   = sa1111_readl(base + SA1111_INTEN1);
+	save->wakepol0 = sa1111_readl(base + SA1111_WAKEPOL0);
+	save->wakepol1 = sa1111_readl(base + SA1111_WAKEPOL1);
+	save->wakeen0  = sa1111_readl(base + SA1111_WAKEEN0);
+	save->wakeen1  = sa1111_readl(base + SA1111_WAKEEN1);
 
 	/*
 	 * Disable.
 	 */
-	if (level == SUSPEND_DISABLE && state == 4) {
-		unsigned int val;
+	val = sa1111_readl(sachip->base + SA1111_SKCR);
+	sa1111_writel(val | SKCR_SLEEP, sachip->base + SA1111_SKCR);
+	sa1111_writel(0, sachip->base + SA1111_SKPWM0);
+	sa1111_writel(0, sachip->base + SA1111_SKPWM1);
 
-		spin_lock_irqsave(&sachip->lock, flags);
-		base = sachip->base;
-
-		sa1111_writel(0, base + SA1111_SKPWM0);
-		sa1111_writel(0, base + SA1111_SKPWM1);
-		val = sa1111_readl(base + SA1111_SKCR);
-		sa1111_writel(val | SKCR_SLEEP, base + SA1111_SKCR);
-
-		spin_unlock_irqrestore(&sachip->lock, flags);
-	}
+	spin_unlock_irqrestore(&sachip->lock, flags);
 
 	return 0;
 }
@@ -819,17 +872,18 @@ static int sa1111_resume(struct device *dev, u32 level)
 	unsigned long flags, id;
 	char *base;
 
-	if (level != RESUME_RESTORE_STATE && level != RESUME_ENABLE)
+	if (level != RESUME_ENABLE)
 		return 0;
 
 	save = (struct sa1111_save_data *)dev->saved_state;
 	if (!save)
 		return 0;
 
-	dev->saved_state = NULL;
+	spin_lock_irqsave(&sachip->lock, flags);
 
 	/*
 	 * Ensure that the SA1111 is still here.
+	 * FIXME: shouldn't do this here.
 	 */
 	id = sa1111_readl(sachip->base + SA1111_SKID);
 	if ((id & SKID_ID_MASK) != SKID_SA1111_ID) {
@@ -839,8 +893,12 @@ static int sa1111_resume(struct device *dev, u32 level)
 		return 0;
 	}
 
-	spin_lock_irqsave(&sachip->lock, flags);
+	/*
+	 * First of all, wake up the chip.
+	 */
 	sa1111_wake(sachip);
+	sa1111_writel(0, sachip->base + SA1111_INTC + SA1111_INTEN0);
+	sa1111_writel(0, sachip->base + SA1111_INTC + SA1111_INTEN1);
 
 	base = sachip->base;
 	sa1111_writel(save->skcr,     base + SA1111_SKCR);
@@ -859,8 +917,10 @@ static int sa1111_resume(struct device *dev, u32 level)
 	sa1111_writel(save->wakepol1, base + SA1111_WAKEPOL1);
 	sa1111_writel(save->wakeen0,  base + SA1111_WAKEEN0);
 	sa1111_writel(save->wakeen1,  base + SA1111_WAKEEN1);
+
 	spin_unlock_irqrestore(&sachip->lock, flags);
 
+	dev->saved_state = NULL;
 	kfree(save);
 
 	return 0;
@@ -868,7 +928,17 @@ static int sa1111_resume(struct device *dev, u32 level)
 
 static int sa1111_probe(struct device *dev)
 {
-	return -ENODEV;
+	struct platform_device *pdev = to_platform_device(dev);
+	struct resource *mem = NULL, *irq = NULL;
+	int i;
+
+	for (i = 0; i < pdev->num_resources; i++) {
+		if (pdev->resource[i].flags & IORESOURCE_MEM)
+			mem = &pdev->resource[i];
+		if (pdev->resource[i].flags & IORESOURCE_IRQ)
+			irq = &pdev->resource[i];
+	}
+	return __sa1111_probe(dev, mem, irq ? irq->start : NO_IRQ);
 }
 
 static int sa1111_remove(struct device *dev)
@@ -897,47 +967,13 @@ static int sa1111_remove(struct device *dev)
  */
 static struct device_driver sa1111_device_driver = {
 	.name		= "sa1111",
-	.bus		= &system_bus_type,
+	.bus		= &platform_bus_type,
 	.probe		= sa1111_probe,
 	.remove		= sa1111_remove,
 	.suspend	= sa1111_suspend,
 	.resume		= sa1111_resume,
 };
 
-/*
- *	Register the SA1111 driver with LDM.
- */
-static int sa1111_driver_init(void)
-{
-	driver_register(&sa1111_device_driver);
-	return 0;
-}
-
-arch_initcall(sa1111_driver_init);
-
-static struct sys_device sa1111_device = {
-	.name		= "SA1111",
-	.id		= 0,
-	.root		= NULL,
-	.dev = {
-		.name	= "Intel Corporation SA1111",
-		.driver	= &sa1111_device_driver,
-	},
-};
-
-int sa1111_init(unsigned long phys, unsigned int irq)
-{
-	int ret;
-
-	snprintf(sa1111_device.dev.bus_id, sizeof(sa1111_device.dev.bus_id), "%8.8lx", phys);
-
-	ret = sys_device_register(&sa1111_device);
-	if (ret)
-		printk("sa1111 device_register failed: %d\n", ret);
-
-	return __sa1111_probe(&sa1111_device.dev, phys, irq);
-}
-
 /*
  *	Get the parent device driver (us) structure
  *	from a child function device
@@ -1049,6 +1085,62 @@ int sa1111_get_audio_rate(struct sa1111_dev *sadev)
 	return __sa1111_pll_clock(sachip) / (256 * div);
 }
 
+void sa1111_set_io_dir(struct sa1111_dev *sadev,
+		       unsigned int bits, unsigned int dir,
+		       unsigned int sleep_dir)
+{
+	struct sa1111 *sachip = sa1111_chip_driver(sadev);
+	unsigned long flags;
+	unsigned int val;
+	void *gpio = sachip->base + SA1111_GPIO;
+
+#define MODIFY_BITS(port, mask, dir)		\
+	if (mask) {				\
+		val = sa1111_readl(port);	\
+		val &= ~(mask);			\
+		val |= (dir) & (mask);		\
+		sa1111_writel(val, port);	\
+	}
+
+	spin_lock_irqsave(&sachip->lock, flags);
+	MODIFY_BITS(gpio + SA1111_GPIO_PADDR, bits & 15, dir);
+	MODIFY_BITS(gpio + SA1111_GPIO_PBDDR, (bits >> 8) & 255, dir >> 8);
+	MODIFY_BITS(gpio + SA1111_GPIO_PCDDR, (bits >> 16) & 255, dir >> 16);
+
+	MODIFY_BITS(gpio + SA1111_GPIO_PASDR, bits & 15, sleep_dir);
+	MODIFY_BITS(gpio + SA1111_GPIO_PBSDR, (bits >> 8) & 255, sleep_dir >> 8);
+	MODIFY_BITS(gpio + SA1111_GPIO_PCSDR, (bits >> 16) & 255, sleep_dir >> 16);
+	spin_unlock_irqrestore(&sachip->lock, flags);
+}
+
+void sa1111_set_io(struct sa1111_dev *sadev, unsigned int bits, unsigned int v)
+{
+	struct sa1111 *sachip = sa1111_chip_driver(sadev);
+	unsigned long flags;
+	unsigned int val;
+	void *gpio = sachip->base + SA1111_GPIO;
+
+	spin_lock_irqsave(&sachip->lock, flags);
+	MODIFY_BITS(gpio + SA1111_GPIO_PADWR, bits & 15, v);
+	MODIFY_BITS(gpio + SA1111_GPIO_PBDWR, (bits >> 8) & 255, v >> 8);
+	MODIFY_BITS(gpio + SA1111_GPIO_PCDWR, (bits >> 16) & 255, v >> 16);
+	spin_unlock_irqrestore(&sachip->lock, flags);
+}
+
+void sa1111_set_sleep_io(struct sa1111_dev *sadev, unsigned int bits, unsigned int v)
+{
+	struct sa1111 *sachip = sa1111_chip_driver(sadev);
+	unsigned long flags;
+	unsigned int val;
+	void *gpio = sachip->base + SA1111_GPIO;
+
+	spin_lock_irqsave(&sachip->lock, flags);
+	MODIFY_BITS(gpio + SA1111_GPIO_PASSR, bits & 15, v);
+	MODIFY_BITS(gpio + SA1111_GPIO_PBSSR, (bits >> 8) & 255, v >> 8);
+	MODIFY_BITS(gpio + SA1111_GPIO_PCSSR, (bits >> 16) & 255, v >> 16);
+	spin_unlock_irqrestore(&sachip->lock, flags);
+}
+
 /*
  * Individual device operations.
  */
@@ -1099,23 +1191,99 @@ static int sa1111_match(struct device *_dev, struct device_driver *_drv)
 	return dev->devid == drv->devid;
 }
 
-struct bus_type sa1111_bus_type = {
-	.name	= "RAB",
-	.match	= sa1111_match,
-};
-
-static int sa1111_rab_bus_init(void)
+static int sa1111_bus_suspend(struct device *dev, u32 state)
 {
-	return bus_register(&sa1111_bus_type);
+	struct sa1111_dev *sadev = SA1111_DEV(dev);
+	struct sa1111_driver *drv = SA1111_DRV(dev->driver);
+	int ret = 0;
+
+	if (drv && drv->suspend)
+		ret = drv->suspend(sadev, state);
+	return ret;
 }
 
-postcore_initcall(sa1111_rab_bus_init);
+static int sa1111_bus_resume(struct device *dev)
+{
+	struct sa1111_dev *sadev = SA1111_DEV(dev);
+	struct sa1111_driver *drv = SA1111_DRV(dev->driver);
+	int ret = 0;
+
+	if (drv && drv->resume)
+		ret = drv->resume(sadev);
+	return ret;
+}
+
+static int sa1111_bus_probe(struct device *dev)
+{
+	struct sa1111_dev *sadev = SA1111_DEV(dev);
+	struct sa1111_driver *drv = SA1111_DRV(dev->driver);
+	int ret = -ENODEV;
+
+	if (drv->probe)
+		ret = drv->probe(sadev);
+	return ret;
+}
+
+static int sa1111_bus_remove(struct device *dev)
+{
+	struct sa1111_dev *sadev = SA1111_DEV(dev);
+	struct sa1111_driver *drv = SA1111_DRV(dev->driver);
+	int ret = 0;
+
+	if (drv->remove)
+		ret = drv->remove(sadev);
+	return ret;
+}
+
+struct bus_type sa1111_bus_type = {
+	.name		= "sa1111-rab",
+	.match		= sa1111_match,
+	.suspend	= sa1111_bus_suspend,
+	.resume		= sa1111_bus_resume,
+};
+
+int sa1111_driver_register(struct sa1111_driver *driver)
+{
+	driver->drv.probe = sa1111_bus_probe;
+	driver->drv.remove = sa1111_bus_remove;
+	driver->drv.bus = &sa1111_bus_type;
+	return driver_register(&driver->drv);
+}
+
+void sa1111_driver_unregister(struct sa1111_driver *driver)
+{
+	driver_unregister(&driver->drv);
+}
+
+static int __init sa1111_init(void)
+{
+	int ret = bus_register(&sa1111_bus_type);
+	if (ret == 0)
+		driver_register(&sa1111_device_driver);
+	return ret;
+}
+
+static void __exit sa1111_exit(void)
+{
+	driver_unregister(&sa1111_device_driver);
+	bus_unregister(&sa1111_bus_type);
+}
+
+module_init(sa1111_init);
+module_exit(sa1111_exit);
+
+MODULE_DESCRIPTION("Intel Corporation SA1111 core driver");
+MODULE_LICENSE("GPL");
 
-EXPORT_SYMBOL(sa1111_check_dma_bug);
 EXPORT_SYMBOL(sa1111_select_audio_mode);
 EXPORT_SYMBOL(sa1111_set_audio_rate);
 EXPORT_SYMBOL(sa1111_get_audio_rate);
+EXPORT_SYMBOL(sa1111_set_io_dir);
+EXPORT_SYMBOL(sa1111_set_io);
+EXPORT_SYMBOL(sa1111_set_sleep_io);
 EXPORT_SYMBOL(sa1111_enable_device);
 EXPORT_SYMBOL(sa1111_disable_device);
 EXPORT_SYMBOL(sa1111_pll_clock);
 EXPORT_SYMBOL(sa1111_bus_type);
+EXPORT_SYMBOL(sa1111_driver_register);
+EXPORT_SYMBOL(sa1111_driver_unregister);
