@@ -9,15 +9,15 @@
  *
  *  2001-09-13: Cliff Brake <cbrake@accelent.com>
  *              Initial code
+ *
+ * Expected command line: mem=32M initrd=0xa1000000,4M root=/dev/ram ramdisk=8192
  */
 #include <linux/kernel.h>
 #include <linux/init.h>
 #include <linux/major.h>
 #include <linux/fs.h>
 #include <linux/interrupt.h>
-#include <linux/sched.h>
 
-#include <asm/types.h>
 #include <asm/setup.h>
 #include <asm/memory.h>
 #include <asm/mach-types.h>
@@ -29,8 +29,6 @@
 
 #include "generic.h"
 
-#define PXA_IDP_REV02
-
 #ifndef PXA_IDP_REV02
 /* shadow registers for write only registers */
 unsigned int idp_cpld_led_control_shadow = 0x1;
@@ -56,14 +54,11 @@ unsigned int idp_control_port_shadow = ((0x7 << 21) | 		/* LCD power */
 
 #endif
 
-static int __init idp_init(void)
+static void __init idp_init(void)
 {
 	printk("idp_init()\n");
-	return 0;
 }
 
-__initcall(idp_init);
-
 static void __init idp_init_irq(void)
 {
 	pxa_init_irq();
@@ -103,12 +98,23 @@ static void __init idp_map_io(void)
 	pxa_map_io();
 	iotable_init(idp_io_desc, ARRAY_SIZE(idp_io_desc));
 
-	set_GPIO_IRQ_edge(IRQ_TO_GPIO_2_80(TOUCH_PANEL_IRQ), TOUCH_PANEL_IRQ_EDGE);
+	set_irq_type(IRQ_TO_GPIO_2_80(TOUCH_PANEL_IRQ), TOUCH_PANEL_IRQ_EDGE);
+
+	// serial ports 2 & 3
+	pxa_gpio_mode(GPIO42_BTRXD_MD);
+	pxa_gpio_mode(GPIO43_BTTXD_MD);
+	pxa_gpio_mode(GPIO44_BTCTS_MD);
+	pxa_gpio_mode(GPIO45_BTRTS_MD);
+	pxa_gpio_mode(GPIO46_STRXD_MD);
+	pxa_gpio_mode(GPIO47_STTXD_MD);
+
 }
 
+
 MACHINE_START(PXA_IDP, "Accelent Xscale IDP")
 	MAINTAINER("Accelent Systems Inc.")
-	BOOT_MEM(0xa0000000, 0x40000000, 0xfc000000)
+	BOOT_MEM(0xa0000000, 0x40000000, io_p2v(0x40000000))
 	MAPIO(idp_map_io)
 	INITIRQ(idp_init_irq)
+	INIT_MACHINE(idp_init)
 MACHINE_END
