@@ -13,10 +13,27 @@
 /* RED-PEN: this is accessed without any locking */
 extern unsigned int *usage_table;
 
-static int mtrr_seq_show(struct seq_file *seq, void *offset);
 
 #define FILE_FCOUNT(f) (((struct seq_file *)((f)->private_data))->private)
 
+static char *mtrr_strings[MTRR_NUM_TYPES] =
+{
+    "uncachable",               /* 0 */
+    "write-combining",          /* 1 */
+    "?",                        /* 2 */
+    "?",                        /* 3 */
+    "write-through",            /* 4 */
+    "write-protect",            /* 5 */
+    "write-back",               /* 6 */
+};
+
+char *mtrr_attrib_to_str(int x)
+{
+	return (x <= 6) ? mtrr_strings[x] : "?";
+}
+
+#ifdef CONFIG_PROC_FS
+
 static int
 mtrr_file_add(unsigned long base, unsigned long size,
 	      unsigned int type, char increment, struct file *file, int page)
@@ -275,6 +292,8 @@ mtrr_close(struct inode *ino, struct file *file)
 	return single_release(ino, file);
 }
 
+static int mtrr_seq_show(struct seq_file *seq, void *offset);
+
 static int mtrr_open(struct inode *inode, struct file *file)
 {
 	if (!mtrr_if) 
@@ -294,16 +313,9 @@ static struct file_operations mtrr_fops = {
 	.release = mtrr_close,
 };
 
-#  ifdef CONFIG_PROC_FS
 
 static struct proc_dir_entry *proc_root_mtrr;
 
-#  endif			/*  CONFIG_PROC_FS  */
-
-char * attrib_to_str(int x)
-{
-	return (x <= 6) ? mtrr_strings[x] : "?";
-}
 
 static int mtrr_seq_show(struct seq_file *seq, void *offset)
 {
@@ -332,7 +344,7 @@ static int mtrr_seq_show(struct seq_file *seq, void *offset)
 			len += seq_printf(seq, 
 				   "reg%02i: base=0x%05lx000 (%4liMB), size=%4i%cB: %s, count=%d\n",
 			     i, base, base >> (20 - PAGE_SHIFT), size, factor,
-			     attrib_to_str(type), usage_table[i]);
+			     mtrr_attrib_to_str(type), usage_table[i]);
 		}
 	}
 	return 0;
@@ -340,15 +352,22 @@ static int mtrr_seq_show(struct seq_file *seq, void *offset)
 
 static int __init mtrr_if_init(void)
 {
-#ifdef CONFIG_PROC_FS
+	struct cpuinfo_x86 *c = &boot_cpu_data;
+
+	if ((!cpu_has(c, X86_FEATURE_MTRR)) &&
+	    (!cpu_has(c, X86_FEATURE_K6_MTRR)) &&
+	    (!cpu_has(c, X86_FEATURE_CYRIX_ARR)) &&
+	    (!cpu_has(c, X86_FEATURE_CENTAUR_MCR)))
+		return -ENODEV;
+
 	proc_root_mtrr =
 	    create_proc_entry("mtrr", S_IWUSR | S_IRUGO, &proc_root);
 	if (proc_root_mtrr) {
 		proc_root_mtrr->owner = THIS_MODULE;
 		proc_root_mtrr->proc_fops = &mtrr_fops;
 	}
-#endif
 	return 0;
 }
 
 arch_initcall(mtrr_if_init);
+#endif			/*  CONFIG_PROC_FS  */
