@@ -10,7 +10,6 @@
 
 #include <linux/config.h>
 #include <linux/sched.h>
-#include <linux/spinlock.h>
 #include <asm/processor.h>
 #include <asm/i387.h>
 #include <asm/math_emu.h>
@@ -25,6 +24,22 @@
 #define HAVE_HWFP 1
 #endif
 
+unsigned long mxcsr_feature_mask = 0xffffffff;
+
+void mxcsr_feature_mask_init(void)
+{
+	unsigned long mask = 0;
+	clts();
+	if (cpu_has_fxsr) {
+		memset(&current->thread.i387.fxsave, 0, sizeof(struct i387_fxsave_struct));
+		asm volatile("fxsave %0" : : "m" (current->thread.i387.fxsave)); 
+		mask = current->thread.i387.fxsave.mxcsr_mask;
+		if (mask == 0) mask = 0x0000ffbf;
+	} 
+	mxcsr_feature_mask &= mask;
+	stts();
+}
+
 /*
  * The _current_ task is using the FPU for the first time
  * so initialize it and set the mxcsr to its default
@@ -40,10 +55,10 @@ void init_fpu(struct task_struct *tsk)
 			tsk->thread.i387.fxsave.mxcsr = 0x1f80;
 	} else {
 		memset(&tsk->thread.i387.fsave, 0, sizeof(struct i387_fsave_struct));
-		tsk->thread.i387.fsave.cwd = 0xffff037f;
-		tsk->thread.i387.fsave.swd = 0xffff0000;
-		tsk->thread.i387.fsave.twd = 0xffffffff;
-		tsk->thread.i387.fsave.fos = 0xffff0000;
+		tsk->thread.i387.fsave.cwd = 0xffff037fu;
+		tsk->thread.i387.fsave.swd = 0xffff0000u;
+		tsk->thread.i387.fsave.twd = 0xffffffffu;
+		tsk->thread.i387.fsave.fos = 0xffff0000u;
 	}
 	tsk->used_math = 1;
 }
@@ -52,29 +67,13 @@ void init_fpu(struct task_struct *tsk)
  * FPU lazy state save handling.
  */
 
-static inline void __save_init_fpu( struct task_struct *tsk )
-{
-	if ( cpu_has_fxsr ) {
-		asm volatile( "fxsave %0 ; fnclex"
-			      : "=m" (tsk->thread.i387.fxsave) );
-	} else {
-		asm volatile( "fnsave %0 ; fwait"
-			      : "=m" (tsk->thread.i387.fsave) );
-	}
-	clear_tsk_thread_flag(tsk, TIF_USEDFPU);
-}
-
-void save_init_fpu( struct task_struct *tsk )
-{
-	__save_init_fpu(tsk);
-	stts();
-}
-
 void kernel_fpu_begin(void)
 {
+	struct thread_info *thread = current_thread_info();
+
 	preempt_disable();
-	if (test_thread_flag(TIF_USEDFPU)) {
-		__save_init_fpu(current);
+	if (thread->status & TS_USEDFPU) {
+		__save_init_fpu(thread->task);
 		return;
 	}
 	clts();
@@ -114,7 +113,7 @@ static inline unsigned long twd_fxsr_to_i387( struct i387_fxsave_struct *fxsave
 	struct _fpxreg *st = NULL;
 	unsigned long twd = (unsigned long) fxsave->twd;
 	unsigned long tag;
-	unsigned long ret = 0xffff0000;
+	unsigned long ret = 0xffff0000u;
 	int i;
 
 #define FPREG_ADDR(f, n)	((char *)&(f)->st_space + (n) * 16);
@@ -199,7 +198,7 @@ void set_fpu_cwd( struct task_struct *tsk, unsigned short cwd )
 	if ( cpu_has_fxsr ) {
 		tsk->thread.i387.fxsave.cwd = cwd;
 	} else {
-		tsk->thread.i387.fsave.cwd = ((long)cwd | 0xffff0000);
+		tsk->thread.i387.fsave.cwd = ((long)cwd | 0xffff0000u);
 	}
 }
 
@@ -208,7 +207,7 @@ void set_fpu_swd( struct task_struct *tsk, unsigned short swd )
 	if ( cpu_has_fxsr ) {
 		tsk->thread.i387.fxsave.swd = swd;
 	} else {
-		tsk->thread.i387.fsave.swd = ((long)swd | 0xffff0000);
+		tsk->thread.i387.fsave.swd = ((long)swd | 0xffff0000u);
 	}
 }
 
@@ -217,14 +216,7 @@ void set_fpu_twd( struct task_struct *tsk, unsigned short twd )
 	if ( cpu_has_fxsr ) {
 		tsk->thread.i387.fxsave.twd = twd_i387_to_fxsr(twd);
 	} else {
-		tsk->thread.i387.fsave.twd = ((long)twd | 0xffff0000);
-	}
-}
-
-void set_fpu_mxcsr( struct task_struct *tsk, unsigned short mxcsr )
-{
-	if ( cpu_has_xmm ) {
-		tsk->thread.i387.fxsave.mxcsr = (mxcsr & 0xffbf);
+		tsk->thread.i387.fsave.twd = ((long)twd | 0xffff0000u);
 	}
 }
 
@@ -232,16 +224,16 @@ void set_fpu_mxcsr( struct task_struct *tsk, unsigned short mxcsr )
  * FXSR floating point environment conversions.
  */
 
-static inline int convert_fxsr_to_user( struct _fpstate *buf,
+static int convert_fxsr_to_user( struct _fpstate __user *buf,
 					struct i387_fxsave_struct *fxsave )
 {
 	unsigned long env[7];
-	struct _fpreg *to;
+	struct _fpreg __user *to;
 	struct _fpxreg *from;
 	int i;
 
-	env[0] = (unsigned long)fxsave->cwd | 0xffff0000;
-	env[1] = (unsigned long)fxsave->swd | 0xffff0000;
+	env[0] = (unsigned long)fxsave->cwd | 0xffff0000ul;
+	env[1] = (unsigned long)fxsave->swd | 0xffff0000ul;
 	env[2] = twd_fxsr_to_i387(fxsave);
 	env[3] = fxsave->fip;
 	env[4] = fxsave->fcs | ((unsigned long)fxsave->fop << 16);
@@ -254,18 +246,23 @@ static inline int convert_fxsr_to_user( struct _fpstate *buf,
 	to = &buf->_st[0];
 	from = (struct _fpxreg *) &fxsave->st_space[0];
 	for ( i = 0 ; i < 8 ; i++, to++, from++ ) {
-		if ( __copy_to_user( to, from, sizeof(*to) ) )
+		unsigned long *t = (unsigned long *)to;
+		unsigned long *f = (unsigned long *)from;
+
+		if (__put_user(*f, t) ||
+				__put_user(*(f + 1), t + 1) ||
+				__put_user(from->exponent, &to->exponent))
 			return 1;
 	}
 	return 0;
 }
 
-static inline int convert_fxsr_from_user( struct i387_fxsave_struct *fxsave,
-					  struct _fpstate *buf )
+static int convert_fxsr_from_user( struct i387_fxsave_struct *fxsave,
+					  struct _fpstate __user *buf )
 {
 	unsigned long env[7];
 	struct _fpxreg *to;
-	struct _fpreg *from;
+	struct _fpreg __user *from;
 	int i;
 
 	if ( __copy_from_user( env, buf, 7 * sizeof(long) ) )
@@ -275,7 +272,7 @@ static inline int convert_fxsr_from_user( struct i387_fxsave_struct *fxsave,
 	fxsave->swd = (unsigned short)(env[1] & 0xffff);
 	fxsave->twd = twd_i387_to_fxsr((unsigned short)(env[2] & 0xffff));
 	fxsave->fip = env[3];
-	fxsave->fop = (unsigned short)((env[4] & 0xffff0000) >> 16);
+	fxsave->fop = (unsigned short)((env[4] & 0xffff0000ul) >> 16);
 	fxsave->fcs = (env[4] & 0xffff);
 	fxsave->foo = env[5];
 	fxsave->fos = env[6];
@@ -283,7 +280,12 @@ static inline int convert_fxsr_from_user( struct i387_fxsave_struct *fxsave,
 	to = (struct _fpxreg *) &fxsave->st_space[0];
 	from = &buf->_st[0];
 	for ( i = 0 ; i < 8 ; i++, to++, from++ ) {
-		if ( __copy_from_user( to, from, sizeof(*from) ) )
+		unsigned long *t = (unsigned long *)to;
+		unsigned long *f = (unsigned long *)from;
+
+		if (__get_user(*t, f) ||
+				__get_user(*(t + 1), f + 1) ||
+				__get_user(to->exponent, &from->exponent))
 			return 1;
 	}
 	return 0;
@@ -293,7 +295,7 @@ static inline int convert_fxsr_from_user( struct i387_fxsave_struct *fxsave,
  * Signal frame handlers.
  */
 
-static inline int save_i387_fsave( struct _fpstate *buf )
+static inline int save_i387_fsave( struct _fpstate __user *buf )
 {
 	struct task_struct *tsk = current;
 
@@ -305,7 +307,7 @@ static inline int save_i387_fsave( struct _fpstate *buf )
 	return 1;
 }
 
-static inline int save_i387_fxsave( struct _fpstate *buf )
+static int save_i387_fxsave( struct _fpstate __user *buf )
 {
 	struct task_struct *tsk = current;
 	int err = 0;
@@ -326,7 +328,7 @@ static inline int save_i387_fxsave( struct _fpstate *buf )
 	return 1;
 }
 
-int save_i387( struct _fpstate *buf )
+int save_i387( struct _fpstate __user *buf )
 {
 	if ( !current->used_math )
 		return 0;
@@ -347,7 +349,7 @@ int save_i387( struct _fpstate *buf )
 	}
 }
 
-static inline int restore_i387_fsave( struct _fpstate *buf )
+static inline int restore_i387_fsave( struct _fpstate __user *buf )
 {
 	struct task_struct *tsk = current;
 	clear_fpu( tsk );
@@ -355,25 +357,25 @@ static inline int restore_i387_fsave( struct _fpstate *buf )
 				 sizeof(struct i387_fsave_struct) );
 }
 
-static inline int restore_i387_fxsave( struct _fpstate *buf )
+static int restore_i387_fxsave( struct _fpstate __user *buf )
 {
 	int err;
 	struct task_struct *tsk = current;
 	clear_fpu( tsk );
 	err = __copy_from_user( &tsk->thread.i387.fxsave, &buf->_fxsr_env[0],
 				sizeof(struct i387_fxsave_struct) );
-	/* mxcsr bit 6 and 31-16 must be zero for security reasons */
-	tsk->thread.i387.fxsave.mxcsr &= 0xffbf;
+	/* mxcsr reserved bits must be masked to zero for security reasons */
+	tsk->thread.i387.fxsave.mxcsr &= mxcsr_feature_mask;
 	return err ? 1 : convert_fxsr_from_user( &tsk->thread.i387.fxsave, buf );
 }
 
-int restore_i387( struct _fpstate *buf )
+int restore_i387( struct _fpstate __user *buf )
 {
 	int err;
 
 	if ( HAVE_HWFP ) {
 		if ( cpu_has_fxsr ) {
-			err =  restore_i387_fxsave( buf );
+			err = restore_i387_fxsave( buf );
 		} else {
 			err = restore_i387_fsave( buf );
 		}
@@ -388,21 +390,21 @@ int restore_i387( struct _fpstate *buf )
  * ptrace request handlers.
  */
 
-static inline int get_fpregs_fsave( struct user_i387_struct *buf,
+static inline int get_fpregs_fsave( struct user_i387_struct __user *buf,
 				    struct task_struct *tsk )
 {
 	return __copy_to_user( buf, &tsk->thread.i387.fsave,
 			       sizeof(struct user_i387_struct) );
 }
 
-static inline int get_fpregs_fxsave( struct user_i387_struct *buf,
+static inline int get_fpregs_fxsave( struct user_i387_struct __user *buf,
 				     struct task_struct *tsk )
 {
-	return convert_fxsr_to_user( (struct _fpstate *)buf,
+	return convert_fxsr_to_user( (struct _fpstate __user *)buf,
 				     &tsk->thread.i387.fxsave );
 }
 
-int get_fpregs( struct user_i387_struct *buf, struct task_struct *tsk )
+int get_fpregs( struct user_i387_struct __user *buf, struct task_struct *tsk )
 {
 	if ( HAVE_HWFP ) {
 		if ( cpu_has_fxsr ) {
@@ -412,25 +414,25 @@ int get_fpregs( struct user_i387_struct *buf, struct task_struct *tsk )
 		}
 	} else {
 		return save_i387_soft( &tsk->thread.i387.soft,
-				       (struct _fpstate *)buf );
+				       (struct _fpstate __user *)buf );
 	}
 }
 
 static inline int set_fpregs_fsave( struct task_struct *tsk,
-				    struct user_i387_struct *buf )
+				    struct user_i387_struct __user *buf )
 {
 	return __copy_from_user( &tsk->thread.i387.fsave, buf,
 				 sizeof(struct user_i387_struct) );
 }
 
 static inline int set_fpregs_fxsave( struct task_struct *tsk,
-				     struct user_i387_struct *buf )
+				     struct user_i387_struct __user *buf )
 {
 	return convert_fxsr_from_user( &tsk->thread.i387.fxsave,
-				       (struct _fpstate *)buf );
+				       (struct _fpstate __user *)buf );
 }
 
-int set_fpregs( struct task_struct *tsk, struct user_i387_struct *buf )
+int set_fpregs( struct task_struct *tsk, struct user_i387_struct __user *buf )
 {
 	if ( HAVE_HWFP ) {
 		if ( cpu_has_fxsr ) {
@@ -440,14 +442,14 @@ int set_fpregs( struct task_struct *tsk, struct user_i387_struct *buf )
 		}
 	} else {
 		return restore_i387_soft( &tsk->thread.i387.soft,
-					  (struct _fpstate *)buf );
+					  (struct _fpstate __user *)buf );
 	}
 }
 
-int get_fpxregs( struct user_fxsr_struct *buf, struct task_struct *tsk )
+int get_fpxregs( struct user_fxsr_struct __user *buf, struct task_struct *tsk )
 {
 	if ( cpu_has_fxsr ) {
-		if (__copy_to_user( (void *)buf, &tsk->thread.i387.fxsave,
+		if (__copy_to_user( buf, &tsk->thread.i387.fxsave,
 				    sizeof(struct user_fxsr_struct) ))
 			return -EFAULT;
 		return 0;
@@ -456,17 +458,20 @@ int get_fpxregs( struct user_fxsr_struct *buf, struct task_struct *tsk )
 	}
 }
 
-int set_fpxregs( struct task_struct *tsk, struct user_fxsr_struct *buf )
+int set_fpxregs( struct task_struct *tsk, struct user_fxsr_struct __user *buf )
 {
+	int ret = 0;
+
 	if ( cpu_has_fxsr ) {
-		__copy_from_user( &tsk->thread.i387.fxsave, (void *)buf,
-				  sizeof(struct user_fxsr_struct) );
-		/* mxcsr bit 6 and 31-16 must be zero for security reasons */
-		tsk->thread.i387.fxsave.mxcsr &= 0xffbf;
-		return 0;
+		if (__copy_from_user( &tsk->thread.i387.fxsave, buf,
+				  sizeof(struct user_fxsr_struct) ))
+			ret = -EFAULT;
+		/* mxcsr reserved bits must be masked to zero for security reasons */
+		tsk->thread.i387.fxsave.mxcsr &= mxcsr_feature_mask;
 	} else {
-		return -EIO;
+		ret = -EIO;
 	}
+	return ret;
 }
 
 /*
@@ -555,13 +560,3 @@ int dump_task_extended_fpu(struct task_struct *tsk, struct user_fxsr_struct *fpu
 	}
 	return fpvalid;
 }
-
-
-#ifdef CONFIG_SMP
-void dump_smp_unlazy_fpu(void)
-{
-	unlazy_fpu(current);
-	return;
-}
-#endif
-
