@@ -36,7 +36,7 @@
 #include <asm/mvme147hw.h>
 
 
-extern void mvme147_process_int (int level, struct pt_regs *regs);
+extern irqreturn_t mvme147_process_int (int level, struct pt_regs *regs);
 extern void mvme147_init_IRQ (void);
 extern void mvme147_free_irq (unsigned int, void *);
 extern int  show_mvme147_interrupts (struct seq_file *, void *);
@@ -44,8 +44,8 @@ extern void mvme147_enable_irq (unsigned int);
 extern void mvme147_disable_irq (unsigned int);
 static void mvme147_get_model(char *model);
 static int  mvme147_get_hardware_list(char *buffer);
-extern int mvme147_request_irq (unsigned int irq, void (*handler)(int, void *, struct pt_regs *), unsigned long flags, const char *devname, void *dev_id);
-extern void mvme147_sched_init(void (*handler)(int, void *, struct pt_regs *));
+extern int mvme147_request_irq (unsigned int irq, irqreturn_t (*handler)(int, void *, struct pt_regs *), unsigned long flags, const char *devname, void *dev_id);
+extern void mvme147_sched_init(irqreturn_t (*handler)(int, void *, struct pt_regs *));
 extern unsigned long mvme147_gettimeoffset (void);
 extern int mvme147_hwclk (int, struct rtc_time *);
 extern int mvme147_set_clock_mmss (unsigned long);
@@ -58,7 +58,7 @@ static int bcd2int (unsigned char b);
 /* Save tick handler routine pointer, will point to do_timer() in
  * kernel/sched.c, called via mvme147_process_int() */
 
-void (*tick_handler)(int, void *, struct pt_regs *);
+irqreturn_t (*tick_handler)(int, void *, struct pt_regs *);
 
 
 int mvme147_parse_bootinfo(const struct bi_record *bi)
@@ -118,15 +118,15 @@ void __init config_mvme147(void)
 
 /* Using pcc tick timer 1 */
 
-static void mvme147_timer_int (int irq, void *dev_id, struct pt_regs *fp)
+static irqreturn_t mvme147_timer_int (int irq, void *dev_id, struct pt_regs *fp)
 {
 	m147_pcc->t1_int_cntrl = PCC_TIMER_INT_CLR;  
 	m147_pcc->t1_int_cntrl = PCC_INT_ENAB|PCC_LEVEL_TIMER1;   
-	tick_handler(irq, dev_id, fp);
+	return tick_handler(irq, dev_id, fp);
 }
 
 
-void mvme147_sched_init (void (*timer_routine)(int, void *, struct pt_regs *))
+void mvme147_sched_init (irqreturn_t (*timer_routine)(int, void *, struct pt_regs *))
 {
 	tick_handler = timer_routine;
 	request_irq (PCC_IRQ_TIMER1, mvme147_timer_int, 
