@@ -29,7 +29,7 @@
  *  with this program; if not, write  to the Free Software Foundation, Inc.,
  *  675 Mass Ave, Cambridge, MA 02139, USA.
  */
-
+#include <linux/config.h>
 #include <linux/init.h>
 #include <linux/pm.h>
 #include <linux/slab.h>
@@ -39,7 +39,9 @@
 #include <asm/uaccess.h>
 #include <asm/io.h>
 #include <asm/system.h>
-#include <asm/au1000.h>
+#include <asm/mach-au1x00/au1000.h>
+
+#ifdef CONFIG_PM
 
 #define DEBUG 1
 #ifdef DEBUG
@@ -63,7 +65,6 @@ extern void local_enable_irq(unsigned int irq_nr);
 #define	ACPI_S1_SLP_TYP 19
 #define	ACPI_SLEEP 21
 
-#ifdef CONFIG_PM
 
 static spinlock_t pm_lock = SPIN_LOCK_UNLOCKED;
 
@@ -357,7 +358,7 @@ static int pm_do_freq(ctl_table * ctl, int write, struct file *file,
 		old_cpu_freq = get_au1x00_speed();
 
 		new_cpu_freq = pll * 12 * 1000000;
-		new_baud_base = (new_cpu_freq / 4) / 16;
+	        new_baud_base =  (new_cpu_freq / (2 * ((int)(au_readl(SYS_POWERCTRL)&0x03) + 2) * 16));
 		set_au1x00_speed(new_cpu_freq);
 		set_au1x00_uart_baud_base(new_baud_base);
 
