@@ -32,6 +32,7 @@
 
 #include <linux/config.h>
 #include <linux/module.h>
+#include <linux/kernel.h>
 #include <linux/errno.h>
 #include <linux/signal.h>
 #include <linux/sched.h>
@@ -63,7 +64,7 @@
 
 
 /* -----------------------------------------------------------------------------
- *  From STB03xxx SICC UART Specification 
+ *  From STB03xxx SICC UART Specification
  * -----------------------------------------------------------------------------
  *  UART Register Offsets.
  */
@@ -139,7 +140,7 @@
 #define _LSR_RX_ERR    (_LSR_LB_BREAK | _LSR_FE_MASK | _LSR_OE_MASK | \
 			 _LSR_PE_MASK )
 
-/* serial port reciever command register */
+/* serial port receiver command register */
 
 #define _RCR_ER_MASK   0x80           /* enable receiver mask */
 #define _RCR_DME_MASK  0x60           /* dma mode */
@@ -183,11 +184,6 @@
 #define SERIAL_SICC_MINOR   1
 #define SERIAL_SICC_NR      1
 
-#define CALLOUT_SICC_NAME   "cuasicc"
-#define CALLOUT_SICC_MAJOR  151
-#define CALLOUT_SICC_MINOR  1
-#define CALLOUT_SICC_NR     SERIAL_SICC_NR
-
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -195,18 +191,10 @@
 #define FALSE 0
 #endif
 
-#define DEBUG 0
-
-
-
 /*
  * Things needed by tty driver
  */
-static struct tty_driver siccnormal_driver, sicccallout_driver;
-static int siccuart_refcount;
-static struct tty_struct *siccuart_table[SERIAL_SICC_NR];
-static struct termios *siccuart_termios[SERIAL_SICC_NR];
-static struct termios *siccuart_termios_locked[SERIAL_SICC_NR];
+static struct tty_driver *siccnormal_driver;
 
 #if defined(CONFIG_SERIAL_SICC_CONSOLE) && defined(CONFIG_MAGIC_SYSRQ)
 #define SUPPORT_SYSRQ
@@ -262,7 +250,7 @@ struct SICC_port {
     unsigned int        fifosize;
     unsigned int        tiocm_support;
     void (*set_mctrl)(struct SICC_port *, u_int mctrl);
-};  
+};
 
 /*
  * This is the state information which is persistent across opens
@@ -274,9 +262,6 @@ struct SICC_state {
     unsigned int        closing_wait;
     unsigned int        custom_divisor;
     unsigned int        flags;
-    struct termios      normal_termios;
-    struct termios      callout_termios;
-
     int         count;
     struct SICC_info    *info;
 };
@@ -304,8 +289,6 @@ struct SICC_info {
     unsigned int        lcr_h;
     unsigned int        mctrl;
     int         blocked_open;
-    pid_t           session;
-    pid_t           pgrp;
 
     struct tasklet_struct   tlet;
 
@@ -431,11 +414,7 @@ static void siccuart_event(struct SICC_info *info, int event)
 }
 
 static void
-#ifdef SUPPORT_SYSRQ
 siccuart_rx_chars(struct SICC_info *info, struct pt_regs *regs)
-#else
-siccuart_rx_chars(struct SICC_info *info)
-#endif
 {
     struct tty_struct *tty = info->tty;
     unsigned int status, ch, rsr, flg, ignored = 0;
@@ -574,25 +553,19 @@ static void siccuart_tx_chars(struct SICC_info *info)
 }
 
 
-static void siccuart_int_rx(int irq, void *dev_id, struct pt_regs *regs)
+static irqreturn_t siccuart_int_rx(int irq, void *dev_id, struct pt_regs *regs)
 {
     struct SICC_info *info = dev_id;
-
-#ifdef SUPPORT_SYSRQ
-            siccuart_rx_chars(info, regs);
-#else
-            siccuart_rx_chars(info);
-#endif
-
-      //powerpcClearUicsrBits(0x00000400);
+    siccuart_rx_chars(info, regs);
+    return IRQ_HANDLED;
 }
 
 
-static void siccuart_int_tx(int irq, void *dev_id, struct pt_regs *regs)
+static irqreturn_t siccuart_int_tx(int irq, void *dev_id, struct pt_regs *regs)
 {
     struct SICC_info *info = dev_id;
     siccuart_tx_chars(info);
-
+    return IRQ_HANDLED;
 }
 
 static void siccuart_tasklet_action(unsigned long data)
@@ -787,9 +760,7 @@ static void siccuart_change_speed(struct SICC_info *info, struct termios *old_te
 
     cflag = info->tty->termios->c_cflag;
 
-#if DEBUG
-    printk("siccuart_set_cflag(0x%x) called\n", cflag);
-#endif
+    pr_debug("siccuart_set_cflag(0x%x) called\n", cflag);
     /* byte size and parity */
     switch (cflag & CSIZE) {
     case CS7: lcr_h =   _LCR_PE_DISABLE | _LCR_DB_7_BITS | _LCR_SB_1_BIT; bits = 9;  break;
@@ -857,7 +828,7 @@ static void siccuart_change_speed(struct SICC_info *info, struct termios *old_te
     /* As a last resort, if the quotient is zero, default to 9600 bps */
     if (!quot)
         quot = (info->port->uartclk / (16 * 9600)) - 1;
-        
+
     info->timeout = info->port->fifosize * HZ * bits / baud;
     info->timeout += HZ/50;     /* Add .02 seconds of slop */
 
@@ -1051,10 +1022,7 @@ static void siccuart_flush_buffer(struct tty_struct *tty)
     struct SICC_info *info = tty->driver_data;
     unsigned long flags;
 
-#if DEBUG
-    printk("siccuart_flush_buffer(%d) called\n",
-           MINOR(tty->device) - tty->driver.minor_start);
-#endif
+    pr_debug("siccuart_flush_buffer(%d) called\n", tty->index);
     save_flags(flags); cli();
     info->xmit.head = info->xmit.tail = 0;
     restore_flags(flags);
@@ -1264,7 +1232,7 @@ static int get_lsr_info(struct SICC_info *info, unsigned int *value)
                SICC_XMIT_SIZE) > 0) &&
          !info->tty->stopped && !info->tty->hw_stopped))
         result &= TIOCSER_TEMT;
-    
+
     return put_user(result, value);
 }
 
@@ -1458,9 +1426,7 @@ static void siccuart_close(struct tty_struct *tty, struct file *filp)
 
     state = info->state;
 
-#if DEBUG
-    //printk("siccuart_close() called\n");
-#endif
+    //pr_debug("siccuart_close() called\n");
 
     save_flags(flags); cli();
 
@@ -1482,7 +1448,7 @@ static void siccuart_close(struct tty_struct *tty, struct file *filp)
         state->count = 1;
     }
     if (--state->count < 0) {
-        printk("rs_close: bad serial port count for %s%d: %d\n", tty->driver.name, info->state->line, state->count);
+        printk("rs_close: bad serial port count for %s: %d\n", tty->name, state->count);
         state->count = 0;
     }
     if (state->count) {
@@ -1492,14 +1458,6 @@ static void siccuart_close(struct tty_struct *tty, struct file *filp)
     }
     info->flags |= ASYNC_CLOSING;
     restore_flags(flags);
-    /*
-     * Save the termios structure, since this port may have
-     * separate termios for callout and dialin.
-     */
-    if (info->flags & ASYNC_NORMAL_ACTIVE)
-        info->state->normal_termios = *tty->termios;
-    if (info->flags & ASYNC_CALLOUT_ACTIVE)
-        info->state->callout_termios = *tty->termios;
     /*
      * Now we wait for the transmit buffer to clear; and we notify
      * the line discipline to only process XON/XOFF characters.
@@ -1521,8 +1479,8 @@ static void siccuart_close(struct tty_struct *tty, struct file *filp)
         siccuart_wait_until_sent(tty, info->timeout);
     }
     siccuart_shutdown(info);
-    if (tty->driver.flush_buffer)
-        tty->driver.flush_buffer(tty);
+    if (tty->driver->flush_buffer)
+        tty->driver->flush_buffer(tty);
     if (tty->ldisc.flush_buffer)
         tty->ldisc.flush_buffer(tty);
     tty->closing = 0;
@@ -1535,8 +1493,7 @@ static void siccuart_close(struct tty_struct *tty, struct file *filp)
         }
         wake_up_interruptible(&info->open_wait);
     }
-    info->flags &= ~(ASYNC_NORMAL_ACTIVE|ASYNC_CALLOUT_ACTIVE|
-             ASYNC_CLOSING);
+    info->flags &= ~(ASYNC_NORMAL_ACTIVE|ASYNC_CLOSING);
     wake_up_interruptible(&info->close_wait);
     MOD_DEC_USE_COUNT;
 }
@@ -1578,11 +1535,9 @@ static void siccuart_wait_until_sent(struct tty_struct *tty, int timeout)
         timeout = 2 * info->timeout;
 
     expire = jiffies + timeout;
-#if DEBUG
-    printk("siccuart_wait_until_sent(%d), jiff=%lu, expire=%lu  char_time=%lu...\n",
-           MINOR(tty->device) - tty->driver.minor_start, jiffies,
+    pr_debug("siccuart_wait_until_sent(%d), jiff=%lu, expire=%lu  char_time=%lu...\n",
+           tty->index, jiffies,
            expire, char_time);
-#endif
     while ((readb(info->port->uart_base + BL_SICC_LSR) & _LSR_TX_ALL) != _LSR_TX_ALL) {
         set_current_state(TASK_INTERRUPTIBLE);
         schedule_timeout(char_time);
@@ -1605,7 +1560,7 @@ static void siccuart_hangup(struct tty_struct *tty)
     siccuart_shutdown(info);
     info->event = 0;
     state->count = 0;
-    info->flags &= ~(ASYNC_NORMAL_ACTIVE|ASYNC_CALLOUT_ACTIVE);
+    info->flags &= ~ASYNC_NORMAL_ACTIVE;
     info->tty = NULL;
     wake_up_interruptible(&info->open_wait);
 }
@@ -1630,44 +1585,18 @@ static int block_til_ready(struct tty_struct *tty, struct file *filp,
             -EAGAIN : -ERESTARTSYS;
     }
 
-    /*
-     * If this is a callout device, then just make sure the normal
-     * device isn't being used.
-     */
-    if (tty->driver.subtype == SERIAL_TYPE_CALLOUT) {
-        if (info->flags & ASYNC_NORMAL_ACTIVE)
-            return -EBUSY;
-        if ((info->flags & ASYNC_CALLOUT_ACTIVE) &&
-            (info->flags & ASYNC_SESSION_LOCKOUT) &&
-            (info->session != current->session))
-            return -EBUSY;
-        if ((info->flags & ASYNC_CALLOUT_ACTIVE) &&
-            (info->flags & ASYNC_PGRP_LOCKOUT) &&
-            (info->pgrp != current->pgrp))
-            return -EBUSY;
-        info->flags |= ASYNC_CALLOUT_ACTIVE;
-        return 0;
-    }
-
     /*
      * If non-blocking mode is set, or the port is not enabled,
      * then make the check up front and then exit.
      */
     if ((filp->f_flags & O_NONBLOCK) ||
         (tty->flags & (1 << TTY_IO_ERROR))) {
-        if (info->flags & ASYNC_CALLOUT_ACTIVE)
-            return -EBUSY;
         info->flags |= ASYNC_NORMAL_ACTIVE;
         return 0;
     }
 
-    if (info->flags & ASYNC_CALLOUT_ACTIVE) {
-        if (state->normal_termios.c_cflag & CLOCAL)
-            do_clocal = 1;
-    } else {
-        if (tty->termios->c_cflag & CLOCAL)
-            do_clocal = 1;
-    }
+    if (tty->termios->c_cflag & CLOCAL)
+	do_clocal = 1;
 
     /*
      * Block waiting for the carrier detect and the line to become
@@ -1687,8 +1616,7 @@ static int block_til_ready(struct tty_struct *tty, struct file *filp,
     info->blocked_open++;
     while (1) {
         save_flags(flags); cli();
-        if (!(info->flags & ASYNC_CALLOUT_ACTIVE) &&
-            (tty->termios->c_cflag & CBAUD)) {
+        if (tty->termios->c_cflag & CBAUD) {
             info->mctrl = TIOCM_DTR | TIOCM_RTS;
             info->port->set_mctrl(info->port, info->mctrl);
         }
@@ -1702,8 +1630,7 @@ static int block_til_ready(struct tty_struct *tty, struct file *filp,
                 retval = -ERESTARTSYS;
             break;
         }
-        if (!(info->flags & ASYNC_CALLOUT_ACTIVE) &&
-            !(info->flags & ASYNC_CLOSING) &&
+        if (!(info->flags & ASYNC_CLOSING) &&
             (do_clocal /*|| (UART_GET_FR(info->port) & SICC_UARTFR_DCD)*/))
             break;
         if (signal_pending(current)) {
@@ -1754,7 +1681,7 @@ static struct SICC_info *siccuart_get(int line)
 static int siccuart_open(struct tty_struct *tty, struct file *filp)
 {
     struct SICC_info *info;
-    int retval, line = MINOR(tty->device) - tty->driver.minor_start;
+    int retval, line = tty->index;
 
 
     // is this a line that we've got?
@@ -1812,15 +1739,6 @@ static int siccuart_open(struct tty_struct *tty, struct file *filp)
         return retval;
     }
 
-    if ((info->state->count == 1) &&
-        (info->flags & ASYNC_SPLIT_TERMIOS)) {
-        if (tty->driver.subtype == SERIAL_TYPE_NORMAL) {
-            *tty->termios = info->state->normal_termios;
-        }
-        else  {
-            *tty->termios = info->state->callout_termios;
-        }
-    }
 #ifdef CONFIG_SERIAL_SICC_CONSOLE
     if (siccuart_cons.cflag && siccuart_cons.index == line) {
         tty->termios->c_cflag = siccuart_cons.cflag;
@@ -1828,74 +1746,56 @@ static int siccuart_open(struct tty_struct *tty, struct file *filp)
         siccuart_change_speed(info, NULL);
     }
 #endif
-    info->session = current->session;
-    info->pgrp = current->pgrp;
     return 0;
 }
 
+static struct tty_operations sicc_ops = {
+	.open = siccuart_open,
+	.close = siccuart_close,
+	.write = siccuart_write,
+	.put_char = siccuart_put_char,
+	.flush_chars = siccuart_flush_chars,
+	.write_room = siccuart_write_room,
+	.chars_in_buffer = siccuart_chars_in_buffer,
+	.flush_buffer  = siccuart_flush_buffer,
+	.ioctl = siccuart_ioctl,
+	.throttle = siccuart_throttle,
+	.unthrottle = siccuart_unthrottle,
+	.send_xchar = siccuart_send_xchar,
+	.set_termios = siccuart_set_termios,
+	.stop = siccuart_stop,
+	.start = siccuart_start,
+	.hangup = siccuart_hangup,
+	.break_ctl = siccuart_break_ctl,
+	.wait_until_sent = siccuart_wait_until_sent,
+};
+
 int __init siccuart_init(void)
 {
     int i;
+    siccnormal_driver = alloc_tty_driver(SERIAL_SICC_NR);
+    if (!siccnormal_driver)
+	return -ENOMEM;
     printk("IBM Vesta SICC serial port driver V 0.1 by Yudong Yang and Yi Ge / IBM CRL .\n");
-    siccnormal_driver.magic = TTY_DRIVER_MAGIC;
-    siccnormal_driver.driver_name = "serial_sicc";
-    siccnormal_driver.name = SERIAL_SICC_NAME;
-    siccnormal_driver.major = SERIAL_SICC_MAJOR;
-    siccnormal_driver.minor_start = SERIAL_SICC_MINOR;
-    siccnormal_driver.num = SERIAL_SICC_NR;
-    siccnormal_driver.type = TTY_DRIVER_TYPE_SERIAL;
-    siccnormal_driver.subtype = SERIAL_TYPE_NORMAL;
-    siccnormal_driver.init_termios = tty_std_termios;
-    siccnormal_driver.init_termios.c_cflag = B9600 | CS8 | CREAD | HUPCL | CLOCAL;
-    siccnormal_driver.flags = TTY_DRIVER_REAL_RAW | TTY_DRIVER_NO_DEVFS;
-    siccnormal_driver.refcount = &siccuart_refcount;
-    siccnormal_driver.table = siccuart_table;
-    siccnormal_driver.termios = siccuart_termios;
-    siccnormal_driver.termios_locked = siccuart_termios_locked;
+    siccnormal_driver->driver_name = "serial_sicc";
+    siccnormal_driver->name = SERIAL_SICC_NAME;
+    siccnormal_driver->major = SERIAL_SICC_MAJOR;
+    siccnormal_driver->minor_start = SERIAL_SICC_MINOR;
+    siccnormal_driver->type = TTY_DRIVER_TYPE_SERIAL;
+    siccnormal_driver->subtype = SERIAL_TYPE_NORMAL;
+    siccnormal_driver->init_termios = tty_std_termios;
+    siccnormal_driver->init_termios.c_cflag = B9600 | CS8 | CREAD | HUPCL | CLOCAL;
+    siccnormal_driver->flags = TTY_DRIVER_REAL_RAW | TTY_DRIVER_NO_DEVFS;
+    tty_set_operations(siccnormal_driver, &sicc_ops);
 
-    siccnormal_driver.open = siccuart_open;
-    siccnormal_driver.close = siccuart_close;
-    siccnormal_driver.write = siccuart_write;
-    siccnormal_driver.put_char = siccuart_put_char;
-    siccnormal_driver.flush_chars = siccuart_flush_chars;
-    siccnormal_driver.write_room = siccuart_write_room;
-    siccnormal_driver.chars_in_buffer = siccuart_chars_in_buffer;
-    siccnormal_driver.flush_buffer  = siccuart_flush_buffer;
-    siccnormal_driver.ioctl = siccuart_ioctl;
-    siccnormal_driver.throttle = siccuart_throttle;
-    siccnormal_driver.unthrottle = siccuart_unthrottle;
-    siccnormal_driver.send_xchar = siccuart_send_xchar;
-    siccnormal_driver.set_termios = siccuart_set_termios;
-    siccnormal_driver.stop = siccuart_stop;
-    siccnormal_driver.start = siccuart_start;
-    siccnormal_driver.hangup = siccuart_hangup;
-    siccnormal_driver.break_ctl = siccuart_break_ctl;
-    siccnormal_driver.wait_until_sent = siccuart_wait_until_sent;
-    siccnormal_driver.read_proc = NULL;
-
-    /*
-     * The callout device is just like the normal device except for
-     * the major number and the subtype code.
-     */
-    sicccallout_driver = siccnormal_driver;
-    sicccallout_driver.name = CALLOUT_SICC_NAME;
-    sicccallout_driver.major = CALLOUT_SICC_MAJOR;
-    sicccallout_driver.subtype = SERIAL_TYPE_CALLOUT;
-    sicccallout_driver.read_proc = NULL;
-    sicccallout_driver.proc_entry = NULL;
-
-    if (tty_register_driver(&siccnormal_driver))
+    if (tty_register_driver(siccnormal_driver))
         panic("Couldn't register SICC serial driver\n");
-    if (tty_register_driver(&sicccallout_driver))
-        panic("Couldn't register SICC callout driver\n");
 
     for (i = 0; i < SERIAL_SICC_NR; i++) {
         struct SICC_state *state = sicc_state + i;
         state->line     = i;
         state->close_delay  = 5 * HZ / 10;
         state->closing_wait = 30 * HZ;
-        state->callout_termios  = sicccallout_driver.init_termios;
-        state->normal_termios   = siccnormal_driver.init_termios;
     }
 
 
@@ -1920,9 +1820,8 @@ static int siccuart_console_read(struct console *co, const char *s, u_int count)
     unsigned int status;
     char *w;
     int c;
-#if DEBUG
-    printk("siccuart_console_read() called\n");
-#endif
+
+    pr_debug("siccuart_console_read() called\n");
 
     c = 0;
     w = s;
@@ -1991,12 +1890,12 @@ static int siccuart_console_wait_key(struct console *co)
     return c;
 }
 
-static kdev_t siccuart_console_device(struct console *c)
+static struct tty_driver *siccuart_console_device(struct console *c, int *index)
 {
-    return MKDEV(SERIAL_SICC_MAJOR, SERIAL_SICC_MINOR + c->index);
+	*index = c->index;
+	return siccnormal_driver;
 }
 
-
 static int __init siccuart_console_setup(struct console *co, char *options)
 {
     struct SICC_port *port;
