@@ -16,6 +16,7 @@
 #include <linux/reboot.h>
 #include <linux/delay.h>
 #include <linux/kallsyms.h>
+#include <linux/cpumask.h>
 
 #include <asm/ptrace.h>
 #include <asm/string.h>
@@ -37,7 +38,7 @@
 #define skipbl	xmon_skipbl
 
 #ifdef CONFIG_SMP
-volatile unsigned long cpus_in_xmon = 0;
+volatile cpumask_t cpus_in_xmon = CPU_MASK_NONE;
 static unsigned long got_xmon = 0;
 static volatile int take_xmon = -1;
 static volatile int leaving_xmon = 0;
@@ -49,14 +50,12 @@ static unsigned long ndump = 64;
 static unsigned long nidump = 16;
 static unsigned long ncsum = 4096;
 static int termch;
+static char tmpstr[128];
 
 static u_int bus_error_jmp[100];
 #define setjmp xmon_setjmp
 #define longjmp xmon_longjmp
 
-/* Max number of stack frames we are willing to produce on a backtrace. */
-#define MAXFRAMECOUNT 50
-
 /* Breakpoint stuff */
 struct bpt {
 	unsigned long address;
@@ -72,11 +71,10 @@ static struct bpt iabr;
 static unsigned bpinstr = 0x7fe00008;	/* trap */
 
 /* Prototypes */
-extern void (*debugger_fault_handler)(struct pt_regs *);
 static int cmds(struct pt_regs *);
 static int mread(unsigned long, void *, int);
 static int mwrite(unsigned long, void *, int);
-static void handle_fault(struct pt_regs *);
+static int handle_fault(struct pt_regs *);
 static void byterev(unsigned char *, int);
 static void memex(void);
 static int bsesc(void);
@@ -84,7 +82,6 @@ static void dump(void);
 static void prdump(unsigned long, long);
 static int ppc_inst_dump(unsigned long, long);
 void print_address(unsigned long);
-static int getsp(void);
 static void backtrace(struct pt_regs *);
 static void excprint(struct pt_regs *);
 static void prregs(struct pt_regs *);
@@ -114,10 +111,8 @@ static void cpu_cmd(void);
 #endif /* CONFIG_SMP */
 static void csum(void);
 static void bootcmds(void);
-static void mem_translate(void);
-static void mem_check(void);
-static void mem_find_real(void);
-static void mem_find_vsid(void);
+void dump_segments(void);
+static void symbol_lookup(void);
 
 static void debug_trace(void);
 
@@ -148,20 +143,29 @@ Commands:\n\
   b	show breakpoints\n\
   bd	set data breakpoint\n\
   bi	set instruction breakpoint\n\
-  bc	clear breakpoint\n\
+  bc	clear breakpoint\n"
+#ifdef CONFIG_SMP
+  "\
+  c	print cpus stopped in xmon\n\
+  ci	send xmon interrupt to all other cpus\n\
+  c#	try to switch to cpu number h (in hex)\n"
+#endif
+  "\
+  C	checksum\n\
   d	dump bytes\n\
   di	dump instructions\n\
   df	dump float values\n\
   dd	dump double values\n\
   e	print exception information\n\
   f	flush cache\n\
+  la	lookup symbol+offset of specified address\n\
+  ls	lookup address of specified symbol\n\
   m	examine/change memory\n\
   mm	move a block of memory\n\
   ms	set a block of memory\n\
   md	compare two blocks of memory\n\
   ml	locate a block of memory\n\
   mz	zero a block of memory\n\
-  mx	translation information for an effective address\n\
   mi	show information about memory allocation\n\
   p 	show the task list\n\
   r	print registers\n\
@@ -169,8 +173,14 @@ Commands:\n\
   S	print special registers\n\
   t	print backtrace\n\
   T	Enable/Disable PPCDBG flags\n\
-  x	exit monitor\n\
-";
+  x	exit monitor and recover\n\
+  X	exit monitor and dont recover\n\
+  u	dump segment table or SLB\n\
+  ?	help\n"
+  "\
+  zr	reboot\n\
+  zh	halt\n"
+;
 
 static int xmon_trace[NR_CPUS];
 #define SSTEP	1		/* stepping because of 's' command */
@@ -223,7 +233,7 @@ static inline void disable_surveillance(void)
 #endif
 }
 
-void
+int
 xmon(struct pt_regs *excp)
 {
 	struct pt_regs regs;
@@ -288,17 +298,18 @@ xmon(struct pt_regs *excp)
 	leaving_xmon = 0;
 	/* possible race condition here if a CPU is held up and gets
 	 * here while we are exiting */
-	if (test_and_set_bit(smp_processor_id(), &cpus_in_xmon)) {
+	if (cpu_test_and_set(smp_processor_id(), cpus_in_xmon)) {
 		/* xmon probably caused an exception itself */
 		printf("We are already in xmon\n");
 		for (;;)
-			;
+			cpu_relax();
 	}
 	while (test_and_set_bit(0, &got_xmon)) {
 		if (take_xmon == smp_processor_id()) {
 			take_xmon = -1;
 			break;
 		}
+		cpu_relax();
 	}
 	/*
 	 * XXX: breakpoints are removed while any cpu is in xmon
@@ -306,6 +317,7 @@ xmon(struct pt_regs *excp)
 #endif /* CONFIG_SMP */
 	remove_bpts();
 	disable_surveillance();
+	printf("press ? for help ");
 	cmd = cmds(excp);
 	if (cmd == 's') {
 		xmon_trace[smp_processor_id()] = SSTEP;
@@ -325,20 +337,14 @@ xmon(struct pt_regs *excp)
 	leaving_xmon = 1;
 	if (cmd != 's')
 		clear_bit(0, &got_xmon);
-	clear_bit(smp_processor_id(), &cpus_in_xmon);
+	cpu_clear(smp_processor_id(), cpus_in_xmon);
 #endif /* CONFIG_SMP */
 	set_msrd(msr);		/* restore interrupt enable */
-}
 
-void
-xmon_irq(int irq, void *d, struct pt_regs *regs)
-{
-	unsigned long flags;
-	local_save_flags(flags);
-	local_irq_disable();
-	printf("Keyboard interrupt\n");
-	xmon(regs);
-	local_irq_restore(flags);
+	if (cmd == 'X')
+		return 0;
+
+	return 1;
 }
 
 int
@@ -446,7 +452,7 @@ insert_bpts()
 		}
 	}
 
-	if ((cur_cpu_spec->cpu_features & CPU_FTR_DABR) && dabr.enabled)
+	if (dabr.enabled)
 		set_dabr(dabr.address);
 	if ((cur_cpu_spec->cpu_features & CPU_FTR_IABR) && iabr.enabled)
 		set_iabr(iabr.address);
@@ -459,8 +465,7 @@ remove_bpts()
 	struct bpt *bp;
 	unsigned instr;
 
-	if ((cur_cpu_spec->cpu_features & CPU_FTR_DABR))
-		set_dabr(0);
+	set_dabr(0);
 	if ((cur_cpu_spec->cpu_features & CPU_FTR_IABR))
 		set_iabr(0);
 
@@ -522,18 +527,6 @@ cmds(struct pt_regs *excp)
 			case 'z':
 				memzcan();
 				break;
-			case 'x':
-				mem_translate();
-				break;
-			case 'c':
-				mem_check();
-				break;
-			case 'f':
-				mem_find_real();
-				break;
-			case 'e':
-				mem_find_vsid();
-				break;
 			case 'i':
 				show_mem();
 				break;
@@ -545,9 +538,11 @@ cmds(struct pt_regs *excp)
 		case 'd':
 			dump();
 			break;
+		case 'l':
+			symbol_lookup();
+			break;
 		case 'r':
-			if (excp != NULL)
-				prregs(excp);	/* print regs */
+			prregs(excp);	/* print regs */
 			break;
 		case 'e':
 			if (excp == NULL)
@@ -566,6 +561,7 @@ cmds(struct pt_regs *excp)
 			break;
 		case 's':
 		case 'x':
+		case 'X':
 		case EOF:
 			return cmd;
 		case '?':
@@ -590,6 +586,9 @@ cmds(struct pt_regs *excp)
 		case 'T':
 			debug_trace();
 			break;
+		case 'u':
+			dump_segments();
+			break;
 		default:
 			printf("Unrecognized command: ");
 		        do {
@@ -602,6 +601,7 @@ cmds(struct pt_regs *excp)
 			printf(" (type ? for help)\n");
 			break;
 		}
+		cpu_relax();
 	}
 }
 
@@ -630,7 +630,7 @@ static void cpu_cmd(void)
 		printf("stopping all cpus\n");
 		/* interrupt other cpu(s) */
 		cpu = MSG_ALL_BUT_SELF;
-		smp_send_xmon_break(cpu);
+		smp_send_debugger_break(cpu);
 		return;
 	}
 	termch = cmd;
@@ -638,7 +638,7 @@ static void cpu_cmd(void)
 		/* print cpus waiting or in xmon */
 		printf("cpus stopped:");
 		for (cpu = 0; cpu < NR_CPUS; ++cpu) {
-			if (test_bit(cpu, &cpus_in_xmon)) {
+			if (cpu_isset(cpu, cpus_in_xmon)) {
 				printf(" %x", cpu);
 				if (cpu == smp_processor_id())
 					printf("*", cpu);
@@ -664,6 +664,7 @@ static void cpu_cmd(void)
 			take_xmon = -1;
 			break;
 		}
+		cpu_relax();
 	}
 }
 #endif /* CONFIG_SMP */
@@ -748,10 +749,6 @@ bpt_cmds(void)
 	cmd = inchar();
 	switch (cmd) {
 	case 'd':	/* bd - hardware data breakpoint */
-		if (!(cur_cpu_spec->cpu_features & CPU_FTR_DABR)) {
-			printf("Not implemented on this cpu\n");
-			break;
-		}
 		mode = 7;
 		cmd = inchar();
 		if (cmd == 'r')
@@ -843,7 +840,8 @@ bpt_cmds(void)
 			break;
 		}
 		
-		if (!(systemcfg->platform & PLATFORM_PSERIES)) {
+		if (systemcfg->platform != PLATFORM_POWERMAC &&
+		   !(systemcfg->platform & PLATFORM_PSERIES)) {
 			printf("Not supported for this platform\n");
 			break;
 		}
@@ -894,99 +892,80 @@ const char *getvecname(unsigned long vec)
 	return ret;
 }
 
-static void
-backtrace(struct pt_regs *excp)
+/*
+ * Most of our exceptions are in the form:
+ *    bl handler
+ *    b .ret_from_exception
+ * and this currently fails to catch them.
+ */
+static inline int exception_frame(unsigned long ip)
 {
-	unsigned long sp;
-	unsigned long lr;
-	unsigned long stack[3];
-	struct pt_regs regs;
-	int framecount;
-	char *funcname;
-	/* declare these as raw ptrs so we don't get func descriptors */
-	extern void *ret_from_except, *ret_from_syscall_1;
+	extern void *ret_from_syscall_1, *ret_from_syscall_2, *ret_from_except;
 
-	if (excp != NULL) {
-	        lr = excp->link;
-		sp = excp->gpr[1];
-	} else {
-	        /* Use care not to call any function before this point
-		 so the saved lr has a chance of being good. */
-	        asm volatile ("mflr %0" : "=r" (lr) :);
-		sp = getsp();
-	}
-	scanhex(&sp);
-	scannl();
-	for (framecount = 0;
-	     sp != 0 && framecount < MAXFRAMECOUNT;
-	     sp = stack[0], framecount++) {
-		if (mread(sp, stack, sizeof(stack)) != sizeof(stack))
-			break;
-#if 0
-		if (lr != 0) {
-		    stack[2] = lr;	/* fake out the first saved lr.  It may not be saved yet. */
-		    lr = 0;
-		}
-#endif
-		printf("%.16lx  %.16lx", sp, stack[2]);
-		/* TAI -- for now only the ones cast to unsigned long will match.
-		 * Need to test the rest...
-		 */
-		if ((stack[2] == (unsigned long)ret_from_except &&
-		            (funcname = "ret_from_except"))
-		    || (stack[2] == (unsigned long)ret_from_syscall_1 &&
-		            (funcname = "ret_from_syscall_1"))
-#if 0
-		    || stack[2] == (unsigned) &ret_from_syscall_2
-		    || stack[2] == (unsigned) &do_signal_ret
-#endif
-		    ) {
-			printf("  %s\n", funcname);
-			if (mread(sp+112, &regs, sizeof(regs)) != sizeof(regs))
-				break;
-			printf("exception: %lx %s regs %lx\n", regs.trap, getvecname(regs.trap), sp+112);
-			printf("                  %.16lx", regs.nip);
-			if (regs.nip & 0xffffffff00000000UL)
-				xmon_print_symbol("  %s", regs.nip);
-			printf("\n");
-                        if (regs.gpr[1] < sp) {
-                            printf("<Stack drops into userspace %.16lx>\n", regs.gpr[1]);
-                            break;
-			}
+	if ((ip == (unsigned long)ret_from_syscall_1) ||
+	    (ip == (unsigned long)ret_from_syscall_2) ||
+	    (ip == (unsigned long)ret_from_except))
+		return 1;
 
-			sp = regs.gpr[1];
-			if (mread(sp, stack, sizeof(stack)) != sizeof(stack))
-				break;
-		} else {
-			if (stack[2])
-				xmon_print_symbol("  %s", stack[2]);
-			printf("\n");
-		}
-		if (stack[0] && stack[0] <= sp) {
-			if ((stack[0] & 0xffffffff00000000UL) == 0)
-				printf("<Stack drops into 32-bit userspace %.16lx>\n", stack[0]);
-			else
-				printf("<Corrupt stack.  Next backchain is %.16lx>\n", stack[0]);
-			break;
-		}
-	}
-	if (framecount >= MAXFRAMECOUNT)
-		printf("<Punt. Too many stack frames>\n");
+	return 0;
 }
 
-int
-getsp()
-{
-	int x;
+static int xmon_depth_to_print = 64;
 
-	asm("mr %0,1" : "=r" (x) :);
-	return x;
+static void xmon_show_stack(unsigned long sp)
+{
+	unsigned long ip;
+	unsigned long newsp;
+	int count = 0;
+	struct pt_regs regs;
+
+	do {
+		if (sp < PAGE_OFFSET) {
+			printf("SP in userspace\n");
+			break;
+		}
+
+		if (!mread((sp + 16), &ip, sizeof(unsigned long)))
+			break;
+
+		printf("[%016lx] [%016lx] ", sp, ip);
+		xmon_print_symbol("%s\n", ip);
+
+		if (exception_frame(ip)) {
+			if (mread(sp+112, &regs, sizeof(regs)) != sizeof(regs))
+				break;
+
+                        printf("  exception: %lx %s regs %lx\n", regs.trap,
+			       getvecname(regs.trap), sp+112);
+		}
+
+		if (!mread(sp, &newsp, sizeof(unsigned long)))
+			break;
+		if (newsp < sp)
+			break;
+
+		sp = newsp;
+	} while (count++ < xmon_depth_to_print);
+}
+
+static void backtrace(struct pt_regs *excp)
+{
+	unsigned long sp;
+
+	if (excp == NULL)
+		sp = __get_SP();
+	else
+		sp = excp->gpr[1];
+
+	scanhex(&sp);
+	scannl();
+
+	xmon_show_stack(sp);
 }
 
 spinlock_t exception_print_lock = SPIN_LOCK_UNLOCKED;
 
-void
-excprint(struct pt_regs *fp)
+void excprint(struct pt_regs *fp)
 {
 	unsigned long flags;
 
@@ -1021,21 +1000,31 @@ excprint(struct pt_regs *fp)
 	spin_unlock_irqrestore(&exception_print_lock, flags);
 }
 
-void
-prregs(struct pt_regs *fp)
+void prregs(struct pt_regs *fp)
 {
 	int n;
 	unsigned long base;
 
 	if (scanhex((void *)&base))
 		fp = (struct pt_regs *) base;
-	for (n = 0; n < 16; ++n)
-		printf("R%.2ld = %.16lx   R%.2ld = %.16lx\n", n, fp->gpr[n],
-		       n+16, fp->gpr[n+16]);
-	printf("pc  = %.16lx   msr = %.16lx\nlr  = %.16lx   cr  = %.16lx\n",
-	       fp->nip, fp->msr, fp->link, fp->ccr);
-	printf("ctr = %.16lx   xer = %.16lx   trap = %8lx\n",
-	       fp->ctr, fp->xer, fp->trap);
+
+	if (setjmp(bus_error_jmp) == 0) {
+		__debugger_fault_handler = handle_fault;
+		sync();
+		for (n = 0; n < 16; ++n)
+			printf("R%.2ld = %.16lx   R%.2ld = %.16lx\n", n,
+			       fp->gpr[n], n+16, fp->gpr[n+16]);
+		printf("pc  = %.16lx   msr = %.16lx\nlr  = %.16lx   "
+		       "cr  = %.16lx\n", fp->nip, fp->msr, fp->link, fp->ccr);
+		printf("ctr = %.16lx   xer = %.16lx   trap = %8lx\n",
+		       fp->ctr, fp->xer, fp->trap);
+
+		sync();
+		/* wait a little while to see if we get a machine check */
+		__delay(200);
+	} else {
+		printf("*** Error reading regs\n");
+	}
 }
 
 void
@@ -1052,14 +1041,23 @@ cacheflush(void)
 		termch = 0;
 	nflush = 1;
 	scanhex(&nflush);
-	nflush = (nflush + 31) / 32;
-	if (cmd != 'i') {
-		for (; nflush > 0; --nflush, adrs += 0x20)
-			cflush((void *) adrs);
-	} else {
-		for (; nflush > 0; --nflush, adrs += 0x20)
-			cinval((void *) adrs);
+	nflush = (nflush + L1_CACHE_BYTES - 1) / L1_CACHE_BYTES;
+	if (setjmp(bus_error_jmp) == 0) {
+		__debugger_fault_handler = handle_fault;
+		sync();
+
+		if (cmd != 'i') {
+			for (; nflush > 0; --nflush, adrs += L1_CACHE_BYTES)
+				cflush((void *) adrs);
+		} else {
+			for (; nflush > 0; --nflush, adrs += L1_CACHE_BYTES)
+				cinval((void *) adrs);
+		}
+		sync();
+		/* wait a little while to see if we get a machine check */
+		__delay(200);
 	}
+	__debugger_fault_handler = 0;
 }
 
 unsigned long
@@ -1068,6 +1066,7 @@ read_spr(int n)
 	unsigned int instrs[2];
 	unsigned long (*code)(void);
 	unsigned long opd[3];
+	unsigned long ret = -1UL;
 
 	instrs[0] = 0x7c6002a6 + ((n & 0x1F) << 16) + ((n & 0x3e0) << 6);
 	instrs[1] = 0x4e800020;
@@ -1078,7 +1077,22 @@ read_spr(int n)
 	store_inst(instrs+1);
 	code = (unsigned long (*)(void)) opd;
 
-	return code();
+	if (setjmp(bus_error_jmp) == 0) {
+		__debugger_fault_handler = handle_fault;
+		sync();
+
+		ret = code();
+
+		sync();
+		/* wait a little while to see if we get a machine check */
+		__delay(200);
+	} else {
+		printf("*** Error reading spr %x\n", n);
+	}
+
+	__debugger_fault_handler = 0;
+
+	return ret;
 }
 
 void
@@ -1097,7 +1111,20 @@ write_spr(int n, unsigned long val)
 	store_inst(instrs+1);
 	code = (unsigned long (*)(unsigned long)) opd;
 
-	code(val);
+	if (setjmp(bus_error_jmp) == 0) {
+		__debugger_fault_handler = handle_fault;
+		sync();
+
+		code(val);
+
+		sync();
+		/* wait a little while to see if we get a machine check */
+		__delay(200);
+	} else {
+		printf("*** Error writing spr %x\n", n);
+	}
+
+	__debugger_fault_handler = 0;
 }
 
 static unsigned long regno;
@@ -1107,11 +1134,13 @@ extern char dec_exc;
 void
 super_regs()
 {
-	int i, cmd;
+	int cmd;
 	unsigned long val;
-	struct paca_struct*  ptrPaca = NULL;
-	struct ItLpPaca*  ptrLpPaca = NULL;
-	struct ItLpRegSave*  ptrLpRegSave = NULL;
+#ifdef CONFIG_PPC_ISERIES
+	struct paca_struct *ptrPaca = NULL;
+	struct ItLpPaca *ptrLpPaca = NULL;
+	struct ItLpRegSave *ptrLpRegSave = NULL;
+#endif
 
 	cmd = skipbl();
 	if (cmd == '\n') {
@@ -1125,10 +1154,7 @@ super_regs()
 		printf("sp   = %.16lx  sprg3= %.16lx\n", sp, get_sprg3());
 		printf("toc  = %.16lx  dar  = %.16lx\n", toc, get_dar());
 		printf("srr0 = %.16lx  srr1 = %.16lx\n", get_srr0(), get_srr1());
-		printf("asr  = %.16lx\n", mfasr());
-		for (i = 0; i < 8; ++i)
-			printf("sr%.2ld = %.16lx  sr%.2ld = %.16lx\n", i, get_sr(i), i+8, get_sr(i+8));
-
+#ifdef CONFIG_PPC_ISERIES
 		// Dump out relevant Paca data areas.
 		printf("Paca: \n");
 		ptrPaca = get_paca();
@@ -1144,7 +1170,8 @@ super_regs()
 		printf("    Saved Sprg0=%.16lx  Saved Sprg1=%.16lx \n", ptrLpRegSave->xSPRG0, ptrLpRegSave->xSPRG0);
 		printf("    Saved Sprg2=%.16lx  Saved Sprg3=%.16lx \n", ptrLpRegSave->xSPRG2, ptrLpRegSave->xSPRG3);
 		printf("    Saved Msr  =%.16lx  Saved Nia  =%.16lx \n", ptrLpRegSave->xMSR, ptrLpRegSave->xNIA);
-    
+#endif
+
 		return;
 	}
 
@@ -1158,11 +1185,6 @@ super_regs()
 	case 'r':
 		printf("spr %lx = %lx\n", regno, read_spr(regno));
 		break;
-	case 's':
-		val = get_sr(regno);
-		scanhex(&val);
-		set_sr(regno, val);
-		break;
 	case 'm':
 		val = get_msr();
 		scanhex(&val);
@@ -1180,7 +1202,7 @@ mread(unsigned long adrs, void *buf, int size)
 
 	n = 0;
 	if (setjmp(bus_error_jmp) == 0) {
-		debugger_fault_handler = handle_fault;
+		__debugger_fault_handler = handle_fault;
 		sync();
 		p = (char *)adrs;
 		q = (char *)buf;
@@ -1205,7 +1227,7 @@ mread(unsigned long adrs, void *buf, int size)
 		__delay(200);
 		n = size;
 	}
-	debugger_fault_handler = 0;
+	__debugger_fault_handler = 0;
 	return n;
 }
 
@@ -1217,7 +1239,7 @@ mwrite(unsigned long adrs, void *buf, int size)
 
 	n = 0;
 	if (setjmp(bus_error_jmp) == 0) {
-		debugger_fault_handler = handle_fault;
+		__debugger_fault_handler = handle_fault;
 		sync();
 		p = (char *) adrs;
 		q = (char *) buf;
@@ -1244,14 +1266,14 @@ mwrite(unsigned long adrs, void *buf, int size)
 	} else {
 		printf("*** Error writing address %x\n", adrs + n);
 	}
-	debugger_fault_handler = 0;
+	__debugger_fault_handler = 0;
 	return n;
 }
 
 static int fault_type;
 static char *fault_chars[] = { "--", "**", "##" };
 
-static void
+static int
 handle_fault(struct pt_regs *regs)
 {
 	switch (regs->trap) {
@@ -1267,6 +1289,8 @@ handle_fault(struct pt_regs *regs)
 	}
 
 	longjmp(bus_error_jmp, 1);
+
+	return 0;
 }
 
 #define SWAP(a, b, t)	((t) = (a), (a) = (b), (b) = (t))
@@ -1605,9 +1629,22 @@ ppc_inst_dump(unsigned long adr, long count)
 void
 print_address(unsigned long addr)
 {
-	printf("0x%lx", addr);
+	const char *name;
+	char *modname;
+	long size, offset;
+
+	name = kallsyms_lookup(addr, &size, &offset, &modname, tmpstr);
+
+	if (name) {
+		if (modname)
+			printf("0x%lx\t# %s:%s+0x%lx", addr, modname, name, offset);
+		else
+			printf("0x%lx\t# %s+0x%lx", addr, name, offset);
+	} else
+		printf("0x%lx", addr);
 }
 
+
 /*
  * Memory operations - move, set, print differences
  */
@@ -1786,8 +1823,33 @@ unsigned long *vp;
 		return 0;
 	}
 
+	/* skip leading "0x" if any */
+
+	if (c == '0') {
+		c = inchar();
+		if (c == 'x')
+			c = inchar();
+	} else if (c == '$') {
+		int i;
+		for (i=0; i<63; i++) {
+			c = inchar();
+			if (isspace(c)) {
+				termch = c;
+				break;
+			}
+			tmpstr[i] = c;
+		}
+		tmpstr[i++] = 0;
+		*vp = kallsyms_lookup_name(tmpstr);
+		if (!(*vp)) {
+			printf("unknown symbol '%s'\n", tmpstr);
+			return 0;
+		}
+		return 1;
+	}
+
 	d = hexdigit(c);
-	if( d == EOF ){
+	if (d == EOF) {
 		termch = c;
 		return 0;
 	}
@@ -1796,7 +1858,7 @@ unsigned long *vp;
 		v = (v << 4) + d;
 		c = inchar();
 		d = hexdigit(c);
-	} while( d != EOF );
+	} while (d != EOF);
 	termch = c;
 	*vp = v;
 	return 1;
@@ -1871,19 +1933,53 @@ char *str;
 	lineptr = str;
 }
 
+
+static void
+symbol_lookup(void)
+{
+	int type = inchar();
+	unsigned long addr;
+	static char tmp[64];
+
+	switch (type) {
+	case 'a':
+		if (scanhex(&addr)) {
+			printf("%lx: ", addr);
+			xmon_print_symbol("%s\n", addr);
+		}
+		termch = 0;
+		break;
+	case 's':
+		getstring(tmp, 64);
+		if (setjmp(bus_error_jmp) == 0) {
+			__debugger_fault_handler = handle_fault;
+			sync();
+			addr = kallsyms_lookup_name(tmp);
+			if (addr)
+				printf("%s: %lx\n", tmp, addr);
+			else
+				printf("Symbol '%s' not found.\n", tmp);
+			sync();
+		}
+		__debugger_fault_handler = 0;
+		termch = 0;
+		break;
+	}
+}
+
+
 /* xmon version of __print_symbol */
 void __xmon_print_symbol(const char *fmt, unsigned long address)
 {
 	char *modname;
 	const char *name;
 	unsigned long offset, size;
-	char namebuf[128];
 
 	if (setjmp(bus_error_jmp) == 0) {
-		debugger_fault_handler = handle_fault;
+		__debugger_fault_handler = handle_fault;
 		sync();
 		name = kallsyms_lookup(address, &size, &offset, &modname,
-				       namebuf);
+				       tmpstr);
 		sync();
 		/* wait a little while to see if we get a machine check */
 		__delay(200);
@@ -1891,7 +1987,7 @@ void __xmon_print_symbol(const char *fmt, unsigned long address)
 		name = "symbol lookup failed";
 	}
 
-	debugger_fault_handler = 0;
+	__debugger_fault_handler = 0;
 
 	if (!name) {
 		char addrstr[sizeof("0x%lx") + (BITS_PER_LONG*3/10)];
@@ -1919,240 +2015,8 @@ void __xmon_print_symbol(const char *fmt, unsigned long address)
 	}
 }
 
-void
-mem_translate()
+static void debug_trace(void)
 {
-	int c;
-	unsigned long ea, va, vsid, vpn, page, hpteg_slot_primary, hpteg_slot_secondary, primary_hash, i, *steg, esid, stabl;
-	HPTE *  hpte;
-	struct mm_struct * mm;
-	pte_t  *ptep = NULL;
-	void * pgdir;
- 
-	c = inchar();
-	if ((isxdigit(c) && c != 'f' && c != 'd') || c == '\n')
-		termch = c;
-	scanhex((void *)&ea);
-  
-	if ((ea >= KRANGE_START) && (ea <= (KRANGE_START + (1UL<<60)))) {
-		ptep = 0;
-		vsid = get_kernel_vsid(ea);
-		va = ( vsid << 28 ) | ( ea & 0x0fffffff );
-	} else {
-		// if in vmalloc range, use the vmalloc page directory
-		if ( ( ea >= VMALLOC_START ) && ( ea <= VMALLOC_END ) ) {
-			mm = &init_mm;
-			vsid = get_kernel_vsid( ea );
-		}
-		// if in ioremap range, use the ioremap page directory
-		else if ( ( ea >= IMALLOC_START ) && ( ea <= IMALLOC_END ) ) {
-			mm = &ioremap_mm;
-			vsid = get_kernel_vsid( ea );
-		}
-		// if in user range, use the current task's page directory
-		else if ( ( ea >= USER_START ) && ( ea <= USER_END ) ) {
-			mm = current->mm;
-			vsid = get_vsid(mm->context, ea );
-		}
-		pgdir = mm->pgd;
-		va = ( vsid << 28 ) | ( ea & 0x0fffffff );
-		ptep = find_linux_pte( pgdir, ea );
-	}
-
-	vpn = ((vsid << 28) | (((ea) & 0xFFFF000))) >> 12;
-	page = vpn & 0xffff;
-	esid = (ea >> 28)  & 0xFFFFFFFFF;
-
-  // Search the primary group for an available slot
-	primary_hash = ( vsid & 0x7fffffffff ) ^ page;
-	hpteg_slot_primary = ( primary_hash & htab_data.htab_hash_mask ) * HPTES_PER_GROUP;
-	hpteg_slot_secondary = ( ~primary_hash & htab_data.htab_hash_mask ) * HPTES_PER_GROUP;
-
-	printf("ea             : %.16lx\n", ea);
-	printf("esid           : %.16lx\n", esid);
-	printf("vsid           : %.16lx\n", vsid);
-
-	printf("\nSoftware Page Table\n-------------------\n");
-	printf("ptep           : %.16lx\n", ((unsigned long *)ptep));
-	if(ptep) {
-		printf("*ptep          : %.16lx\n", *((unsigned long *)ptep));
-	}
-
-	hpte  = htab_data.htab  + hpteg_slot_primary;
-	printf("\nHardware Page Table\n-------------------\n");
-	printf("htab base      : %.16lx\n", htab_data.htab);
-	printf("slot primary   : %.16lx\n", hpteg_slot_primary);
-	printf("slot secondary : %.16lx\n", hpteg_slot_secondary);
-	printf("\nPrimary Group\n");
-	for (i=0; i<8; ++i) {
-		if ( hpte->dw0.dw0.v != 0 ) {
-			printf("%d: (hpte)%.16lx %.16lx\n", i, hpte->dw0.dword0, hpte->dw1.dword1);
-			printf("          vsid: %.13lx   api: %.2lx  hash: %.1lx\n", 
-			       (hpte->dw0.dw0.avpn)>>5, 
-			       (hpte->dw0.dw0.avpn) & 0x1f,
-			       (hpte->dw0.dw0.h));
-			printf("          rpn: %.13lx \n", (hpte->dw1.dw1.rpn));
-			printf("           pp: %.1lx \n", 
-			       ((hpte->dw1.dw1.pp0)<<2)|(hpte->dw1.dw1.pp));
-			printf("        wimgn: %.2lx  reference: %.1lx  change: %.1lx\n", 
-			       ((hpte->dw1.dw1.w)<<4)|
-			       ((hpte->dw1.dw1.i)<<3)|
-			       ((hpte->dw1.dw1.m)<<2)|
-			       ((hpte->dw1.dw1.g)<<1)|
-			       ((hpte->dw1.dw1.n)<<0),
-			       hpte->dw1.dw1.r, hpte->dw1.dw1.c);
-		}
-		hpte++;
-	}
-
-	printf("\nSecondary Group\n");
-	// Search the secondary group
-	hpte  = htab_data.htab  + hpteg_slot_secondary;
-	for (i=0; i<8; ++i) {
-		if(hpte->dw0.dw0.v) {
-			printf("%d: (hpte)%.16lx %.16lx\n", i, hpte->dw0.dword0, hpte->dw1.dword1);
-			printf("          vsid: %.13lx   api: %.2lx  hash: %.1lx\n", 
-			       (hpte->dw0.dw0.avpn)>>5, 
-			       (hpte->dw0.dw0.avpn) & 0x1f,
-			       (hpte->dw0.dw0.h));
-			printf("          rpn: %.13lx \n", (hpte->dw1.dw1.rpn));
-			printf("           pp: %.1lx \n", 
-			       ((hpte->dw1.dw1.pp0)<<2)|(hpte->dw1.dw1.pp));
-			printf("        wimgn: %.2lx  reference: %.1lx  change: %.1lx\n", 
-			       ((hpte->dw1.dw1.w)<<4)|
-			       ((hpte->dw1.dw1.i)<<3)|
-			       ((hpte->dw1.dw1.m)<<2)|
-			       ((hpte->dw1.dw1.g)<<1)|
-			       ((hpte->dw1.dw1.n)<<0),
-			       hpte->dw1.dw1.r, hpte->dw1.dw1.c);
-		}
-		hpte++;
-	}
-
-	printf("\nHardware Segment Table\n-----------------------\n");
-	stabl = (unsigned long)(KERNELBASE+(_ASR&0xFFFFFFFFFFFFFFFE));
-	steg = (unsigned long *)((stabl) | ((esid & 0x1f) << 7));
-
-	printf("stab base      : %.16lx\n", stabl);
-	printf("slot           : %.16lx\n", steg);
-
-	for (i=0; i<8; ++i) {
-		printf("%d: (ste) %.16lx %.16lx\n", i,
-		       *((unsigned long *)(steg+i*2)),*((unsigned long *)(steg+i*2+1)) );
-	}
-}
-
-void mem_check()
-{
-	unsigned long htab_size_bytes;
-	unsigned long htab_end;
-	unsigned long last_rpn;
-	HPTE *hpte1, *hpte2;
-
-	htab_size_bytes = htab_data.htab_num_ptegs * 128; // 128B / PTEG
-	htab_end = (unsigned long)htab_data.htab + htab_size_bytes;
-	// last_rpn = (naca->physicalMemorySize-1) >> PAGE_SHIFT;
-	last_rpn = 0xfffff;
-
-	printf("\nHardware Page Table Check\n-------------------\n");
-	printf("htab base      : %.16lx\n", htab_data.htab);
-	printf("htab size      : %.16lx\n", htab_size_bytes);
-
-#if 1
-	for(hpte1 = htab_data.htab; hpte1 < (HPTE *)htab_end; hpte1++) {
-		if ( hpte1->dw0.dw0.v != 0 ) {
-			if ( hpte1->dw1.dw1.rpn <= last_rpn ) {
-				for(hpte2 = hpte1+1; hpte2 < (HPTE *)htab_end; hpte2++) {
-					if ( hpte2->dw0.dw0.v != 0 ) {
-						if(hpte1->dw1.dw1.rpn == hpte2->dw1.dw1.rpn) {
-							printf(" Duplicate rpn: %.13lx \n", (hpte1->dw1.dw1.rpn));
-							printf("   hpte1: %16.16lx  *hpte1: %16.16lx %16.16lx\n",
-							       hpte1, hpte1->dw0.dword0, hpte1->dw1.dword1);
-							printf("   hpte2: %16.16lx  *hpte2: %16.16lx %16.16lx\n",
-							       hpte2, hpte2->dw0.dword0, hpte2->dw1.dword1);
-						}
-					}
-				}
-			} else {
-				printf(" Bogus rpn: %.13lx \n", (hpte1->dw1.dw1.rpn));
-				printf("   hpte: %16.16lx  *hpte: %16.16lx %16.16lx\n",
-				       hpte1, hpte1->dw0.dword0, hpte1->dw1.dword1);
-			}
-		}
-	}
-#endif
-	printf("\nDone -------------------\n");
-}
-
-void mem_find_real()
-{
-	unsigned long htab_size_bytes;
-	unsigned long htab_end;
-	unsigned long last_rpn;
-	HPTE *hpte1;
-	unsigned long pa, rpn;
-	int c;
-
-	c = inchar();
-	if ((isxdigit(c) && c != 'f' && c != 'd') || c == '\n')
-		termch = c;
-	scanhex((void *)&pa);
-	rpn = pa >> 12;
-  
-	htab_size_bytes = htab_data.htab_num_ptegs * 128; // 128B / PTEG
-	htab_end = (unsigned long)htab_data.htab + htab_size_bytes;
-	// last_rpn = (naca->physicalMemorySize-1) >> PAGE_SHIFT;
-	last_rpn = 0xfffff;
-
-	printf("\nMem Find RPN\n-------------------\n");
-	printf("htab base      : %.16lx\n", htab_data.htab);
-	printf("htab size      : %.16lx\n", htab_size_bytes);
-
-	for(hpte1 = htab_data.htab; hpte1 < (HPTE *)htab_end; hpte1++) {
-		if ( hpte1->dw0.dw0.v != 0 ) {
-			if ( hpte1->dw1.dw1.rpn == rpn ) {
-				printf(" Found rpn: %.13lx \n", (hpte1->dw1.dw1.rpn));
-				printf("      hpte: %16.16lx  *hpte1: %16.16lx %16.16lx\n",
-				       hpte1, hpte1->dw0.dword0, hpte1->dw1.dword1);
-			}
-		}
-	}
-	printf("\nDone -------------------\n");
-}
-
-void mem_find_vsid()
-{
-	unsigned long htab_size_bytes;
-	unsigned long htab_end;
-	HPTE *hpte1;
-	unsigned long vsid;
-	int c;
-
-	c = inchar();
-	if ((isxdigit(c) && c != 'f' && c != 'd') || c == '\n')
-		termch = c;
-	scanhex((void *)&vsid);
-  
-	htab_size_bytes = htab_data.htab_num_ptegs * 128; // 128B / PTEG
-	htab_end = (unsigned long)htab_data.htab + htab_size_bytes;
-
-	printf("\nMem Find VSID\n-------------------\n");
-	printf("htab base      : %.16lx\n", htab_data.htab);
-	printf("htab size      : %.16lx\n", htab_size_bytes);
-
-	for(hpte1 = htab_data.htab; hpte1 < (HPTE *)htab_end; hpte1++) {
-		if ( hpte1->dw0.dw0.v != 0 ) {
-			if ( ((hpte1->dw0.dw0.avpn)>>5) == vsid ) {
-				printf(" Found vsid: %.16lx \n", ((hpte1->dw0.dw0.avpn) >> 5));
-				printf("       hpte: %16.16lx  *hpte1: %16.16lx %16.16lx\n",
-				       hpte1, hpte1->dw0.dword0, hpte1->dw1.dword1);
-			}
-		}
-	}
-	printf("\nDone -------------------\n");
-}
-
-static void debug_trace(void) {
         unsigned long val, cmd, on;
 
 	cmd = skipbl();
@@ -2199,3 +2063,56 @@ static void debug_trace(void) {
 		cmd = skipbl();
 	}
 }
+
+static void dump_slb(void)
+{
+	int i;
+	unsigned long tmp;
+
+	printf("SLB contents of cpu %d\n", smp_processor_id());
+
+	for (i = 0; i < naca->slb_size; i++) {
+		asm volatile("slbmfee  %0,%1" : "=r" (tmp) : "r" (i));
+		printf("%02d %016lx ", i, tmp);
+
+		asm volatile("slbmfev  %0,%1" : "=r" (tmp) : "r" (i));
+		printf("%016lx\n", tmp);
+	}
+}
+
+static void dump_stab(void)
+{
+	int i;
+	unsigned long *tmp = (unsigned long *)get_paca()->xStab_data.virt;
+
+	printf("Segment table contents of cpu %d\n", smp_processor_id());
+
+	for (i = 0; i < PAGE_SIZE/16; i++) {
+		unsigned long a, b;
+
+		a = *tmp++;
+		b = *tmp++;
+
+		if (a || b) {
+			printf("%03d %016lx ", i, a);
+			printf("%016lx\n", b);
+		}
+	}
+}
+
+void xmon_init(void)
+{
+	__debugger = xmon;
+	__debugger_bpt = xmon_bpt;
+	__debugger_sstep = xmon_sstep;
+	__debugger_iabr_match = xmon_iabr_match;
+	__debugger_dabr_match = xmon_dabr_match;
+}
+
+void dump_segments(void)
+{
+	if (cur_cpu_spec->cpu_features & CPU_FTR_SLB)
+		dump_slb();
+	else
+		dump_stab();
+}
