@@ -46,26 +46,6 @@ struct pci_controller_info *pci_controller_root = NULL;
 /* Each PCI controller found gets a unique index. */
 int pci_num_controllers = 0;
 
-/* Given an 8-bit PCI bus number, this yields the
- * controlling PBM module info.
- *
- * Some explanation is in order here.  The Linux APIs for
- * the PCI subsystem require that the configuration space
- * types are enough to signify PCI configuration space
- * accesses correctly.  This gives us 8-bits for the bus
- * number, however we have multiple PCI controllers on
- * UltraSparc systems.
- *
- * So what we do is give the PCI busses under each controller
- * a unique portion of the 8-bit PCI bus number space.
- * Therefore one can obtain the controller from the bus
- * number.  For example, say PSYCHO PBM-A a subordinate bus
- * space of 0 to 4, and PBM-B has a space of 0 to 2.  PBM-A
- * will use 0 to 4, and PBM-B will use 5 to 7.
- */
-struct pci_pbm_info *pci_bus2pbm[256];
-unsigned char pci_highest_busnum = 0;
-
 /* At boot time the user can give the kernel a command
  * line option which controls if and how PCI devices
  * are reordered at PCI bus probing time.
@@ -205,23 +185,29 @@ void pci_config_write32(u32 *addr, u32 val)
 extern void sabre_init(int, char *);
 extern void psycho_init(int, char *);
 extern void schizo_init(int, char *);
+extern void schizo_plus_init(int, char *);
+extern void tomatillo_init(int, char *);
 
 static struct {
 	char *model_name;
 	void (*init)(int, char *);
-} pci_controller_table[] = {
+} pci_controller_table[] __initdata = {
 	{ "SUNW,sabre", sabre_init },
 	{ "pci108e,a000", sabre_init },
 	{ "pci108e,a001", sabre_init },
 	{ "SUNW,psycho", psycho_init },
 	{ "pci108e,8000", psycho_init },
 	{ "SUNW,schizo", schizo_init },
-	{ "pci108e,8001", schizo_init }
+	{ "pci108e,8001", schizo_init },
+	{ "SUNW,schizo+", schizo_plus_init },
+	{ "pci108e,8002", schizo_plus_init },
+	{ "SUNW,tomatillo", tomatillo_init },
+	{ "pci108e,a801", tomatillo_init },
 };
 #define PCI_NUM_CONTROLLER_TYPES (sizeof(pci_controller_table) / \
 				  sizeof(pci_controller_table[0]))
 
-static void pci_controller_init(char *model_name, int namelen, int node)
+static int __init pci_controller_init(char *model_name, int namelen, int node)
 {
 	int i;
 
@@ -230,15 +216,17 @@ static void pci_controller_init(char *model_name, int namelen, int node)
 			     pci_controller_table[i].model_name,
 			     namelen)) {
 			pci_controller_table[i].init(node, model_name);
-			return;
+			return 1;
 		}
 	}
 	printk("PCI: Warning unknown controller, model name [%s]\n",
 	       model_name);
 	printk("PCI: Ignoring controller...\n");
+
+	return 0;
 }
 
-static int pci_is_controller(char *model_name, int namelen, int node)
+static int __init pci_is_controller(char *model_name, int namelen, int node)
 {
 	int i;
 
@@ -252,37 +240,46 @@ static int pci_is_controller(char *model_name, int namelen, int node)
 	return 0;
 }
 
-/* Is there some PCI controller in the system?  */
-int pcic_present(void)
+static int __init pci_controller_scan(int (*handler)(char *, int, int))
 {
-	char namebuf[16];
+	char namebuf[64];
 	int node;
+	int count = 0;
 
 	node = prom_getchild(prom_root_node);
 	while ((node = prom_searchsiblings(node, "pci")) != 0) {
-		int len, ret;
+		int len;
 
-		len = prom_getproperty(node, "model",
-				       namebuf, sizeof(namebuf));
+		if ((len = prom_getproperty(node, "model", namebuf, sizeof(namebuf))) > 0 ||
+		    (len = prom_getproperty(node, "compatible", namebuf, sizeof(namebuf))) > 0) {
+			int item_len = 0;
 
-		ret = 0;
-		if (len > 0) {
-			ret = pci_is_controller(namebuf, len, node);
-		} else {
-			len = prom_getproperty(node, "compatible",
-					       namebuf, sizeof(namebuf));
-			if (len > 0)
-				ret = pci_is_controller(namebuf, len, node);
+			/* Our value may be a multi-valued string in the
+			 * case of some compatible properties. For sanity,
+			 * only try the first one. */
+
+			while (namebuf[item_len] && len) {
+				len--;
+				item_len++;
+			}
+
+			if (handler(namebuf, item_len, node))
+				count++;
 		}
-		if (ret)
-			return ret;
 
 		node = prom_getsibling(node);
 		if (!node)
 			break;
 	}
 
-	return 0;
+	return count;
+}
+
+
+/* Is there some PCI controller in the system?  */
+int __init pcic_present(void)
+{
+	return pci_controller_scan(pci_is_controller);
 }
 
 /* Find each controller in the system, attach and initialize
@@ -290,33 +287,14 @@ int pcic_present(void)
  * pci_controller_root.  Setup the controller enough such
  * that bus scanning can be done.
  */
-static void pci_controller_probe(void)
+static void __init pci_controller_probe(void)
 {
-	char namebuf[16];
-	int node;
-
 	printk("PCI: Probing for controllers.\n");
-	node = prom_getchild(prom_root_node);
-	while ((node = prom_searchsiblings(node, "pci")) != 0) {
-		int len;
 
-		len = prom_getproperty(node, "model",
-				       namebuf, sizeof(namebuf));
-		if (len > 0)
-			pci_controller_init(namebuf, len, node);
-		else {
-			len = prom_getproperty(node, "compatible",
-					       namebuf, sizeof(namebuf));
-			if (len > 0)
-				pci_controller_init(namebuf, len, node);
-		}
-		node = prom_getsibling(node);
-		if (!node)
-			break;
-	}
+	pci_controller_scan(pci_controller_init);
 }
 
-static void pci_scan_each_controller_bus(void)
+static void __init pci_scan_each_controller_bus(void)
 {
 	struct pci_controller_info *p;
 	unsigned long flags;
@@ -387,10 +365,9 @@ void pcibios_fixup_bus(struct pci_bus *pbus)
 	pbus->resource[1] = &pbm->mem_space;
 }
 
-/* NOTE: This can get called before we've fixed up pdev->sysdata. */
 int pci_claim_resource(struct pci_dev *pdev, int resource)
 {
-	struct pci_pbm_info *pbm = pci_bus2pbm[pdev->bus->number];
+	struct pci_pbm_info *pbm = pdev->bus->sysdata;
 	struct resource *res = &pdev->resource[resource];
 	struct resource *root;
 
@@ -461,7 +438,7 @@ int pci_assign_resource(struct pci_dev *pdev, int resource)
 
 	if (err < 0) {
 		printk("PCI: Failed to allocate resource %d for %s\n",
-		       resource, pdev->dev.name);
+		       resource, pci_name(pdev));
 	} else {
 		/* Update PCI config space. */
 		pbm->parent->base_address_update(pdev, resource);
@@ -488,7 +465,7 @@ void pdev_sort_resources(struct pci_dev *dev, struct resource_list *head)
 		if (!r_align) {
 			printk(KERN_WARNING "PCI: Ignore bogus resource %d "
 					    "[%lx:%lx] of %s\n",
-					    i, r->start, r->end, dev->dev.name);
+					    i, r->start, r->end, pci_name(dev));
 			continue;
 		}
 		r_align = (i < PCI_BRIDGE_RESOURCES) ? r_align + 1 : r->start;
@@ -535,7 +512,7 @@ int pcibios_enable_device(struct pci_dev *pdev, int mask)
 void pcibios_resource_to_bus(struct pci_dev *pdev, struct pci_bus_region *region,
 			     struct resource *res)
 {
-	struct pci_pbm_info *pbm = pci_bus2pbm[pdev->bus->number];
+	struct pci_pbm_info *pbm = pdev->bus->sysdata;
 	struct resource zero_res, *root;
 
 	zero_res.start = 0;
@@ -556,7 +533,7 @@ void pcibios_resource_to_bus(struct pci_dev *pdev, struct pci_bus_region *region
 void pcibios_bus_to_resource(struct pci_dev *pdev, struct resource *res,
 			     struct pci_bus_region *region)
 {
-	struct pci_pbm_info *pbm = pci_bus2pbm[pdev->bus->number];
+	struct pci_pbm_info *pbm = pdev->bus->sysdata;
 	struct resource *root;
 
 	res->start = region->start;
@@ -802,32 +779,33 @@ int pci_mmap_page_range(struct pci_dev *dev, struct vm_area_struct *vma,
 	return 0;
 }
 
-/* Return the index of the PCI controller for device PDEV. */
+/* Return the domain nuber for this pci bus */
 
-int pci_controller_num(struct pci_dev *pdev)
+int pci_domain_nr(struct pci_bus *pbus)
 {
-	struct pcidev_cookie *cookie = pdev->sysdata;
+	struct pci_pbm_info *pbm = pbus->sysdata;
 	int ret;
 
-	if (cookie != NULL) {
-		struct pci_pbm_info *pbm = cookie->pbm;
-		if (pbm == NULL || pbm->parent == NULL) {
-			ret = -ENXIO;
-		} else {
-			struct pci_controller_info *p = pbm->parent;
-
-			ret = p->index;
-			if (p->pbms_same_domain == 0)
-				ret = ((ret << 1) +
-				       ((pbm == &pbm->parent->pbm_B) ? 1 : 0));
-		}
-	} else {
+	if (pbm == NULL || pbm->parent == NULL) {
 		ret = -ENXIO;
+	} else {
+		struct pci_controller_info *p = pbm->parent;
+
+		ret = p->index;
+		if (p->pbms_same_domain == 0)
+			ret = ((ret << 1) +
+			       ((pbm == &pbm->parent->pbm_B) ? 1 : 0));
 	}
 
 	return ret;
 }
 
+int pci_name_bus(char *name, struct pci_bus *bus)
+{
+	sprintf(name, "%04x:%02x", pci_domain_nr(bus), bus->number);
+	return 0;
+}
+
 int pcibios_prep_mwi(struct pci_dev *dev)
 {
 	/* We set correct PCI_CACHE_LINE_SIZE register values for every
