@@ -2,38 +2,38 @@
  * drivers/base/node.c - basic Node class support
  */
 
-#include <linux/device.h>
+#include <linux/sysdev.h>
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/mm.h>
 #include <linux/node.h>
+#include <linux/cpumask.h>
+#include <linux/topology.h>
 
-#include <asm/topology.h>
-
-
-static struct class node_class = {
-	.name		= "node",
+static struct sysdev_class node_class = {
+	set_kset_name("node"),
 };
 
 
-static struct device_driver node_driver = {
-	.name		= "node",
-	.bus		= &system_bus_type,
-};
-
-
-static ssize_t node_read_cpumap(struct device * dev, char * buf)
+static ssize_t node_read_cpumap(struct sys_device * dev, char * buf)
 {
-	struct node *node_dev = to_node(to_root(dev));
-        return sprintf(buf,"%lx\n",node_dev->cpumap);
+	struct node *node_dev = to_node(dev);
+	cpumask_t mask = node_dev->cpumap;
+	int len;
+
+	/* FIXME - someone should pass us a buffer size (count) or
+	 * use seq_file or something to avoid buffer overrun risk. */
+	len = cpumask_scnprintf(buf, 99 /* XXX FIXME */, mask);
+	len += sprintf(buf + len, "\n");
+	return len;
 }
-static DEVICE_ATTR(cpumap,S_IRUGO,node_read_cpumap,NULL);
+
+static SYSDEV_ATTR(cpumap,S_IRUGO,node_read_cpumap,NULL);
 
 #define K(x) ((x) << (PAGE_SHIFT - 10))
-static ssize_t node_read_meminfo(struct device * dev, char * buf)
+static ssize_t node_read_meminfo(struct sys_device * dev, char * buf)
 {
-	struct sys_root *node = to_root(dev);
-	int nid = node->id;
+	int nid = dev->id;
 	struct sysinfo i;
 	si_meminfo_node(&i, nid);
 	return sprintf(buf, "\n"
@@ -53,7 +53,7 @@ static ssize_t node_read_meminfo(struct device * dev, char * buf)
 		       nid, K(i.freeram-i.freehigh));
 }
 #undef K 
-static DEVICE_ATTR(meminfo,S_IRUGO,node_read_meminfo,NULL);
+static SYSDEV_ATTR(meminfo,S_IRUGO,node_read_meminfo,NULL);
 
 
 /*
@@ -67,17 +67,13 @@ int __init register_node(struct node *node, int num, struct node *parent)
 	int error;
 
 	node->cpumap = node_to_cpumask(num);
-	node->sysroot.id = num;
-	if (parent)
-		node->sysroot.dev.parent = &parent->sysroot.sysdev;
-	snprintf(node->sysroot.dev.name, DEVICE_NAME_SIZE, "Node %u", num);
-	snprintf(node->sysroot.dev.bus_id, BUS_ID_SIZE, "node%u", num);
-	node->sysroot.dev.driver = &node_driver;
-	node->sysroot.dev.bus = &system_bus_type;
-	error = sys_register_root(&node->sysroot);
+	node->sysdev.id = num;
+	node->sysdev.cls = &node_class;
+	error = sysdev_register(&node->sysdev);
+
 	if (!error){
-		device_create_file(&node->sysroot.dev, &dev_attr_cpumap);
-		device_create_file(&node->sysroot.dev, &dev_attr_meminfo);
+		sysdev_create_file(&node->sysdev, &attr_cpumap);
+		sysdev_create_file(&node->sysdev, &attr_meminfo);
 	}
 	return error;
 }
@@ -85,14 +81,6 @@ int __init register_node(struct node *node, int num, struct node *parent)
 
 int __init register_node_type(void)
 {
-	int error;
-	
-	error = class_register(&node_class);
-	if (!error) {
-		error = driver_register(&node_driver);
-		if (error)
-			class_unregister(&node_class);
-	}
-	return error;
+	return sysdev_class_register(&node_class);
 }
 postcore_initcall(register_node_type);
