@@ -31,12 +31,11 @@
  *
  * $Id: hci_usb.c,v 1.8 2002/07/18 17:23:09 maxk Exp $    
  */
-#define VERSION "2.4"
+#define VERSION "2.5"
 
 #include <linux/config.h>
 #include <linux/module.h>
 
-#include <linux/version.h>
 #include <linux/kernel.h>
 #include <linux/init.h>
 #include <linux/sched.h>
@@ -56,16 +55,16 @@
 
 #include "hci_usb.h"
 
-#ifndef HCI_USB_DEBUG
+#ifndef CONFIG_BT_HCIUSB_DEBUG
 #undef  BT_DBG
 #define BT_DBG( A... )
 #undef  BT_DMP
 #define BT_DMP( A... )
 #endif
 
-#ifndef CONFIG_BT_USB_ZERO_PACKET
-#undef  USB_ZERO_PACKET
-#define USB_ZERO_PACKET 0
+#ifndef CONFIG_BT_HCIUSB_ZERO_PACKET
+#undef  URB_ZERO_PACKET
+#define URB_ZERO_PACKET 0
 #endif
 
 static struct usb_driver hci_usb_driver; 
@@ -74,9 +73,15 @@ static struct usb_device_id bluetooth_ids[] = {
 	/* Generic Bluetooth USB device */
 	{ USB_DEVICE_INFO(HCI_DEV_CLASS, HCI_DEV_SUBCLASS, HCI_DEV_PROTOCOL) },
 
+	/* AVM BlueFRITZ! USB v2.0 */
+	{ USB_DEVICE(0x057c, 0x3800) },
+
 	/* Ericsson with non-standard id */
 	{ USB_DEVICE(0x0bdb, 0x1002) },
 
+	/* ALPS Module with non-standard id */
+	{ USB_DEVICE(0x044e, 0x3002) },
+
 	/* Bluetooth Ultraport Module from IBM */
 	{ USB_DEVICE(0x04bf, 0x030a) },
 
@@ -85,11 +90,17 @@ static struct usb_device_id bluetooth_ids[] = {
 
 MODULE_DEVICE_TABLE (usb, bluetooth_ids);
 
-static struct usb_device_id ignore_ids[] = {
+static struct usb_device_id blacklist_ids[] = {
 	/* Broadcom BCM2033 without firmware */
-	{ USB_DEVICE(0x0a5c, 0x2033) },
+	{ USB_DEVICE(0x0a5c, 0x2033), .driver_info = HCI_IGNORE },
 
-	{ }	/* Terminating entry */
+	/* Broadcom BCM2035 */
+	{ USB_DEVICE(0x0a5c, 0x200a), .driver_info = HCI_RESET },
+
+	/* Digianswer device */
+	{ USB_DEVICE(0x08fd, 0x0001), .driver_info = HCI_DIGIANSWER },
+
+	{ }     /* Terminating entry */
 };
 
 struct _urb *_urb_alloc(int isoc, int gfp)
@@ -98,8 +109,7 @@ struct _urb *_urb_alloc(int isoc, int gfp)
 				sizeof(struct usb_iso_packet_descriptor) * isoc, gfp);
 	if (_urb) {
 		memset(_urb, 0, sizeof(*_urb));
-		_urb->urb.count = (atomic_t)ATOMIC_INIT(1);
-		spin_lock_init(&_urb->urb.lock);
+		usb_init_urb(&_urb->urb);
 	}
 	return _urb;
 }
@@ -107,8 +117,8 @@ struct _urb *_urb_alloc(int isoc, int gfp)
 struct _urb *_urb_dequeue(struct _urb_queue *q)
 {
 	struct _urb *_urb = NULL;
-        unsigned long flags;
-        spin_lock_irqsave(&q->lock, flags);
+	unsigned long flags;
+	spin_lock_irqsave(&q->lock, flags);
 	{
 		struct list_head *head = &q->head;
 		struct list_head *next = head->next;
@@ -135,7 +145,7 @@ static inline struct _urb *__get_completed(struct hci_usb *husb, int type)
 	return _urb_dequeue(__completed_q(husb, type)); 
 }
 
-#ifdef CONFIG_BT_USB_SCO
+#ifdef CONFIG_BT_HCIUSB_SCO
 static void __fill_isoc_desc(struct urb *urb, int len, int mtu)
 {
 	int offset = 0, i;
@@ -164,9 +174,9 @@ static int hci_usb_intr_rx_submit(struct hci_usb *husb)
 	int err, pipe, interval, size;
 	void *buf;
 
-	BT_DBG("%s", husb->hdev.name);
+	BT_DBG("%s", husb->hdev->name);
 
-        size = husb->intr_in_ep->desc.wMaxPacketSize;
+	size = husb->intr_in_ep->desc.wMaxPacketSize;
 
 	buf = kmalloc(size, GFP_ATOMIC);
 	if (!buf)
@@ -188,7 +198,7 @@ static int hci_usb_intr_rx_submit(struct hci_usb *husb)
 	err = usb_submit_urb(urb, GFP_ATOMIC);
 	if (err) {
 		BT_ERR("%s intr rx submit failed urb %p err %d",
-				husb->hdev.name, urb, err);
+				husb->hdev->name, urb, err);
 		_urb_unlink(_urb);
 		_urb_free(_urb);
 		kfree(buf);
@@ -217,15 +227,15 @@ static int hci_usb_bulk_rx_submit(struct hci_usb *husb)
 
 	urb  = &_urb->urb;
 	pipe = usb_rcvbulkpipe(husb->udev, husb->bulk_in_ep->desc.bEndpointAddress);
-        usb_fill_bulk_urb(urb, husb->udev, pipe, buf, size, hci_usb_rx_complete, husb);
-        urb->transfer_flags = 0;
+	usb_fill_bulk_urb(urb, husb->udev, pipe, buf, size, hci_usb_rx_complete, husb);
+	urb->transfer_flags = 0;
 
-	BT_DBG("%s urb %p", husb->hdev.name, urb);
+	BT_DBG("%s urb %p", husb->hdev->name, urb);
 
 	err = usb_submit_urb(urb, GFP_ATOMIC);
 	if (err) {
 		BT_ERR("%s bulk rx submit failed urb %p err %d",
-				husb->hdev.name, urb, err);
+				husb->hdev->name, urb, err);
 		_urb_unlink(_urb);
 		_urb_free(_urb);
 		kfree(buf);
@@ -233,7 +243,7 @@ static int hci_usb_bulk_rx_submit(struct hci_usb *husb)
 	return err;
 }
 
-#ifdef CONFIG_BT_USB_SCO
+#ifdef CONFIG_BT_HCIUSB_SCO
 static int hci_usb_isoc_rx_submit(struct hci_usb *husb)
 {
 	struct _urb *_urb;
@@ -242,7 +252,7 @@ static int hci_usb_isoc_rx_submit(struct hci_usb *husb)
 	void *buf;
 
 	mtu  = husb->isoc_in_ep->desc.wMaxPacketSize;
-        size = mtu * HCI_MAX_ISOC_FRAMES;
+	size = mtu * HCI_MAX_ISOC_FRAMES;
 
 	buf = kmalloc(size, GFP_ATOMIC);
 	if (!buf)
@@ -263,18 +273,20 @@ static int hci_usb_isoc_rx_submit(struct hci_usb *husb)
 	urb->pipe     = usb_rcvisocpipe(husb->udev, husb->isoc_in_ep->desc.bEndpointAddress);
 	urb->complete = hci_usb_rx_complete;
 
+	urb->interval = husb->isoc_in_ep->desc.bInterval;
+
 	urb->transfer_buffer_length = size;
 	urb->transfer_buffer = buf;
 	urb->transfer_flags  = URB_ISO_ASAP;
 
 	__fill_isoc_desc(urb, size, mtu);
 
-	BT_DBG("%s urb %p", husb->hdev.name, urb);
+	BT_DBG("%s urb %p", husb->hdev->name, urb);
 
 	err = usb_submit_urb(urb, GFP_ATOMIC);
 	if (err) {
 		BT_ERR("%s isoc rx submit failed urb %p err %d",
-				husb->hdev.name, urb, err);
+				husb->hdev->name, urb, err);
 		_urb_unlink(_urb);
 		_urb_free(_urb);
 		kfree(buf);
@@ -302,8 +314,10 @@ static int hci_usb_open(struct hci_dev *hdev)
 		for (i = 0; i < HCI_MAX_BULK_RX; i++)
 			hci_usb_bulk_rx_submit(husb);
 
-#ifdef CONFIG_BT_USB_SCO
-		hci_usb_isoc_rx_submit(husb);
+#ifdef CONFIG_BT_HCIUSB_SCO
+		if (husb->isoc_iface)
+			for (i = 0; i < HCI_MAX_ISOC_RX; i++)
+				hci_usb_isoc_rx_submit(husb);
 #endif
 	} else {
 		clear_bit(HCI_RUNNING, &hdev->flags);
@@ -326,11 +340,19 @@ static int hci_usb_flush(struct hci_dev *hdev)
 	return 0;
 }
 
+static inline void hci_usb_wait_for_urb(struct urb *urb)
+{
+	while (atomic_read(&urb->count) > 1) {
+		current->state = TASK_UNINTERRUPTIBLE;
+		schedule_timeout((5 * HZ + 999) / 1000);
+	}
+}
+
 static void hci_usb_unlink_urbs(struct hci_usb *husb)
 {
 	int i;
 
-	BT_DBG("%s", husb->hdev.name);
+	BT_DBG("%s", husb->hdev->name);
 
 	for (i=0; i < 4; i++) {
 		struct _urb *_urb;
@@ -340,8 +362,9 @@ static void hci_usb_unlink_urbs(struct hci_usb *husb)
 		while ((_urb = _urb_dequeue(&husb->pending_q[i]))) {
 			urb = &_urb->urb;
 			BT_DBG("%s unlinking _urb %p type %d urb %p", 
-					husb->hdev.name, _urb, _urb->type, urb);
+					husb->hdev->name, _urb, _urb->type, urb);
 			usb_unlink_urb(urb);
+			hci_usb_wait_for_urb(urb);
 			_urb_queue_tail(__completed_q(husb, _urb->type), _urb);
 		}
 
@@ -349,7 +372,7 @@ static void hci_usb_unlink_urbs(struct hci_usb *husb)
 		while ((_urb = _urb_dequeue(&husb->completed_q[i]))) {
 			urb = &_urb->urb;
 			BT_DBG("%s freeing _urb %p type %d urb %p",
-					husb->hdev.name, _urb, _urb->type, urb);
+					husb->hdev->name, _urb, _urb->type, urb);
 			if (urb->setup_packet)
 				kfree(urb->setup_packet);
 			if (urb->transfer_buffer)
@@ -390,13 +413,13 @@ static int __tx_submit(struct hci_usb *husb, struct _urb *_urb)
 	struct urb *urb = &_urb->urb;
 	int err;
 
-	BT_DBG("%s urb %p type %d", husb->hdev.name, urb, _urb->type);
+	BT_DBG("%s urb %p type %d", husb->hdev->name, urb, _urb->type);
 	
 	_urb_queue_tail(__pending_q(husb, _urb->type), _urb);
 	err = usb_submit_urb(urb, GFP_ATOMIC);
 	if (err) {
 		BT_ERR("%s tx submit failed urb %p type %d err %d",
-				husb->hdev.name, urb, _urb->type, err);
+				husb->hdev->name, urb, _urb->type, err);
 		_urb_unlink(_urb);
 		_urb_queue_tail(__completed_q(husb, _urb->type), _urb);
 	} else
@@ -412,8 +435,8 @@ static inline int hci_usb_send_ctrl(struct hci_usb *husb, struct sk_buff *skb)
 	struct urb *urb;
 
 	if (!_urb) {
-	       	_urb = _urb_alloc(0, GFP_ATOMIC);
-	       	if (!_urb)
+		_urb = _urb_alloc(0, GFP_ATOMIC);
+		if (!_urb)
 			return -ENOMEM;
 		_urb->type = skb->pkt_type;
 
@@ -425,7 +448,7 @@ static inline int hci_usb_send_ctrl(struct hci_usb *husb, struct sk_buff *skb)
 	} else
 		dr = (void *) _urb->urb.setup_packet;
 
-	dr->bRequestType = HCI_CTRL_REQ;
+	dr->bRequestType = husb->ctrl_req;
 	dr->bRequest = 0;
 	dr->wIndex   = 0;
 	dr->wValue   = 0;
@@ -435,7 +458,7 @@ static inline int hci_usb_send_ctrl(struct hci_usb *husb, struct sk_buff *skb)
 	usb_fill_control_urb(urb, husb->udev, usb_sndctrlpipe(husb->udev, 0),
 		(void *) dr, skb->data, skb->len, hci_usb_tx_complete, husb);
 
-	BT_DBG("%s skb %p len %d", husb->hdev.name, skb, skb->len);
+	BT_DBG("%s skb %p len %d", husb->hdev->name, skb, skb->len);
 	
 	_urb->priv = skb;
 	return __tx_submit(husb, _urb);
@@ -448,8 +471,8 @@ static inline int hci_usb_send_bulk(struct hci_usb *husb, struct sk_buff *skb)
 	int pipe;
 
 	if (!_urb) {
-	       	_urb = _urb_alloc(0, GFP_ATOMIC);
-	       	if (!_urb)
+		_urb = _urb_alloc(0, GFP_ATOMIC);
+		if (!_urb)
 			return -ENOMEM;
 		_urb->type = skb->pkt_type;
 	}
@@ -458,40 +481,42 @@ static inline int hci_usb_send_bulk(struct hci_usb *husb, struct sk_buff *skb)
 	pipe = usb_sndbulkpipe(husb->udev, husb->bulk_out_ep->desc.bEndpointAddress);
 	usb_fill_bulk_urb(urb, husb->udev, pipe, skb->data, skb->len, 
 			hci_usb_tx_complete, husb);
-	urb->transfer_flags = USB_ZERO_PACKET;
+	urb->transfer_flags = URB_ZERO_PACKET;
 
-	BT_DBG("%s skb %p len %d", husb->hdev.name, skb, skb->len);
+	BT_DBG("%s skb %p len %d", husb->hdev->name, skb, skb->len);
 
 	_urb->priv = skb;
 	return __tx_submit(husb, _urb);
 }
 
-#ifdef CONFIG_BT_USB_SCO
+#ifdef CONFIG_BT_HCIUSB_SCO
 static inline int hci_usb_send_isoc(struct hci_usb *husb, struct sk_buff *skb)
 {
 	struct _urb *_urb = __get_completed(husb, skb->pkt_type);
 	struct urb *urb;
-	
+
 	if (!_urb) {
-	       	_urb = _urb_alloc(HCI_MAX_ISOC_FRAMES, GFP_ATOMIC);
-	       	if (!_urb)
+		_urb = _urb_alloc(HCI_MAX_ISOC_FRAMES, GFP_ATOMIC);
+		if (!_urb)
 			return -ENOMEM;
 		_urb->type = skb->pkt_type;
 	}
 
-	BT_DBG("%s skb %p len %d", husb->hdev.name, skb, skb->len);
+	BT_DBG("%s skb %p len %d", husb->hdev->name, skb, skb->len);
 
 	urb = &_urb->urb;
-	
+
 	urb->context  = husb;
 	urb->dev      = husb->udev;
 	urb->pipe     = usb_sndisocpipe(husb->udev, husb->isoc_out_ep->desc.bEndpointAddress);
 	urb->complete = hci_usb_tx_complete;
 	urb->transfer_flags = URB_ISO_ASAP;
 
+	urb->interval = husb->isoc_out_ep->desc.bInterval;
+
 	urb->transfer_buffer = skb->data;
 	urb->transfer_buffer_length = skb->len;
-	
+
 	__fill_isoc_desc(urb, skb->len, husb->isoc_out_ep->desc.wMaxPacketSize);
 
 	_urb->priv = skb;
@@ -504,7 +529,7 @@ static void hci_usb_tx_process(struct hci_usb *husb)
 	struct sk_buff_head *q;
 	struct sk_buff *skb;
 
-	BT_DBG("%s", husb->hdev.name);
+	BT_DBG("%s", husb->hdev->name);
 
 	do {
 		clear_bit(HCI_USB_TX_WAKEUP, &husb->state);
@@ -517,10 +542,10 @@ static void hci_usb_tx_process(struct hci_usb *husb)
 				skb_queue_head(q, skb);
 		}
 
-#ifdef CONFIG_BT_USB_SCO
+#ifdef CONFIG_BT_HCIUSB_SCO
 		/* Process SCO queue */
 		q = __transmit_q(husb, HCI_SCODATA_PKT);
-		if (!atomic_read(__pending_tx(husb, HCI_SCODATA_PKT)) &&
+		if (atomic_read(__pending_tx(husb, HCI_SCODATA_PKT)) < HCI_MAX_ISOC_TX &&
 				(skb = skb_dequeue(q))) {
 			if (hci_usb_send_isoc(husb, skb) < 0)
 				skb_queue_head(q, skb);
@@ -576,7 +601,7 @@ static int hci_usb_send_frame(struct sk_buff *skb)
 		hdev->stat.acl_tx++;
 		break;
 
-#ifdef CONFIG_BT_USB_SCO
+#ifdef CONFIG_BT_HCIUSB_SCO
 	case HCI_SCODATA_PKT:
 		hdev->stat.sco_tx++;
 		break;
@@ -598,9 +623,9 @@ static int hci_usb_send_frame(struct sk_buff *skb)
 
 static inline int __recv_frame(struct hci_usb *husb, int type, void *data, int count)
 {
-	BT_DBG("%s type %d data %p count %d", husb->hdev.name, type, data, count);
+	BT_DBG("%s type %d data %p count %d", husb->hdev->name, type, data, count);
 
-	husb->hdev.stat.byte_rx += count;
+	husb->hdev->stat.byte_rx += count;
 
 	while (count) {
 		struct sk_buff *skb = __reassembly(husb, type);
@@ -626,7 +651,7 @@ static inline int __recv_frame(struct hci_usb *husb, int type, void *data, int c
 				} else
 					return -EILSEQ;
 				break;
-#ifdef CONFIG_BT_USB_SCO
+#ifdef CONFIG_BT_HCIUSB_SCO
 			case HCI_SCODATA_PKT:
 				if (count >= HCI_SCO_HDR_SIZE) {
 					struct hci_sco_hdr *h = data;
@@ -637,13 +662,13 @@ static inline int __recv_frame(struct hci_usb *husb, int type, void *data, int c
 #endif
 			}
 			BT_DBG("new packet len %d", len);
-				
+
 			skb = bt_skb_alloc(len, GFP_ATOMIC);
 			if (!skb) {
-				BT_ERR("%s no memory for the packet", husb->hdev.name);
+				BT_ERR("%s no memory for the packet", husb->hdev->name);
 				return -ENOMEM;
 			}
-			skb->dev = (void *) &husb->hdev;
+			skb->dev = (void *) husb->hdev;
 			skb->pkt_type = type;
 	
 			__reassembly(husb, type) = skb;
@@ -676,22 +701,22 @@ static void hci_usb_rx_complete(struct urb *urb, struct pt_regs *regs)
 {
 	struct _urb *_urb = container_of(urb, struct _urb, urb);
 	struct hci_usb *husb = (void *) urb->context;
-	struct hci_dev *hdev = &husb->hdev;
+	struct hci_dev *hdev = husb->hdev;
 	int    err, count = urb->actual_length;
 
 	BT_DBG("%s urb %p type %d status %d count %d flags %x", hdev->name, urb,
 			_urb->type, urb->status, count, urb->transfer_flags);
 
-	if (!test_bit(HCI_RUNNING, &hdev->flags))
-		return;
-
 	read_lock(&husb->completion_lock);
 
+	if (!test_bit(HCI_RUNNING, &hdev->flags))
+		goto unlock;
+
 	if (urb->status || !count)
 		goto resubmit;
 
 	if (_urb->type == HCI_SCODATA_PKT) {
-#ifdef CONFIG_BT_USB_SCO
+#ifdef CONFIG_BT_HCIUSB_SCO
 		int i;
 		for (i=0; i < urb->number_of_packets; i++) {
 			BT_DBG("desc %d status %d offset %d len %d", i,
@@ -711,7 +736,7 @@ static void hci_usb_rx_complete(struct urb *urb, struct pt_regs *regs)
 		err = __recv_frame(husb, _urb->type, urb->transfer_buffer, count);
 		if (err < 0) { 
 			BT_ERR("%s corrupted packet: type %d count %d",
-					husb->hdev.name, _urb->type, count);
+					husb->hdev->name, _urb->type, count);
 			hdev->stat.err_rx++;
 		}
 	}
@@ -722,6 +747,7 @@ resubmit:
 	BT_DBG("%s urb %p type %d resubmit status %d", hdev->name, urb,
 			_urb->type, err);
 
+unlock:
 	read_unlock(&husb->completion_lock);
 }
 
@@ -729,7 +755,7 @@ static void hci_usb_tx_complete(struct urb *urb, struct pt_regs *regs)
 {
 	struct _urb *_urb = container_of(urb, struct _urb, urb);
 	struct hci_usb *husb = (void *) urb->context;
-	struct hci_dev *hdev = &husb->hdev;
+	struct hci_dev *hdev = husb->hdev;
 
 	BT_DBG("%s urb %p status %d flags %x", hdev->name, urb,
 			urb->status, urb->transfer_flags);
@@ -781,13 +807,22 @@ int hci_usb_probe(struct usb_interface *intf, const struct usb_device_id *id)
 	struct hci_dev *hdev;
 	int i, a, e, size, ifn, isoc_ifnum, isoc_alts;
 
-	BT_DBG("udev %p ifnum %d", udev, ifnum);
+	BT_DBG("udev %p intf %p", udev, intf);
 
-	iface = &udev->actconfig->interface[0];
+	if (!id->driver_info) {
+		const struct usb_device_id *match;
+		match = usb_match_id(intf, blacklist_ids);
+		if (match)
+			id = match;
+	}
 
-	/* Check our black list */
-	if (usb_match_id(intf, ignore_ids))
-		return -EIO;
+	iface = udev->actconfig->interface[0];
+
+	if (id->driver_info & HCI_IGNORE)
+		return -ENODEV;
+
+	if (intf->altsetting->desc.bInterfaceNumber > 0)
+		return -ENODEV;
 
 	/* Check number of endpoints */
 	if (intf->altsetting[0].desc.bNumEndpoints < 3)
@@ -807,7 +842,7 @@ int hci_usb_probe(struct usb_interface *intf, const struct usb_device_id *id)
 
 	ifn = min_t(unsigned int, udev->actconfig->desc.bNumInterfaces, HCI_MAX_IFACE_NUM);
 	for (i = 0; i < ifn; i++) {
-		iface = &udev->actconfig->interface[i];
+		iface = udev->actconfig->interface[i];
 		for (a = 0; a < iface->num_altsetting; a++) {
 			uif = &iface->altsetting[a];
 			for (e = 0; e < uif->desc.bNumEndpoints; e++) {
@@ -826,9 +861,9 @@ int hci_usb_probe(struct usb_interface *intf, const struct usb_device_id *id)
 						bulk_out_ep[i] = ep;
 					break;
 
-#ifdef CONFIG_BT_USB_SCO
+#ifdef CONFIG_BT_HCIUSB_SCO
 				case USB_ENDPOINT_XFER_ISOC:
-					if (ep->desc.wMaxPacketSize < size)
+					if (ep->desc.wMaxPacketSize < size || a > 2)
 						break;
 					size = ep->desc.wMaxPacketSize;
 
@@ -852,7 +887,7 @@ int hci_usb_probe(struct usb_interface *intf, const struct usb_device_id *id)
 		goto done;
 	}
 
-#ifdef CONFIG_BT_USB_SCO
+#ifdef CONFIG_BT_HCIUSB_SCO
 	if (!isoc_in_ep[1] || !isoc_out_ep[1]) {
 		BT_DBG("Isoc endpoints not found");
 		isoc_iface = NULL;
@@ -871,7 +906,12 @@ int hci_usb_probe(struct usb_interface *intf, const struct usb_device_id *id)
 	husb->bulk_in_ep  = bulk_in_ep[0];
 	husb->intr_in_ep  = intr_in_ep[0];
 
-#ifdef CONFIG_BT_USB_SCO
+	if (id->driver_info & HCI_DIGIANSWER)
+		husb->ctrl_req = HCI_DIGI_REQ;
+	else
+		husb->ctrl_req = HCI_CTRL_REQ;
+
+#ifdef CONFIG_BT_HCIUSB_SCO
 	if (isoc_iface) {
 		BT_DBG("isoc ifnum %d alts %d", isoc_ifnum, isoc_alts);
 		if (usb_set_interface(udev, isoc_ifnum, isoc_alts)) {
@@ -894,10 +934,17 @@ int hci_usb_probe(struct usb_interface *intf, const struct usb_device_id *id)
 	}
 
 	/* Initialize and register HCI device */
-	hdev = &husb->hdev;
+	hdev = hci_alloc_dev();
+	if (!hdev) {
+		BT_ERR("Can't allocate HCI device");
+		goto probe_error;
+	}
 
-	hdev->type  = HCI_USB;
+	husb->hdev = hdev;
+
+	hdev->type = HCI_USB;
 	hdev->driver_data = husb;
+	SET_HCIDEV_DEV(hdev, &intf->dev);
 
 	hdev->open  = hci_usb_open;
 	hdev->close = hci_usb_close;
@@ -907,8 +954,12 @@ int hci_usb_probe(struct usb_interface *intf, const struct usb_device_id *id)
 
 	hdev->owner = THIS_MODULE;
 
+	if (id->driver_info & HCI_RESET)
+		set_bit(HCI_QUIRK_RESET_ON_INIT, &hdev->quirks);
+
 	if (hci_register_dev(hdev) < 0) {
 		BT_ERR("Can't register HCI device");
+		hci_free_dev(hdev);
 		goto probe_error;
 	}
 
@@ -925,7 +976,7 @@ done:
 static void hci_usb_disconnect(struct usb_interface *intf)
 {
 	struct hci_usb *husb = usb_get_intfdata(intf);
-	struct hci_dev *hdev = &husb->hdev;
+	struct hci_dev *hdev = husb->hdev;
 
 	if (!husb)
 		return;
@@ -940,16 +991,19 @@ static void hci_usb_disconnect(struct usb_interface *intf)
 
 	if (hci_unregister_dev(hdev) < 0)
 		BT_ERR("Can't unregister HCI device %s", hdev->name);
+
+	hci_free_dev(hdev);
 }
 
 static struct usb_driver hci_usb_driver = {
+	.owner      =  THIS_MODULE,
 	.name       =  "hci_usb",
 	.probe      =  hci_usb_probe,
 	.disconnect =  hci_usb_disconnect,
 	.id_table   =  bluetooth_ids,
 };
 
-int hci_usb_init(void)
+static int __init hci_usb_init(void)
 {
 	int err;
 
@@ -961,14 +1015,15 @@ int hci_usb_init(void)
 	return err;
 }
 
-void hci_usb_cleanup(void)
+static void __exit hci_usb_exit(void)
 {
 	usb_deregister(&hci_usb_driver);
 }
 
 module_init(hci_usb_init);
-module_exit(hci_usb_cleanup);
+module_exit(hci_usb_exit);
 
-MODULE_AUTHOR("Maxim Krasnyansky <maxk@qualcomm.com>");
+MODULE_AUTHOR("Maxim Krasnyansky <maxk@qualcomm.com>, Marcel Holtmann <marcel@holtmann.org>");
 MODULE_DESCRIPTION("Bluetooth HCI USB driver ver " VERSION);
+MODULE_VERSION(VERSION);
 MODULE_LICENSE("GPL");
