@@ -13,9 +13,7 @@
 #include <linux/mm.h>
 #include "agp.h"
 
-
-/* registers */
-#define NVIDIA_0_APBASE		0x10
+/* NVIDIA registers */
 #define NVIDIA_0_APSIZE		0x80
 #define NVIDIA_1_WBC		0xf0
 #define NVIDIA_2_GARTCTRL	0xd0
@@ -26,8 +24,6 @@
 #define NVIDIA_3_APLIMIT	0x54
 
 
-static int agp_try_unsupported __initdata = 0;
-
 static struct _nvidia_private {
 	struct pci_dev *dev_1;
 	struct pci_dev *dev_2;
@@ -61,10 +57,54 @@ static int nvidia_fetch_size(void)
 	return 0;
 }
 
+#define SYSCFG          0xC0010010
+#define IORR_BASE0      0xC0010016
+#define IORR_MASK0      0xC0010017
+#define AMD_K7_NUM_IORR 2
+
+static int nvidia_init_iorr(u32 base, u32 size)
+{
+	u32 base_hi, base_lo;
+	u32 mask_hi, mask_lo;
+	u32 sys_hi, sys_lo;
+	u32 iorr_addr, free_iorr_addr;
+
+	/* Find the iorr that is already used for the base */
+	/* If not found, determine the uppermost available iorr */
+	free_iorr_addr = AMD_K7_NUM_IORR;
+	for(iorr_addr = 0; iorr_addr < AMD_K7_NUM_IORR; iorr_addr++) {
+		rdmsr(IORR_BASE0 + 2 * iorr_addr, base_lo, base_hi);
+		rdmsr(IORR_MASK0 + 2 * iorr_addr, mask_lo, mask_hi);
+
+		if ((base_lo & 0xfffff000) == (base & 0xfffff000))
+			break;
+
+		if ((mask_lo & 0x00000800) == 0)
+			free_iorr_addr = iorr_addr;
+	}
+	
+	if (iorr_addr >= AMD_K7_NUM_IORR) {
+		iorr_addr = free_iorr_addr;
+		if (iorr_addr >= AMD_K7_NUM_IORR)
+			return -EINVAL;
+	}
+    base_hi = 0x0;
+    base_lo = (base & ~0xfff) | 0x18;
+    mask_hi = 0xf;
+    mask_lo = ((~(size - 1)) & 0xfffff000) | 0x800;
+    wrmsr(IORR_BASE0 + 2 * iorr_addr, base_lo, base_hi);
+    wrmsr(IORR_MASK0 + 2 * iorr_addr, mask_lo, mask_hi);
+
+    rdmsr(SYSCFG, sys_lo, sys_hi);
+    sys_lo |= 0x00100000;
+    wrmsr(SYSCFG, sys_lo, sys_hi);
+
+	return 0;
+}
 
 static int nvidia_configure(void)
 {
-	int i, num_dirs;
+	int i, rc, num_dirs;
 	u32 apbase, aplimit;
 	struct aper_size_info_8 *current_size;
 	u32 temp;
@@ -76,7 +116,7 @@ static int nvidia_configure(void)
 		current_size->size_value);
 
     /* address to map to */
-	pci_read_config_dword(agp_bridge->dev, NVIDIA_0_APBASE, &apbase);
+	pci_read_config_dword(agp_bridge->dev, AGP_APBASE, &apbase);
 	apbase &= PCI_BASE_ADDRESS_MEM_MASK;
 	agp_bridge->gart_bus_addr = apbase;
 	aplimit = apbase + (current_size->size * 1024 * 1024) - 1;
@@ -84,6 +124,8 @@ static int nvidia_configure(void)
 	pci_write_config_dword(nvidia_private.dev_2, NVIDIA_2_APLIMIT, aplimit);
 	pci_write_config_dword(nvidia_private.dev_3, NVIDIA_3_APBASE, apbase);
 	pci_write_config_dword(nvidia_private.dev_3, NVIDIA_3_APLIMIT, aplimit);
+	if (0 != (rc = nvidia_init_iorr(apbase, current_size->size * 1024 * 1024)))
+		return rc;
 
 	/* directory size is 64k */
 	num_dirs = current_size->size / 64;
@@ -137,6 +179,10 @@ static void nvidia_cleanup(void)
 	previous_size = A_SIZE_8(agp_bridge->previous_size);
 	pci_write_config_byte(agp_bridge->dev, NVIDIA_0_APSIZE,
 		previous_size->size_value);
+
+	/* restore iorr for previous aperture size */
+	nvidia_init_iorr(agp_bridge->gart_bus_addr,
+		previous_size->size * 1024 * 1024);
 }
 
 
@@ -211,7 +257,7 @@ static void nvidia_tlbflush(struct agp_memory *mem)
 			pci_read_config_dword(nvidia_private.dev_1,
 					NVIDIA_1_WBC, &wbc_reg);
 			if ((signed)(end - jiffies) <= 0) {
-				printk(KERN_ERR
+				printk(KERN_ERR PFX
 				    "TLB flush took more than 3 seconds.\n");
 			}
 		} while (wbc_reg & nvidia_private.wbc_mask);
@@ -265,8 +311,8 @@ struct agp_bridge_driver nvidia_driver = {
 	.agp_destroy_page	= agp_generic_destroy_page,
 };
 
-static int __init agp_nvidia_probe(struct pci_dev *pdev,
-				   const struct pci_device_id *ent)
+static int __devinit agp_nvidia_probe(struct pci_dev *pdev,
+				      const struct pci_device_id *ent)
 {
 	struct agp_bridge_data *bridge;
 	u8 cap_ptr;
@@ -279,9 +325,8 @@ static int __init agp_nvidia_probe(struct pci_dev *pdev,
 		pci_find_slot((unsigned int)pdev->bus->number, PCI_DEVFN(30, 0));
 	
 	if (!nvidia_private.dev_1 || !nvidia_private.dev_2 || !nvidia_private.dev_3) {
-		printk(KERN_INFO PFX "agpgart: Detected an NVIDIA "
-			"nForce/nForce2 chipset, but could not find "
-			"the secondary devices.\n");
+		printk(KERN_INFO PFX "Detected an NVIDIA nForce/nForce2 "
+			"chipset, but could not find the secondary devices.\n");
 		return -ENODEV;
 	}
 
@@ -299,17 +344,9 @@ static int __init agp_nvidia_probe(struct pci_dev *pdev,
 		nvidia_private.wbc_mask = 0x80000000;
 		break;
 	default:
-		if (!agp_try_unsupported) {
-			printk(KERN_ERR PFX
-			    "Unsupported NVIDIA chipset (device id: %04x),"
-			    " you might want to try agp_try_unsupported=1.\n",
+		printk(KERN_ERR PFX "Unsupported NVIDIA chipset (device id: %04x)\n",
 			    pdev->device);
-			return -ENODEV;
-		}
-		printk(KERN_WARNING PFX
-		    "Trying generic NVIDIA routines for device id: %04x\n",
-		    pdev->device);
-		break;
+		return -ENODEV;
 	}
 
 	bridge = agp_alloc_bridge();
@@ -338,7 +375,7 @@ static void __devexit agp_nvidia_remove(struct pci_dev *pdev)
 	agp_put_bridge(bridge);
 }
 
-static struct pci_device_id agp_nvidia_pci_table[] __initdata = {
+static struct pci_device_id agp_nvidia_pci_table[] = {
 	{
 	.class		= (PCI_CLASS_BRIDGE_HOST << 8),
 	.class_mask	= ~0,
@@ -372,7 +409,6 @@ static void __exit agp_nvidia_cleanup(void)
 module_init(agp_nvidia_init);
 module_exit(agp_nvidia_cleanup);
 
-MODULE_PARM(agp_try_unsupported, "1i");
 MODULE_LICENSE("GPL and additional rights");
 MODULE_AUTHOR("NVIDIA Corporation");
 
