@@ -25,10 +25,6 @@
 */
 
 #include <linux/config.h>
-#ifdef CONFIG_I2C_DEBUG_BUS
-#define DEBUG	1
-#endif
-
 #include <linux/config.h>
 #include <linux/module.h>
 #include <linux/errno.h>
