@@ -147,8 +147,6 @@ int pdc4030_identify(ide_drive_t *drive)
 	return pdc4030_cmd(drive, PROMISE_IDENTIFY);
 }
 
-int enable_promise_support;
-
 /*
  * setup_pdc4030()
  * Completes the setup of a Promise DC4030 controller card, once found.
@@ -227,7 +225,7 @@ int __init setup_pdc4030(ide_hwif_t *hwif)
 	hwif2->mate	= hwif;
 	hwif2->channel	= 1;
 	hwif->rqsize	= hwif2->rqsize = 127;
-	hwif->addressing = hwif2->addressing = 1;
+	hwif->no_lba48 = hwif2->no_lba48 = 1;
 	hwif->selectproc = hwif2->selectproc = &promise_selectproc;
 	hwif->serialized = hwif2->serialized = 1;
 	/* DC4030 hosted drives need their own identify... */
@@ -296,113 +294,30 @@ int __init detect_pdc4030(ide_hwif_t *hwif)
 	}
 }
 
-
-#ifndef MODULE
-void __init ide_probe_for_pdc4030(void)
-#else
-int ide_probe_for_pdc4030(void)
-#endif
+int __init pdc4030_init(void)
 {
 	unsigned int	index;
 	ide_hwif_t	*hwif;
 
-#ifndef MODULE
-	if (enable_promise_support == 0)
-		return;
-#endif
-
 	for (index = 0; index < MAX_HWIFS; index++) {
 		hwif = &ide_hwifs[index];
 		if (hwif->chipset == ide_unknown && detect_pdc4030(hwif)) {
-#ifndef MODULE
-			setup_pdc4030(hwif);
-#else
-			return setup_pdc4030(hwif);
-#endif
+			if (!setup_pdc4030(hwif))
+				return -ENODEV;
+			return 0;
 		}
 	}
+	return -ENODEV;
+}
+
 #ifdef MODULE
-	return 0;
+module_init(pdc4030_init);
 #endif
-}
-
-void __init release_pdc4030(ide_hwif_t *hwif, ide_hwif_t *mate)
-{
-	hwif->chipset = ide_unknown;
-	hwif->selectproc = NULL;
-	hwif->serialized = 0;
-	hwif->drives[0].io_32bit = 0;
-	hwif->drives[1].io_32bit = 0;
-	hwif->drives[0].keep_settings = 0;
-	hwif->drives[1].keep_settings = 0;
-	hwif->drives[0].noprobe = 0;
-	hwif->drives[1].noprobe = 0;
-
-	if (mate != NULL) {
-		mate->chipset = ide_unknown;
-		mate->selectproc = NULL;
-		mate->serialized = 0;
-		mate->drives[0].io_32bit = 0;
-		mate->drives[1].io_32bit = 0;
-		mate->drives[0].keep_settings = 0;
-		mate->drives[1].keep_settings = 0;
-		mate->drives[0].noprobe = 0;
-		mate->drives[1].noprobe = 0;
-	}
-}
-
-#ifndef MODULE
-/*
- * init_pdc4030:
- *
- * called by ide.c when parsing command line
- */
-
-void __init init_pdc4030(void)
-{
-	enable_promise_support = 1;
-}
-
-#else
 
 MODULE_AUTHOR("Peter Denison");
 MODULE_DESCRIPTION("Support of Promise 4030 VLB series IDE chipsets");
 MODULE_LICENSE("GPL");
 
-int __init pdc4030_mod_init(void)
-{
-	if (enable_promise_support == 0)
-		enable_promise_support = 1;
-
-	if (!ide_probe_for_pdc4030())
-		return -ENODEV;
-        return 0;
-}
-module_init(pdc4030_mod_init);
-
-void __init pdc4030_mod_exit(void)
-{
-	unsigned int    index;
-	ide_hwif_t      *hwif;
-
-	if (enable_promise_support == 0)
-		return;
- 
-	for (index = 0; index < MAX_HWIFS; index++) {
-		hwif = &ide_hwifs[index];
-		if (hwif->chipset == ide_pdc4030) {
-			ide_hwif_t *mate = &ide_hwifs[hwif->index+1];
-			if (mate->chipset == ide_pdc4030)
-				release_pdc4030(hwif, mate);
-			else
-				release_pdc4030(hwif, NULL);
-                }
-        }
-	enable_promise_support = 0;
-}
-module_exit(pdc4030_mod_exit);
-#endif
-
 /*
  * promise_read_intr() is the handler for disk read/multread interrupts
  */
@@ -506,7 +421,12 @@ read_next:
 static ide_startstop_t promise_complete_pollfunc(ide_drive_t *drive)
 {
 	ide_hwgroup_t *hwgroup = HWGROUP(drive);
+#ifdef CONFIG_IDE_TASKFILE_IO
 	struct request *rq = hwgroup->rq;
+#else
+	struct request *rq = &hwgroup->wrq;
+	struct bio *bio = rq->bio;
+#endif
 
 	if ((HWIF(drive)->INB(IDE_STATUS_REG)) & BUSY_STAT) {
 		if (time_before(jiffies, hwgroup->poll_timeout)) {
@@ -535,6 +455,8 @@ static ide_startstop_t promise_complete_pollfunc(ide_drive_t *drive)
 	while (rq->bio != rq->cbio)
 		(void) DRIVER(drive)->end_request(drive, 1, bio_sectors(rq->bio));
 #else
+	bio->bi_idx = bio->bi_vcnt - rq->nr_cbio_segments;
+	rq = hwgroup->rq;
 	DRIVER(drive)->end_request(drive, 1, rq->hard_nr_sectors);
 #endif
 	return ide_stopped;
@@ -593,7 +515,7 @@ static void promise_multwrite (ide_drive_t *drive, unsigned int mcount)
 			 * all bvecs in this one.
 			 */
 			if (++bio->bi_idx >= bio->bi_vcnt) {
-				bio->bi_idx = 0;
+				bio->bi_idx = bio->bi_vcnt - rq->nr_cbio_segments;
 				bio = bio->bi_next;
 			}
 
@@ -602,7 +524,8 @@ static void promise_multwrite (ide_drive_t *drive, unsigned int mcount)
 				mcount = 0;
 			} else {
 				rq->bio = bio;
-				rq->current_nr_sectors = bio_iovec(bio)->bv_len >> 9;
+				rq->nr_cbio_segments = bio_segments(bio);
+				rq->current_nr_sectors = bio_cur_sectors(bio);
 				rq->hard_cur_sectors = rq->current_nr_sectors;
 			}
 		}
@@ -613,8 +536,6 @@ static void promise_multwrite (ide_drive_t *drive, unsigned int mcount)
 		 */
 		taskfile_output_data(drive, buffer, nsect<<7);
 	} while (mcount);
-
-	return 0;
 }
 #endif
 
@@ -624,7 +545,12 @@ static void promise_multwrite (ide_drive_t *drive, unsigned int mcount)
 static ide_startstop_t promise_write_pollfunc (ide_drive_t *drive)
 {
 	ide_hwgroup_t *hwgroup = HWGROUP(drive);
+#ifdef CONFIG_IDE_TASKFILE_IO
 	struct request *rq = hwgroup->rq;
+#else
+	struct request *rq = &hwgroup->wrq;
+	struct bio *bio = rq->bio;
+#endif
 
 	if (HWIF(drive)->INB(IDE_NSECTOR_REG) != 0) {
 		if (time_before(jiffies, hwgroup->poll_timeout)) {
@@ -638,6 +564,9 @@ static ide_startstop_t promise_write_pollfunc (ide_drive_t *drive)
 		}
 		hwgroup->poll_timeout = 0;
 		printk(KERN_ERR "%s: write timed-out!\n",drive->name);
+#ifndef CONFIG_IDE_TASKFILE_IO
+		bio->bi_idx = bio->bi_vcnt - rq->nr_cbio_segments;
+#endif
 		return DRIVER(drive)->error(drive, "write timeout",
 				HWIF(drive)->INB(IDE_STATUS_REG));
 	}
@@ -818,7 +747,8 @@ static ide_startstop_t promise_rw_disk (ide_drive_t *drive, struct request *rq,
 	   Feature register.
 	   FIXME: Is promise_selectproc now redundant??
 	*/
-	int drive_number = (HWIF(drive)->channel << 1) + drive->select.b.unit;
+	ide_hwif_t *hwif = HWIF(drive);
+	int drive_number = (hwif->channel << 1) + drive->select.b.unit;
 #ifdef CONFIG_IDE_TASKFILE_IO
 	struct hd_drive_task_hdr taskfile;
 	ide_task_t args;
@@ -862,11 +792,10 @@ static ide_startstop_t promise_rw_disk (ide_drive_t *drive, struct request *rq,
 
 	memcpy(args.tfRegister, &taskfile, sizeof(struct hd_drive_task_hdr));
 	memset(args.hobRegister, 0, sizeof(struct hd_drive_hob_hdr));
-	/* We can't call ide_cmd_type_parser here, since it won't understand
-	   our command, but that doesn't matter, since we don't use the
-	   generic interrupt handlers either. Setup the bits of args that we
-	   do need.
-	*/
+	/*
+	 * Setup the bits of args that we do need.
+	 * Note that we don't use the generic interrupt handlers.
+	 */
 	args.handler		= NULL;
 	args.rq			= (struct request *) rq;
 	rq->special		= (ide_task_t *)&args;
