@@ -319,7 +319,7 @@ void ohci1394_stop_it_ctx(struct ti_ohci *ohci, int ctx, int synchronous)
 			control = reg_read(ohci, OHCI1394_IsoXmitContextControlSet + ctx * 16);
 			if ((control & OHCI1394_CONTEXT_ACTIVE) == 0)
 				break;
-			
+
 			set_current_state(TASK_INTERRUPTIBLE);
 			schedule_timeout(1);
 		}
@@ -408,7 +408,7 @@ static void stream_shift_packet_lists(unsigned long l)
 
 	/* Now that we know the list is non-empty, we can get the head
 	 * of the list without locking, because the process context
-	 * only adds to the tail.  
+	 * only adds to the tail.
 	 */
 	pl = list_entry(s->dma_packet_lists.next, struct packet_list, link);
 	last = &pl->packets[PACKET_LIST_SIZE - 1];
@@ -424,7 +424,7 @@ static void stream_shift_packet_lists(unsigned long l)
 	if (last->db->payload_desc.status == 0) {
 		HPSB_INFO("weird interrupt...");
 		return;
-	}		
+	}
 
 	/* If the last descriptor block does not specify a branch
 	 * address, we have a sample underflow.
@@ -469,7 +469,7 @@ static struct packet *stream_current_packet(struct stream *s)
 
 	return &s->current_packet_list->packets[s->current_packet];
 }
-	
+
 static void stream_queue_packet(struct stream *s)
 {
 	s->current_packet++;
@@ -543,13 +543,13 @@ void packet_initialize(struct packet *p, struct packet *next)
 		DMA_CTL_OUTPUT_MORE | DMA_CTL_IMMEDIATE | 8;
 
 	if (next) {
-		p->db->payload_desc.control = 
+		p->db->payload_desc.control =
 			DMA_CTL_OUTPUT_LAST | DMA_CTL_BRANCH;
 		p->db->payload_desc.branch = next->db_bus | 3;
 		p->db->header_desc.skip = next->db_bus | 3;
 	}
 	else {
-		p->db->payload_desc.control = 
+		p->db->payload_desc.control =
 			DMA_CTL_OUTPUT_LAST | DMA_CTL_BRANCH |
 			DMA_CTL_UPDATE | DMA_CTL_IRQ;
 		p->db->payload_desc.branch = 0;
@@ -580,7 +580,7 @@ struct packet_list *packet_list_alloc(struct stream *s)
 	for (i = 0; i < PACKET_LIST_SIZE; i++) {
 		if (i < PACKET_LIST_SIZE - 1)
 			next = &pl->packets[i + 1];
-		else 
+		else
 			next = NULL;
 		packet_initialize(&pl->packets[i], next);
 	}
@@ -695,7 +695,7 @@ static u32 get_header_bits(struct stream *s, int sub_frame, u32 sample)
 	case AMDTP_FORMAT_IEC958_PCM:
 	case AMDTP_FORMAT_IEC958_AC3:
 		return get_iec958_header_bits(s, sub_frame, sample);
-		
+
 	case AMDTP_FORMAT_RAW:
 		return 0x40;
 
@@ -739,18 +739,18 @@ static void fill_packet(struct stream *s, struct packet *packet, int nevents)
 
 	/* Fill IEEE1394 headers */
 	packet->db->header_desc.header[0] =
-		(IEEE1394_SPEED_100 << 16) | (0x01 << 14) | 
+		(IEEE1394_SPEED_100 << 16) | (0x01 << 14) |
 		(s->iso_channel << 8) | (TCODE_ISO_DATA << 4);
 	packet->db->header_desc.header[1] = size << 16;
-	
+
 	/* Calculate synchronization timestamp (syt). First we
 	 * determine syt_index, that is, the index in the packet of
 	 * the sample for which the timestamp is valid. */
 	syt_index = (s->syt_interval - s->dbc) & (s->syt_interval - 1);
 	if (syt_index < nevents) {
-		syt = ((atomic_read(&s->cycle_count) << 12) | 
+		syt = ((atomic_read(&s->cycle_count) << 12) |
 		       s->cycle_offset.integer) & 0xffff;
-		fraction_add(&s->cycle_offset, 
+		fraction_add(&s->cycle_offset,
 			     &s->cycle_offset, &s->ticks_per_syt_offset);
 
 		/* This next addition should be modulo 8000 (0x1f40),
@@ -763,7 +763,7 @@ static void fill_packet(struct stream *s, struct packet *packet, int nevents)
 		syt = 0xffff;
 
 	atomic_inc(&s->cycle_count2);
-	
+
 	/* Fill cip header */
 	packet->payload->eoh0 = 0;
 	packet->payload->sid = s->host->host->node_id & 0x3f;
@@ -1072,7 +1072,7 @@ void stream_free(struct stream *s)
 	 * that sometimes generates an it transmit interrupt if we
 	 * later re-enable the context.
 	 */
-	wait_event_interruptible(s->packet_list_wait, 
+	wait_event_interruptible(s->packet_list_wait,
 				 list_empty(&s->dma_packet_lists));
 
 	ohci1394_stop_it_ctx(s->host->ohci, s->iso_tasklet.context, 1);
@@ -1102,7 +1102,7 @@ static ssize_t amdtp_write(struct file *file, const char *buffer, size_t count,
 	unsigned char *p;
 	int i;
 	size_t length;
-	
+
 	if (s->packet_pool == NULL)
 		return -EBADFD;
 
@@ -1123,16 +1123,16 @@ static ssize_t amdtp_write(struct file *file, const char *buffer, size_t count,
 			return -EFAULT;
 		if (s->input->length < s->input->size)
 			continue;
-		
+
 		stream_flush(s);
-		
+
 		if (s->current_packet_list != NULL)
 			continue;
 
 		if (file->f_flags & O_NONBLOCK)
 			return i + length > 0 ? i + length : -EAGAIN;
 
-		if (wait_event_interruptible(s->packet_list_wait, 
+		if (wait_event_interruptible(s->packet_list_wait,
 					     !list_empty(&s->free_packet_lists)))
 			return -EINTR;
 	}
@@ -1152,7 +1152,7 @@ static int amdtp_ioctl(struct inode *inode, struct file *file,
 	case AMDTP_IOC_CHANNEL:
 		if (copy_from_user(&cfg, (struct amdtp_ioctl *) arg, sizeof cfg))
 			return -EFAULT;
-		else 
+		else
 			return stream_configure(s, cmd, &cfg);
 
 	default:
@@ -1266,6 +1266,7 @@ static int __init amdtp_init_module (void)
 {
 	cdev_init(&amdtp_cdev, &amdtp_fops);
 	amdtp_cdev.owner = THIS_MODULE;
+	kobject_set_name(&amdtp_cdev.kobj, "amdtp");
 	if (cdev_add(&amdtp_cdev, IEEE1394_AMDTP_DEV, 16)) {
 		HPSB_ERR("amdtp: unable to add char device");
  		return -EIO;
