@@ -72,7 +72,6 @@
 #include <linux/netdevice.h>
 #include <linux/etherdevice.h>
 #include <linux/skbuff.h>
-#include <linux/version.h>
 #include <linux/module.h>
 
 #include <asm/bitops.h>
@@ -245,6 +244,7 @@ struct priv
 	int cmdr_addr;
 	int cardno;
 	int features;
+	spinlock_t ring_lock;
 };
 
 static int  ni65_probe1(struct net_device *dev,int);
@@ -299,7 +299,7 @@ static int ni65_open(struct net_device *dev)
 	int irqval = request_irq(dev->irq, &ni65_interrupt,0,
                         cards[p->cardno].cardname,dev);
 	if (irqval) {
-		printk ("%s: unable to get IRQ %d (irqval=%d).\n",
+		printk(KERN_ERR "%s: unable to get IRQ %d (irqval=%d).\n",
 		          dev->name,dev->irq, irqval);
 		return -EAGAIN;
 	}
@@ -343,29 +343,64 @@ static int ni65_close(struct net_device *dev)
 	return 0;
 }
 
+static void cleanup_card(struct net_device *dev)
+{
+	struct priv *p = (struct priv *) dev->priv;
+	disable_dma(dev->dma);
+	free_dma(dev->dma);
+	release_region(dev->base_addr, cards[p->cardno].total_size);
+	ni65_free_buffer(p);
+}
+
+/* set: io,irq,dma or set it when calling insmod */
+static int irq;
+static int io;
+static int dma;
+
 /*
  * Probe The Card (not the lance-chip)
  */
-#ifdef MODULE
-static
-#endif
-int __init ni65_probe(struct net_device *dev)
+struct net_device * __init ni65_probe(int unit)
 {
-	int *port;
+	struct net_device *dev = alloc_etherdev(0);
 	static int ports[] = {0x360,0x300,0x320,0x340, 0};
+	int *port;
+	int err = 0;
 
-	if (dev->base_addr > 0x1ff)          /* Check a single specified location. */
-		 return ni65_probe1(dev, dev->base_addr);
-	else if (dev->base_addr > 0)         /* Don't probe at all. */
-		 return -ENXIO;
+	if (!dev)
+		return ERR_PTR(-ENOMEM);
 
-	for (port = ports; *port; port++)
-	{
-		if (ni65_probe1(dev, *port) == 0)
-			 return 0;
+	if (unit >= 0) {
+		sprintf(dev->name, "eth%d", unit);
+		netdev_boot_setup_check(dev);
+		irq = dev->irq;
+		dma = dev->dma;
+	} else {
+		dev->base_addr = io;
 	}
 
-	return -ENODEV;
+	if (dev->base_addr > 0x1ff) { /* Check a single specified location. */
+		err = ni65_probe1(dev, dev->base_addr);
+	} else if (dev->base_addr > 0) { /* Don't probe at all. */
+		err = -ENXIO;
+	} else {
+		for (port = ports; *port && ni65_probe1(dev, *port); port++)
+			;
+		if (!*port)
+			err = -ENODEV;
+	}
+	if (err)
+		goto out;
+
+	err = register_netdev(dev);
+	if (err)
+		goto out1;
+	return dev;
+out1:
+	cleanup_card(dev);
+out:
+	free_netdev(dev);
+	return ERR_PTR(err);
 }
 
 /*
@@ -377,6 +412,9 @@ static int __init ni65_probe1(struct net_device *dev,int ioaddr)
 	struct priv *p;
 	unsigned long flags;
 
+	dev->irq = irq;
+	dev->dma = dma;
+
 	for(i=0;i<NUM_CARDS;i++) {
 		if(!request_region(ioaddr, cards[i].total_size, cards[i].cardname))
 			continue;
@@ -409,12 +447,14 @@ static int __init ni65_probe1(struct net_device *dev,int ioaddr)
 	p = (struct priv *) dev->priv;
 	p->cmdr_addr = ioaddr + cards[i].cmd_offset;
 	p->cardno = i;
+	spin_lock_init(&p->ring_lock);
 
-	printk("%s: %s found at %#3x, ", dev->name, cards[p->cardno].cardname , ioaddr);
+	printk(KERN_INFO "%s: %s found at %#3x, ", dev->name, cards[p->cardno].cardname , ioaddr);
 
 	outw(inw(PORT+L_RESET),PORT+L_RESET); /* first: reset the card */
 	if( (j=readreg(CSR0)) != 0x4) {
-		 printk(KERN_ERR "can't RESET card: %04x\n",j);
+		 printk("failed.\n");
+		 printk(KERN_ERR "%s: Can't RESET card: %04x\n", dev->name, j);
 		 ni65_free_buffer(p);
 		 release_region(ioaddr, cards[p->cardno].total_size);
 		 return -EAGAIN;
@@ -467,7 +507,8 @@ static int __init ni65_probe1(struct net_device *dev,int ioaddr)
 					break;
 			}
 			if(i == 5) {
-				printk("Can't detect DMA channel!\n");
+				printk("failed.\n");
+				printk(KERN_ERR "%s: Can't detect DMA channel!\n", dev->name);
 				ni65_free_buffer(p);
 				release_region(ioaddr, cards[p->cardno].total_size);
 				return -EAGAIN;
@@ -480,13 +521,13 @@ static int __init ni65_probe1(struct net_device *dev,int ioaddr)
 
 		if(dev->irq < 2)
 		{
-			unsigned long irq_mask, delay;
+			unsigned long irq_mask;
 
 			ni65_init_lance(p,dev->dev_addr,0,0);
 			irq_mask = probe_irq_on();
 			writereg(CSR0_INIT|CSR0_INEA,CSR0); /* trigger interrupt */
-			delay = jiffies + HZ/50;
-			while (time_before(jiffies, delay)) ;
+			set_current_state(TASK_UNINTERRUPTIBLE);
+			schedule_timeout(HZ/50);
 			dev->irq = probe_irq_off(irq_mask);
 			if(!dev->irq)
 			{
@@ -503,7 +544,7 @@ static int __init ni65_probe1(struct net_device *dev,int ioaddr)
 
 	if(request_dma(dev->dma, cards[p->cardno].cardname ) != 0)
 	{
-		printk("%s: Can't request dma-channel %d\n",dev->name,(int) dev->dma);
+		printk(KERN_ERR "%s: Can't request dma-channel %d\n",dev->name,(int) dev->dma);
 		ni65_free_buffer(p);
 		release_region(ioaddr, cards[p->cardno].total_size);
 		return -EAGAIN;
@@ -518,9 +559,6 @@ static int __init ni65_probe1(struct net_device *dev,int ioaddr)
 	dev->watchdog_timeo	= HZ/2;
 	dev->get_stats		= ni65_get_stats;
 	dev->set_multicast_list = set_multicast_list;
-
-	ether_setup(dev);
-
 	return 0; /* everything is OK */
 }
 
@@ -570,7 +608,7 @@ static void *ni65_alloc_mem(struct net_device *dev,char *what,int size,int type)
 	if(type) {
 		ret = skb = alloc_skb(2+16+size,GFP_KERNEL|GFP_DMA);
 		if(!skb) {
-			printk("%s: unable to allocate %s memory.\n",dev->name,what);
+			printk(KERN_WARNING "%s: unable to allocate %s memory.\n",dev->name,what);
 			return NULL;
 		}
 		skb->dev = dev;
@@ -581,12 +619,12 @@ static void *ni65_alloc_mem(struct net_device *dev,char *what,int size,int type)
 	else {
 		ret = ptr = kmalloc(T_BUF_SIZE,GFP_KERNEL | GFP_DMA);
 		if(!ret) {
-			printk("%s: unable to allocate %s memory.\n",dev->name,what);
+			printk(KERN_WARNING "%s: unable to allocate %s memory.\n",dev->name,what);
 			return NULL;
 		}
 	}
 	if( (u32) virt_to_phys(ptr+size) > 0x1000000) {
-		printk("%s: unable to allocate %s memory in lower 16MB!\n",dev->name,what);
+		printk(KERN_WARNING "%s: unable to allocate %s memory in lower 16MB!\n",dev->name,what);
 		if(type)
 			kfree_skb(skb);
 		else
@@ -692,7 +730,7 @@ static void ni65_stop_start(struct net_device *dev,struct priv *p)
 	writedatareg(CSR0_STOP);
 
 	if(debuglevel > 1)
-		printk("ni65_stop_start\n");
+		printk(KERN_DEBUG "ni65_stop_start\n");
 
 	if(p->features & INIT_RING_BEFORE_START) {
 		int i;
@@ -846,6 +884,8 @@ static irqreturn_t ni65_interrupt(int irq, void * dev_id, struct pt_regs * regs)
 
 	p = (struct priv *) dev->priv;
 
+	spin_lock(&p->ring_lock);
+	
 	while(--bcnt) {
 		csr0 = inw(PORT+L_DATAREG);
 
@@ -867,7 +907,7 @@ static irqreturn_t ni65_interrupt(int irq, void * dev_id, struct pt_regs * regs)
 		{
 			struct priv *p = (struct priv *) dev->priv;
 			if(debuglevel > 1)
-				printk("%s: general error: %04x.\n",dev->name,csr0);
+				printk(KERN_ERR "%s: general error: %04x.\n",dev->name,csr0);
 			if(csr0 & CSR0_BABL)
 				p->stats.tx_errors++;
 			if(csr0 & CSR0_MISS) {
@@ -879,7 +919,7 @@ static irqreturn_t ni65_interrupt(int irq, void * dev_id, struct pt_regs * regs)
 			}
 			if(csr0 & CSR0_MERR) {
 				if(debuglevel > 1)
-					printk("%s: Ooops .. memory error: %04x.\n",dev->name,csr0);
+					printk(KERN_ERR "%s: Ooops .. memory error: %04x.\n",dev->name,csr0);
 				ni65_stop_start(dev,p);
 			}
 		}
@@ -932,12 +972,13 @@ static irqreturn_t ni65_interrupt(int irq, void * dev_id, struct pt_regs * regs)
 #endif
 
 	if( (csr0 & (CSR0_RXON | CSR0_TXON)) != (CSR0_RXON | CSR0_TXON) ) {
-		printk("%s: RX or TX was offline -> restart\n",dev->name);
+		printk(KERN_DEBUG "%s: RX or TX was offline -> restart\n",dev->name);
 		ni65_stop_start(dev,p);
 	}
 	else
 		writedatareg(CSR0_INEA);
 
+	spin_unlock(&p->ring_lock);
 	return IRQ_HANDLED;
 }
 
@@ -1147,9 +1188,7 @@ static int ni65_send_packet(struct sk_buff *skb, struct net_device *dev)
 				memset((char *)p->tmdbounce[p->tmdbouncenum]+skb->len, 0, len-skb->len);
 			dev_kfree_skb (skb);
 
-			save_flags(flags);
-			cli();
-
+			spin_lock_irqsave(&p->ring_lock, flags);
 			tmdp = p->tmdhead + p->tmdnum;
 			tmdp->u.buffer = (u32) isa_virt_to_bus(p->tmdbounce[p->tmdbouncenum]);
 			p->tmdbouncenum = (p->tmdbouncenum + 1) & (TMDNUM - 1);
@@ -1157,8 +1196,7 @@ static int ni65_send_packet(struct sk_buff *skb, struct net_device *dev)
 #ifdef XMT_VIA_SKB
 		}
 		else {
-			save_flags(flags);
-			cli();
+			spin_lock_irqsave(&p->ring_lock, flags);
 
 			tmdp = p->tmdhead + p->tmdnum;
 			tmdp->u.buffer = (u32) isa_virt_to_bus(skb->data);
@@ -1178,8 +1216,8 @@ static int ni65_send_packet(struct sk_buff *skb, struct net_device *dev)
 			
 		p->lock = 0;
 		dev->trans_start = jiffies;
-
-		restore_flags(flags);
+		
+		spin_unlock_irqrestore(&p->ring_lock, flags);
 	}
 
 	return 0;
@@ -1210,12 +1248,7 @@ static void set_multicast_list(struct net_device *dev)
 }
 
 #ifdef MODULE
-static struct net_device dev_ni65 = { .base_addr = 0x360, .irq = 9, .init = ni65_probe };
-
-/* set: io,irq,dma or set it when calling insmod */
-static int irq;
-static int io;
-static int dma;
+static struct net_device *dev_ni65;
 
 MODULE_PARM(irq, "i");
 MODULE_PARM(io, "i");
@@ -1226,30 +1259,18 @@ MODULE_PARM_DESC(dma, "ni6510 ISA DMA channel (ignored for some cards)");
 
 int init_module(void)
 {
-	dev_ni65.irq = irq;
-	dev_ni65.dma = dma;
-	dev_ni65.base_addr = io;
-	if (register_netdev(&dev_ni65) != 0)
-		return -EIO;
-	return 0;
+ 	dev_ni65 = ni65_probe(-1);
+	return IS_ERR(dev_ni65) ? PTR_ERR(dev_ni65) : 0;
 }
 
 void cleanup_module(void)
 {
-	struct priv *p;
-	p = (struct priv *) dev_ni65.priv;
-	if(!p) {
-		printk("Ooops .. no private struct\n");
-		return;
-	}
-	disable_dma(dev_ni65.dma);
-	free_dma(dev_ni65.dma);
-	unregister_netdev(&dev_ni65);
-	release_region(dev_ni65.base_addr,cards[p->cardno].total_size);
-	ni65_free_buffer(p);
-	dev_ni65.priv = NULL;
+ 	unregister_netdev(dev_ni65);
+ 	cleanup_card(dev_ni65);
+ 	free_netdev(dev_ni65);
 }
 #endif /* MODULE */
+
 MODULE_LICENSE("GPL");
 
 /*
