@@ -58,7 +58,7 @@
 #include <linux/interrupt.h>
 #include <linux/stat.h>
 #include <linux/config.h>
-#include <linux/blk.h>
+#include <linux/blkdev.h>
 #include <linux/spinlock.h>
 #include <asm/io.h>
 
@@ -873,13 +873,13 @@ static void find_pio_PCI(struct get_conf *buf, Scsi_Host_Template * tpnt)
 	u32 base, x;
 
 	while ((dev = pci_find_device(PCI_VENDOR_ID_DPT, PCI_DEVICE_ID_DPT, dev)) != NULL) {
-		DBG(DBG_PROBE && DBG_PCI, printk("eata_pio: find_PCI, HBA at %s\n", dev->dev.name));
+		DBG(DBG_PROBE && DBG_PCI, printk("eata_pio: find_PCI, HBA at %s\n", pci_name(dev)));
 		if (pci_enable_device(dev))
 			continue;
 		pci_set_master(dev);
 		base = pci_resource_flags(dev, 0);
 		if (base & IORESOURCE_MEM) {
-			printk("eata_pio: invalid base address of device %s\n", dev->dev.name);
+			printk("eata_pio: invalid base address of device %s\n", pci_name(dev));
 			continue;
 		}
 		base = pci_resource_start(dev, 0);
