@@ -1,9 +1,8 @@
-
 /******************************************************************************
  *                  QLOGIC LINUX SOFTWARE
  *
- * QLogic ISP2x00 device driver for Linux 2.4.x
- * Copyright (C) 2002 Qlogic Corporation
+ * QLogic ISP2x00 device driver for Linux 2.6.x
+ * Copyright (C) 2003-2004 QLogic Corporation
  * (www.qlogic.com)
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -20,8 +19,9 @@
 /*
  * Driver version 
  */
-#if DEBUG_QLA2100
-#define QLA2100_VERSION      "6.04.00-debug"
-#else
-#define QLA2100_VERSION      "6.04.00"
-#endif
+#define QLA2XXX_VERSION      "8.00.00b11-k"
+
+#define QLA_DRIVER_MAJOR_VER	8
+#define QLA_DRIVER_MINOR_VER	0
+#define QLA_DRIVER_PATCH_VER	0
+#define QLA_DRIVER_BETA_VER	11
