@@ -1,8 +1,8 @@
 /*
  * bluetty.c   Version 0.13
  *
- * Copyright (c) 2000, 2001 Greg Kroah-Hartman	<greg@kroah.com>
- * Copyright (c) 2000 Mark Douglas Corner	<mcorner@umich.edu>
+ * Copyright (C) 2000, 2001 Greg Kroah-Hartman	<greg@kroah.com>
+ * Copyright (C) 2000 Mark Douglas Corner	<mcorner@umich.edu>
  *
  * USB Bluetooth TTY driver, based on the Bluetooth Spec version 1.0B
  * 
@@ -168,7 +168,7 @@ struct usb_bluetooth {
 	int			magic;
 	struct usb_device *	dev;
 	struct tty_driver *	tty_driver;	/* the tty_driver for this device */
-	struct tty_struct *	tty;		/* the coresponding tty for this port */
+	struct tty_struct *	tty;		/* the corresponding tty for this port */
 
 	unsigned char		minor;		/* the starting minor number for this device */
 	int			throttle;	/* throttled by tty layer */
@@ -190,7 +190,6 @@ struct usb_bluetooth {
 	int			bulk_in_buffer_size;
 
 	int			bulk_out_buffer_size;
-	struct urb *		write_urb_pool[NUM_BULK_URBS];
 	__u8			bulk_out_endpointAddress;
 
 	wait_queue_head_t	write_wait;
@@ -234,17 +233,14 @@ static struct usb_device_id usb_bluetooth_ids [] = {
 MODULE_DEVICE_TABLE (usb, usb_bluetooth_ids);
 
 static struct usb_driver usb_bluetooth_driver = {
+	.owner =	THIS_MODULE,
 	.name =		"bluetty",
 	.probe =	usb_bluetooth_probe,
 	.disconnect =	usb_bluetooth_disconnect,
 	.id_table =	usb_bluetooth_ids,
 };
 
-static int			bluetooth_refcount;
-static struct tty_driver	bluetooth_tty_driver;
-static struct tty_struct *	bluetooth_tty[BLUETOOTH_TTY_MINORS];
-static struct termios *		bluetooth_termios[BLUETOOTH_TTY_MINORS];
-static struct termios *		bluetooth_termios_locked[BLUETOOTH_TTY_MINORS];
+static struct tty_driver	*bluetooth_tty_driver;
 static struct usb_bluetooth	*bluetooth_table[BLUETOOTH_TTY_MINORS];
 
 
@@ -267,7 +263,7 @@ static inline struct usb_bluetooth* get_usb_bluetooth (struct usb_bluetooth *blu
 {
 	if (!bluetooth || 
 	    bluetooth_paranoia_check (bluetooth, function)) { 
-		/* then say that we dont have a valid usb_bluetooth thing, which will
+		/* then say that we don't have a valid usb_bluetooth thing, which will
 		 * end up generating -ENODEV return values */
 		return NULL;
 	}
@@ -276,9 +272,9 @@ static inline struct usb_bluetooth* get_usb_bluetooth (struct usb_bluetooth *blu
 }
 
 
-static inline struct usb_bluetooth *get_bluetooth_by_minor (int minor)
+static inline struct usb_bluetooth *get_bluetooth_by_index (int index)
 {
-	return bluetooth_table[minor];
+	return bluetooth_table[index];
 }
 
 
@@ -357,7 +353,7 @@ static int bluetooth_open (struct tty_struct *tty, struct file * filp)
 	tty->driver_data = NULL;
 
 	/* get the bluetooth object associated with this tty pointer */
-	bluetooth = get_bluetooth_by_minor (minor(tty->device));
+	bluetooth = get_bluetooth_by_index (tty->index);
 
 	if (bluetooth_paranoia_check (bluetooth, __FUNCTION__)) {
 		return -ENODEV;
@@ -411,7 +407,6 @@ static int bluetooth_open (struct tty_struct *tty, struct file * filp)
 static void bluetooth_close (struct tty_struct *tty, struct file * filp)
 {
 	struct usb_bluetooth *bluetooth = get_usb_bluetooth ((struct usb_bluetooth *)tty->driver_data, __FUNCTION__);
-	int i;
 
 	if (!bluetooth) {
 		return;
@@ -430,9 +425,7 @@ static void bluetooth_close (struct tty_struct *tty, struct file * filp)
 	if (bluetooth->open_count <= 0) {
 		bluetooth->open_count = 0;
 
-		/* shutdown any bulk reads and writes that might be going on */
-		for (i = 0; i < NUM_BULK_URBS; ++i)
-			usb_unlink_urb (bluetooth->write_urb_pool[i]);
+		/* shutdown any in-flight urbs that we know about */
 		usb_unlink_urb (bluetooth->read_urb);
 		usb_unlink_urb (bluetooth->interrupt_in_urb);
 	}
@@ -446,9 +439,7 @@ static int bluetooth_write (struct tty_struct * tty, int from_user, const unsign
 	struct urb *urb = NULL;
 	unsigned char *temp_buffer = NULL;
 	const unsigned char *current_buffer;
-	const unsigned char *current_position;
-	int bytes_sent;
-	int buffer_size;
+	unsigned char *urb_buffer;
 	int i;
 	int retval = 0;
 
@@ -487,7 +478,7 @@ static int bluetooth_write (struct tty_struct * tty, int from_user, const unsign
 			retval = -ENOMEM;
 			goto exit;
 		}
-		if (copy_from_user (temp_buffer, buf, count)) {
+		if (copy_from_user (temp_buffer, (void __user *)buf, count)) {
 			retval = -EFAULT;
 			goto exit;
 		}
@@ -509,54 +500,46 @@ static int bluetooth_write (struct tty_struct * tty, int from_user, const unsign
 			break;
 
 		case ACL_PKT:
-			current_position = current_buffer;
-			++current_position;
+			++current_buffer;
 			--count;
-			bytes_sent = 0;
 
-			while (count > 0) {
-				urb = NULL;
-
-				/* try to find a free urb in our list */
-				for (i = 0; i < NUM_BULK_URBS; ++i) {
-					if (bluetooth->write_urb_pool[i]->status != -EINPROGRESS) {
-						urb = bluetooth->write_urb_pool[i];
-						break;
-					}
-				}
-				if (urb == NULL) {
-					dbg ("%s - no free urbs", __FUNCTION__);
-					retval = bytes_sent;
-					goto exit;
-				}
-				
-
-				buffer_size = min (count, bluetooth->bulk_out_buffer_size);
-				memcpy (urb->transfer_buffer, current_position, buffer_size);
-
-				/* build up our urb */
-				usb_fill_bulk_urb (urb, bluetooth->dev, usb_sndbulkpipe(bluetooth->dev, bluetooth->bulk_out_endpointAddress),
-						urb->transfer_buffer, buffer_size, bluetooth_write_bulk_callback, bluetooth);
-
-				/* send it down the pipe */
-				retval = usb_submit_urb(urb, GFP_KERNEL);
-				if (retval) {
-					dbg("%s - usb_submit_urb(write bulk) failed with error = %d", __FUNCTION__, retval);
-					goto exit;
-				}
-#ifdef BTBUGGYHARDWARE
-				/* A workaround for the stalled data bug */
-				/* May or may not be needed...*/
-				if (count != 0) {
-					udelay(500);
-				}
-#endif
-				current_position += buffer_size;
-				bytes_sent += buffer_size;
-				count -= buffer_size;
+			urb_buffer = kmalloc (count, GFP_ATOMIC);
+			if (!urb_buffer) {
+				dev_err(&bluetooth->dev->dev, "out of memory\n");
+				retval = -ENOMEM;
+				goto exit;
 			}
 
-			retval = bytes_sent + 1;
+			urb = usb_alloc_urb(0, GFP_ATOMIC);
+			if (!urb) {
+				dev_err(&bluetooth->dev->dev, "no more free urbs\n");
+				kfree(urb_buffer);
+				retval = -ENOMEM;
+				goto exit;
+			}
+			memcpy (urb_buffer, current_buffer, count);
+
+			/* build up our urb */
+			usb_fill_bulk_urb(urb, bluetooth->dev, 
+					  usb_sndbulkpipe(bluetooth->dev,
+						  	  bluetooth->bulk_out_endpointAddress),
+					  urb_buffer,
+					  count,
+					  bluetooth_write_bulk_callback,
+					  bluetooth);
+
+
+			/* send it down the pipe */
+			retval = usb_submit_urb(urb, GFP_KERNEL);
+			if (retval) {
+				dbg("%s - usb_submit_urb(write bulk) failed with error = %d", __FUNCTION__, retval);
+				goto exit;
+			}
+
+			/* we are done with this urb, so let the host driver
+			 * really free it when it is finished with it */
+			usb_free_urb (urb);
+			retval = count + 1;
 			break;
 		
 		default :
@@ -566,8 +549,7 @@ static int bluetooth_write (struct tty_struct * tty, int from_user, const unsign
 	}
 
 exit:
-	if (temp_buffer != NULL)
-		kfree (temp_buffer);
+	kfree (temp_buffer);
 
 	return retval;
 } 
@@ -575,55 +557,28 @@ exit:
 
 static int bluetooth_write_room (struct tty_struct *tty) 
 {
-	struct usb_bluetooth *bluetooth = get_usb_bluetooth ((struct usb_bluetooth *)tty->driver_data, __FUNCTION__);
-	int room = 0;
-	int i;
-
-	if (!bluetooth) {
-		return -ENODEV;
-	}
-
 	dbg("%s", __FUNCTION__);
 
-	if (!bluetooth->open_count) {
-		dbg ("%s - device not open", __FUNCTION__);
-		return -EINVAL;
-	}
-
-	for (i = 0; i < NUM_BULK_URBS; ++i) {
-		if (bluetooth->write_urb_pool[i]->status != -EINPROGRESS) {
-			room += bluetooth->bulk_out_buffer_size;
-		}
-	}
-
-	dbg("%s - returns %d", __FUNCTION__, room);
-	return room;
+	/*
+	 * We really can take anything the user throws at us
+	 * but let's pick a nice big number to tell the tty
+	 * layer that we have lots of free space
+	 */
+	return 2048;
 }
 
 
 static int bluetooth_chars_in_buffer (struct tty_struct *tty) 
 {
-	struct usb_bluetooth *bluetooth = get_usb_bluetooth ((struct usb_bluetooth *)tty->driver_data, __FUNCTION__);
-	int chars = 0;
-	int i;
+	dbg("%s", __FUNCTION__);
 
-	if (!bluetooth) {
-		return -ENODEV;
-	}
-
-	if (!bluetooth->open_count) {
-		dbg ("%s - device not open", __FUNCTION__);
-		return -EINVAL;
-	}
-
-	for (i = 0; i < NUM_BULK_URBS; ++i) {
-		if (bluetooth->write_urb_pool[i]->status == -EINPROGRESS) {
-			chars += bluetooth->write_urb_pool[i]->transfer_buffer_length;
-		}
-	}
-
-	dbg ("%s - returns %d", __FUNCTION__, chars);
-	return chars;
+	/* 
+	 * We can't really account for how much data we
+	 * have sent out, but hasn't made it through to the
+	 * device, so just tell the tty layer that everything
+	 * is flushed.
+	 */
+	return 0;
 }
 
 
@@ -1012,6 +967,9 @@ static void bluetooth_write_bulk_callback (struct urb *urb, struct pt_regs *regs
 
 	dbg("%s", __FUNCTION__);
 
+	/* free up the transfer buffer, as usb_free_urb() does not do this */
+	kfree(urb->transfer_buffer);
+
 	if (!bluetooth) {
 		dbg("%s - bad bluetooth pointer, exiting", __FUNCTION__);
 		return;
@@ -1067,7 +1025,7 @@ static int usb_bluetooth_probe (struct usb_interface *intf,
 	int num_bulk_in = 0;
 	int num_bulk_out = 0;
 
-	interface = &intf->altsetting[0];
+	interface = intf->cur_altsetting;
 	control_out_endpoint = interface->desc.bInterfaceNumber;
 
 	/* find the endpoints that we need */
@@ -1107,20 +1065,17 @@ static int usb_bluetooth_probe (struct usb_interface *intf,
 		return -EIO;
 	}
 
-	MOD_INC_USE_COUNT;
 	info("USB Bluetooth converter detected");
 
 	for (minor = 0; minor < BLUETOOTH_TTY_MINORS && bluetooth_table[minor]; ++minor)
 		;
 	if (bluetooth_table[minor]) {
 		err("No more free Bluetooth devices");
-		MOD_DEC_USE_COUNT;
 		return -ENODEV;
 	}
 
 	if (!(bluetooth = kmalloc(sizeof(struct usb_bluetooth), GFP_KERNEL))) {
 		err("Out of memory");
-		MOD_DEC_USE_COUNT;
 		return -ENOMEM;
 	}
 
@@ -1167,21 +1122,6 @@ static int usb_bluetooth_probe (struct usb_interface *intf,
 	bluetooth->bulk_out_endpointAddress = endpoint->bEndpointAddress;
 	bluetooth->bulk_out_buffer_size = endpoint->wMaxPacketSize * 2;
 
-	/* create our write urb pool */ 
-	for (i = 0; i < NUM_BULK_URBS; ++i) {
-		struct urb  *urb = usb_alloc_urb(0, GFP_KERNEL);
-		if (urb == NULL) {
-			err("No free urbs available");
-			goto probe_error;
-		}
-		urb->transfer_buffer = kmalloc (bluetooth->bulk_out_buffer_size, GFP_KERNEL);
-		if (urb->transfer_buffer == NULL) {
-			err("out of memory");
-			goto probe_error;
-		}
-		bluetooth->write_urb_pool[i] = urb;
-	}
-
 	endpoint = interrupt_in_endpoint[0];
 	bluetooth->interrupt_in_urb = usb_alloc_urb(0, GFP_KERNEL);
 	if (!bluetooth->interrupt_in_urb) {
@@ -1201,7 +1141,7 @@ static int usb_bluetooth_probe (struct usb_interface *intf,
 		     bluetooth, endpoint->bInterval);
 
 	/* initialize the devfs nodes for this device and let the user know what bluetooths we are bound to */
-	tty_register_device (&bluetooth_tty_driver, minor);
+	tty_register_device (bluetooth_tty_driver, minor, &intf->dev);
 	info("Bluetooth converter now attached to ttyUB%d (or usb/ttub/%d for devfs)", minor, minor);
 
 	bluetooth_table[minor] = bluetooth;
@@ -1219,12 +1159,6 @@ probe_error:
 		usb_free_urb (bluetooth->interrupt_in_urb);
 	if (bluetooth->interrupt_in_buffer)
 		kfree (bluetooth->interrupt_in_buffer);
-	for (i = 0; i < NUM_BULK_URBS; ++i)
-		if (bluetooth->write_urb_pool[i]) {
-			if (bluetooth->write_urb_pool[i]->transfer_buffer)
-				kfree (bluetooth->write_urb_pool[i]->transfer_buffer);
-			usb_free_urb (bluetooth->write_urb_pool[i]);
-		}
 	for (i = 0; i < NUM_CONTROL_URBS; ++i) 
 		if (bluetooth->control_urb_pool[i]) {
 			if (bluetooth->control_urb_pool[i]->transfer_buffer)
@@ -1236,7 +1170,6 @@ probe_error:
 
 	/* free up any memory that we allocated */
 	kfree (bluetooth);
-	MOD_DEC_USE_COUNT;
 	return -EIO;
 }
 
@@ -1267,16 +1200,8 @@ static void usb_bluetooth_disconnect(struct usb_interface *intf)
 		if (bluetooth->interrupt_in_buffer)
 			kfree (bluetooth->interrupt_in_buffer);
 
-		tty_unregister_device (&bluetooth_tty_driver, bluetooth->minor);
+		tty_unregister_device (bluetooth_tty_driver, bluetooth->minor);
 
-		for (i = 0; i < NUM_BULK_URBS; ++i) {
-			if (bluetooth->write_urb_pool[i]) {
-				usb_unlink_urb (bluetooth->write_urb_pool[i]);
-				if (bluetooth->write_urb_pool[i]->transfer_buffer)
-					kfree (bluetooth->write_urb_pool[i]->transfer_buffer);
-				usb_free_urb (bluetooth->write_urb_pool[i]);
-			}
-		}
 		for (i = 0; i < NUM_CONTROL_URBS; ++i) {
 			if (bluetooth->control_urb_pool[i]) {
 				usb_unlink_urb (bluetooth->control_urb_pool[i]);
@@ -1295,27 +1220,9 @@ static void usb_bluetooth_disconnect(struct usb_interface *intf)
 	} else {
 		info("device disconnected");
 	}
-
-	MOD_DEC_USE_COUNT;
 }
 
-
-static struct tty_driver bluetooth_tty_driver = {
-	.magic =		TTY_DRIVER_MAGIC,
-	.driver_name =		"usb-bluetooth",
-	.name =			"usb/ttub/%d",
-	.major =		BLUETOOTH_TTY_MAJOR,
-	.minor_start =		0,
-	.num =			BLUETOOTH_TTY_MINORS,
-	.type =			TTY_DRIVER_TYPE_SERIAL,
-	.subtype =		SERIAL_TYPE_NORMAL,
-	.flags =		TTY_DRIVER_REAL_RAW | TTY_DRIVER_NO_DEVFS,
-
-	.refcount =		&bluetooth_refcount,
-	.table =		bluetooth_tty,
-	.termios =		bluetooth_termios,
-	.termios_locked =	bluetooth_termios_locked,
-
+static struct tty_operations bluetooth_ops = {
 	.open =			bluetooth_open,
 	.close =		bluetooth_close,
 	.write =		bluetooth_write,
@@ -1327,31 +1234,45 @@ static struct tty_driver bluetooth_tty_driver = {
 	.chars_in_buffer =	bluetooth_chars_in_buffer,
 };
 
-
 int usb_bluetooth_init(void)
 {
 	int i;
 	int result;
 
-	/* Initalize our global data */
+	/* Initialize our global data */
 	for (i = 0; i < BLUETOOTH_TTY_MINORS; ++i) {
 		bluetooth_table[i] = NULL;
 	}
 
 	info ("USB Bluetooth support registered");
 
-	/* register the tty driver */
-	bluetooth_tty_driver.init_termios          = tty_std_termios;
-	bluetooth_tty_driver.init_termios.c_cflag  = B9600 | CS8 | CREAD | HUPCL | CLOCAL;
-	if (tty_register_driver (&bluetooth_tty_driver)) {
+	bluetooth_tty_driver = alloc_tty_driver(BLUETOOTH_TTY_MINORS);
+	if (!bluetooth_tty_driver)
+		return -ENOMEM;
+
+	bluetooth_tty_driver->owner = THIS_MODULE;
+	bluetooth_tty_driver->driver_name = "usb-bluetooth";
+	bluetooth_tty_driver->name = "ttyUB";
+	bluetooth_tty_driver->devfs_name = "usb/ttub/";
+	bluetooth_tty_driver->major = BLUETOOTH_TTY_MAJOR;
+	bluetooth_tty_driver->minor_start = 0;
+	bluetooth_tty_driver->type = TTY_DRIVER_TYPE_SERIAL;
+	bluetooth_tty_driver->subtype = SERIAL_TYPE_NORMAL;
+	bluetooth_tty_driver->flags = TTY_DRIVER_REAL_RAW | TTY_DRIVER_NO_DEVFS;
+	bluetooth_tty_driver->init_termios = tty_std_termios;
+	bluetooth_tty_driver->init_termios.c_cflag = B9600 | CS8 | CREAD | HUPCL | CLOCAL;
+	tty_set_operations(bluetooth_tty_driver, &bluetooth_ops);
+	if (tty_register_driver (bluetooth_tty_driver)) {
 		err("%s - failed to register tty driver", __FUNCTION__);
+		put_tty_driver(bluetooth_tty_driver);
 		return -1;
 	}
 
 	/* register the USB driver */
 	result = usb_register(&usb_bluetooth_driver);
 	if (result < 0) {
-		tty_unregister_driver(&bluetooth_tty_driver);
+		tty_unregister_driver(bluetooth_tty_driver);
+		put_tty_driver(bluetooth_tty_driver);
 		err("usb_register failed for the USB bluetooth driver. Error number %d", result);
 		return -1;
 	}
@@ -1365,7 +1286,8 @@ int usb_bluetooth_init(void)
 void usb_bluetooth_exit(void)
 {
 	usb_deregister(&usb_bluetooth_driver);
-	tty_unregister_driver(&bluetooth_tty_driver);
+	tty_unregister_driver(bluetooth_tty_driver);
+	put_tty_driver(bluetooth_tty_driver);
 }
 
 
