@@ -105,7 +105,7 @@ static void dump_hci_status(struct usb_hcd *hcd, const char *label)
 }
 #endif
 
-static void usb_hcd_sa1111_hcim_irq (int irq, void *__hcd, struct pt_regs * r)
+static irqreturn_t usb_hcd_sa1111_hcim_irq (int irq, void *__hcd, struct pt_regs * r)
 {
 	struct usb_hcd *hcd = __hcd;
 //	unsigned long status = sa1111_readl(hcd->regs + SA1111_USB_STATUS);
@@ -121,6 +121,12 @@ static void usb_hcd_sa1111_hcim_irq (int irq, void *__hcd, struct pt_regs * r)
 #endif
 
 	usb_hcd_irq(irq, hcd, r);
+
+	/*
+	 * SA1111 seems to re-assert its interrupt immediately
+	 * after processing an interrupt.  Always return IRQ_HANDLED.
+	 */
+	return IRQ_HANDLED;
 }
 
 /*-------------------------------------------------------------------------*/
@@ -167,9 +173,7 @@ int usb_hcd_sa1111_probe (const struct hc_driver *driver,
 	hcd->description = driver->description;
 	hcd->irq = dev->irq[1];
 	hcd->regs = dev->mapbase;
-	hcd->pdev = SA1111_FAKE_PCIDEV;
 	hcd->self.controller = &dev->dev;
-	hcd->controller = hcd->self.controller;
 
 	retval = hcd_buffer_create (hcd);
 	if (retval != 0) {
@@ -254,8 +258,6 @@ void usb_hcd_sa1111_remove (struct usb_hcd *hcd, struct sa1111_dev *dev)
 	hcd_buffer_destroy (hcd);
 
 	usb_deregister_bus (&hcd->self);
-	if (atomic_read (&hcd->self.refcnt) != 1)
-		err ("%s: %s, count != 1", __FUNCTION__, hcd->self.bus_name);
 
 	base = hcd->regs;
 	hcd->driver->hcd_free (hcd);
@@ -272,14 +274,12 @@ ohci_sa1111_start (struct usb_hcd *hcd)
 	struct ohci_hcd	*ohci = hcd_to_ohci (hcd);
 	int		ret;
 
-	if (hcd->pdev) {
-		ohci->hcca = pci_alloc_consistent (hcd->pdev,
-				sizeof *ohci->hcca, &ohci->hcca_dma);
-		if (!ohci->hcca)
-			return -ENOMEM;
-	}
-
-        memset (ohci->hcca, 0, sizeof (struct ohci_hcca));
+	ohci->hcca = dma_alloc_coherent (hcd->self.controller,
+			sizeof *ohci->hcca, &ohci->hcca_dma, 0);
+	if (!ohci->hcca)
+		return -ENOMEM;
+        
+	memset (ohci->hcca, 0, sizeof (struct ohci_hcca));
 	if ((ret = ohci_mem_init (ohci)) < 0) {
 		ohci_stop (hcd);
 		return ret;
@@ -296,6 +296,7 @@ ohci_sa1111_start (struct usb_hcd *hcd)
 		ohci_stop (hcd);
 		return -EBUSY;
 	}
+	create_debug_files (ohci);
 
 #ifdef	DEBUG
 	ohci_dump (ohci, 1);
@@ -351,9 +352,8 @@ static const struct hc_driver ohci_sa1111_hc_driver = {
 
 /*-------------------------------------------------------------------------*/
 
-static int ohci_hcd_sa1111_drv_probe(struct device *_dev)
+static int ohci_hcd_sa1111_drv_probe(struct sa1111_dev *dev)
 {
-	struct sa1111_dev *dev = SA1111_DEV(_dev);
 	struct usb_hcd *hcd = NULL;
 	int ret;
 
@@ -363,43 +363,29 @@ static int ohci_hcd_sa1111_drv_probe(struct device *_dev)
 	ret = usb_hcd_sa1111_probe(&ohci_sa1111_hc_driver, &hcd, dev);
 
 	if (ret == 0)
-		dev->dev.driver_data = hcd;
+		sa1111_set_drvdata(dev, hcd);
 
 	return ret;
 }
 
-static int ohci_hcd_sa1111_drv_remove(struct device *_dev)
+static int ohci_hcd_sa1111_drv_remove(struct sa1111_dev *dev)
 {
-	struct sa1111_dev *dev = SA1111_DEV(_dev);
-	struct usb_hcd *hcd = dev->dev.driver_data;
+	struct usb_hcd *hcd = sa1111_get_drvdata(dev);
 
 	usb_hcd_sa1111_remove(hcd, dev);
 
-	dev->dev.driver_data = NULL;
+	sa1111_set_drvdata(dev, NULL);
 
 	return 0;
 }
 
-static int ohci_hcd_sa1111_drv_suspend(struct device *dev, u32 state, u32 level)
-{
-	return 0;
-}
-
-static int ohci_hcd_sa1111_drv_resume(struct device *dev, u32 level)
-{
-	return 0;
-}
-
 static struct sa1111_driver ohci_hcd_sa1111_driver = {
 	.drv = {
-		.name		= "sa1111-ohci",
-		.bus		= &sa1111_bus_type,
-		.probe		= ohci_hcd_sa1111_drv_probe,
-		.remove		= ohci_hcd_sa1111_drv_remove,
-		.suspend	= ohci_hcd_sa1111_drv_suspend,
-		.resume		= ohci_hcd_sa1111_drv_resume,
+		.name	= "sa1111-ohci",
 	},
-	.devid			= SA1111_DEVID_USB,
+	.devid		= SA1111_DEVID_USB,
+	.probe		= ohci_hcd_sa1111_drv_probe,
+	.remove		= ohci_hcd_sa1111_drv_remove,
 };
 
 static int __init ohci_hcd_sa1111_init (void)
@@ -408,12 +394,12 @@ static int __init ohci_hcd_sa1111_init (void)
 	dbg ("block sizes: ed %d td %d",
 		sizeof (struct ed), sizeof (struct td));
 
-	return driver_register(&ohci_hcd_sa1111_driver.drv);
+	return sa1111_driver_register(&ohci_hcd_sa1111_driver);
 }
 
 static void __exit ohci_hcd_sa1111_cleanup (void)
 {
-	driver_unregister(&ohci_hcd_sa1111_driver.drv);
+	sa1111_driver_unregister(&ohci_hcd_sa1111_driver);
 }
 
 module_init (ohci_hcd_sa1111_init);
