@@ -23,7 +23,6 @@
 
 #include "usb-serial.h"
 
-
 static int usb_serial_device_match (struct device *dev, struct device_driver *drv)
 {
 	struct usb_serial_device_type *driver;
@@ -77,8 +76,7 @@ static int usb_serial_device_probe (struct device *dev)
 	}
 
 	minor = port->number;
-
-	tty_register_device (&usb_serial_tty_driver, minor);
+	tty_register_device (usb_serial_tty_driver, minor, dev);
 	dev_info(&port->serial->dev->dev, 
 		 "%s converter now attached to ttyUSB%d (or usb/tts/%d for devfs)\n",
 		 driver->name, minor, minor);
@@ -111,7 +109,7 @@ static int usb_serial_device_remove (struct device *dev)
 	}
 exit:
 	minor = port->number;
-	tty_unregister_device (&usb_serial_tty_driver, minor);
+	tty_unregister_device (usb_serial_tty_driver, minor);
 	dev_info(dev, "%s converter now disconnected from ttyUSB%d\n",
 		 driver->name, minor);
 
@@ -129,7 +127,6 @@ int usb_serial_bus_register(struct usb_serial_device_type *device)
 	device->driver.bus = &usb_serial_bus_type;
 	device->driver.probe = usb_serial_device_probe;
 	device->driver.remove = usb_serial_device_remove;
-	device->driver.devclass = &tty_devclass;
 
 	retval = driver_register(&device->driver);
 
