@@ -123,7 +123,6 @@ struct cyberfb_par {
 static struct cyberfb_par current_par;
 
 static int current_par_valid = 0;
-static int currcon = 0;
 
 static struct display disp;
 static struct fb_info fb_info;
@@ -247,8 +246,9 @@ static int cyberfb_set_var(struct fb_var_screeninfo *var, int con,
 			   struct fb_info *info);
 static int cyberfb_get_cmap(struct fb_cmap *cmap, int kspc, int con,
 			    struct fb_info *info);
-static int cyberfb_set_cmap(struct fb_cmap *cmap, int kspc, int con,
-			    struct fb_info *info);
+static int cyberfb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
+			     u_int transp, struct fb_info *info);
+static int cyberfb_blank(int blank, struct fb_info *info);
 
 /*
  *    Interface to the low level console driver
@@ -257,7 +257,6 @@ static int cyberfb_set_cmap(struct fb_cmap *cmap, int kspc, int con,
 int cyberfb_init(void);
 static int Cyberfb_switch(int con, struct fb_info *info);
 static int Cyberfb_updatevar(int con, struct fb_info *info);
-static void Cyberfb_blank(int blank, struct fb_info *info);
 
 /*
  *    Text console acceleration
@@ -295,8 +294,6 @@ static int Cyber_encode_var(struct fb_var_screeninfo *var,
 			    struct cyberfb_par *par);
 static int Cyber_getcolreg(u_int regno, u_int *red, u_int *green, u_int *blue,
 			   u_int *transp, struct fb_info *info);
-static int Cyber_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
-			   u_int transp, struct fb_info *info);
 
 /*
  *    Internal routines
@@ -305,7 +302,6 @@ static int Cyber_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
 static void cyberfb_get_par(struct cyberfb_par *par);
 static void cyberfb_set_par(struct cyberfb_par *par);
 static int do_fb_set_var(struct fb_var_screeninfo *var, int isactive);
-static void do_install_cmap(int con, struct fb_info *info);
 static void cyberfb_set_disp(int con, struct fb_info *info);
 static int get_video_mode(const char *name);
 
@@ -379,8 +375,8 @@ static int Cyber_init(void)
 		*(CursorBase+3+(i*4)) = 0xffff0000;
 	}
 
-	Cyber_setcolreg (255, 56<<8, 100<<8, 160<<8, 0, NULL /* unused */);
-	Cyber_setcolreg (254, 0, 0, 0, 0, NULL /* unused */);
+	cyberfb_setcolreg (255, 56<<8, 100<<8, 160<<8, 0, NULL /* unused */);
+	cyberfb_setcolreg (254, 0, 0, 0, 0, NULL /* unused */);
 
 	DPRINTK("EXIT\n");
 	return 0;
@@ -517,7 +513,7 @@ static int Cyber_encode_var(struct fb_var_screeninfo *var,
  *    Set a single color register. Return != 0 for invalid regno.
  */
 
-static int Cyber_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
+static int cyberfb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
 			   u_int transp, struct fb_info *info)
 {
 	volatile unsigned char *regs = CyberRegs;
@@ -580,8 +576,7 @@ static int Cyber_getcolreg(u_int regno, u_int *red, u_int *green, u_int *blue,
 *    blank: 1 = zero fb cmap
 *           0 = restore fb cmap from local cmap
 */
-
-void Cyberfb_blank(int blank, struct fb_info *info)
+static int cyberfb_blank(int blank, struct fb_info *info)
 {
 	volatile unsigned char *regs = CyberRegs;
 	int i;
@@ -609,6 +604,7 @@ void Cyberfb_blank(int blank, struct fb_info *info)
 	}
 #endif
 	DPRINTK("EXIT\n");
+	return 0;
 }
 
 
@@ -800,25 +796,6 @@ static int do_fb_set_var(struct fb_var_screeninfo *var, int isactive)
 	return 0;
 }
 
-
-static void do_install_cmap(int con, struct fb_info *info)
-{
-	DPRINTK("ENTER\n");
-	if (con != currcon) {
-		DPRINTK("EXIT - Not current console\n");
-		return;
-	}
-	if (fb_display[con].cmap.len) {
-		DPRINTK("Use console cmap\n");
-		fb_set_cmap(&fb_display[con].cmap, 1, Cyber_setcolreg, info);
-	} else {
-		DPRINTK("Use default cmap\n");
-		fb_set_cmap(fb_default_cmap(1<<fb_display[con].var.bits_per_pixel),
-			    1, Cyber_setcolreg, info);
-	}
-	DPRINTK("EXIT\n");
-}
-
 /*
  *    Get the Fixed Part of the Display
  */
@@ -878,7 +855,6 @@ static void cyberfb_set_disp(int con, struct fb_info *info)
 	cyberfb_get_fix(&fix, con, info);
 	if (con == -1)
 		con = 0;
-	display->screen_base = (unsigned char *)CyberMem;
 	display->visual = fix.visual;
 	display->type = fix.type;
 	display->type_aux = fix.type_aux;
@@ -919,7 +895,7 @@ static int cyberfb_set_var(struct fb_var_screeninfo *var, int con,
 	int err, oldxres, oldyres, oldvxres, oldvyres, oldbpp, oldaccel;
 
 	DPRINTK("ENTER\n");
-	if ((err = do_fb_set_var(var, con == currcon))) {
+	if ((err = do_fb_set_var(var, con == info->currcon))) {
 		DPRINTK("EXIT - do_fb_set_var failed\n");
 		return(err);
 	}
@@ -956,7 +932,7 @@ static int cyberfb_get_cmap(struct fb_cmap *cmap, int kspc, int con,
 			    struct fb_info *info)
 {
 	DPRINTK("ENTER\n");
-	if (con == currcon) { /* current console? */
+	if (con == info->currcon) { /* current console? */
 		DPRINTK("EXIT - console is current console\n");
 		return(fb_get_cmap(cmap, kspc, Cyber_getcolreg, info));
 	} else if (fb_display[con].cmap.len) { /* non default colormap? */
@@ -971,43 +947,15 @@ static int cyberfb_get_cmap(struct fb_cmap *cmap, int kspc, int con,
 	return(0);
 }
 
-
-/*
- *    Set the Colormap
- */
-
-static int cyberfb_set_cmap(struct fb_cmap *cmap, int kspc, int con,
-			    struct fb_info *info)
-{
-	int err;
-
-	DPRINTK("ENTER\n");
-	if (!fb_display[con].cmap.len) {       /* no colormap allocated? */
-		if ((err = fb_alloc_cmap(&fb_display[con].cmap,
-					 1<<fb_display[con].var.bits_per_pixel,
-					 0))) {
-			DPRINTK("EXIT - fb_alloc_cmap failed\n");
-			return(err);
-		}
-	}
-	if (con == currcon) {		 /* current console? */
-		DPRINTK("EXIT - Current console\n");
-		return(fb_set_cmap(cmap, kspc, Cyber_setcolreg, info));
-	} else {
-		fb_copy_cmap(cmap, &fb_display[con].cmap, kspc ? 0 : 1);
-	}
-	DPRINTK("EXIT\n");
-	return(0);
-}
-
-
 static struct fb_ops cyberfb_ops = {
-	owner:		THIS_MODULE,
-	fb_get_fix:	cyberfb_get_fix,
-	fb_get_var:	cyberfb_get_var,
-	fb_set_var:	cyberfb_set_var,
-	fb_get_cmap:	cyberfb_get_cmap,
-	fb_set_cmap:	cyberfb_set_cmap,
+	.owner =	THIS_MODULE,
+	.fb_get_fix =	cyberfb_get_fix,
+	.fb_get_var =	cyberfb_get_var,
+	.fb_set_var =	cyberfb_set_var,
+	.fb_get_cmap =	cyberfb_get_cmap,
+	.fb_set_cmap =	gen_set_cmap,
+	.fb_setcolreg =	cyberfb_setcolreg,
+	.fb_blank =	cyberfb_blank,
 };
 
 int __init cyberfb_setup(char *options)
@@ -1085,12 +1033,12 @@ int __init cyberfb_init(void)
 
 	    strcpy(fb_info.modename, cyberfb_name);
 	    fb_info.changevar = NULL;
-	    fb_info.node = NODEV;
 	    fb_info.fbops = &cyberfb_ops;
+	    fb_info.screen_base = (unsigned char *)CyberMem;
 	    fb_info.disp = &disp;
+	    fb_info.currcon = -1;
 	    fb_info.switch_con = &Cyberfb_switch;
 	    fb_info.updatevar = &Cyberfb_updatevar;
-	    fb_info.blank = &Cyberfb_blank;
 
 	    Cyber_init();
 	    /* ++Andre: set cyberfb default mode */
@@ -1114,7 +1062,7 @@ int __init cyberfb_init(void)
 	    }
 
 	    printk("fb%d: %s frame buffer device, using %ldK of video memory\n",
-		   GET_FB_IDX(fb_info.node), fb_info.modename, CyberSize>>10);
+		   fb_info.node, fb_info.modename, CyberSize>>10);
 
 	    /* TODO: This driver cannot be unloaded yet */
 	    MOD_INC_USE_COUNT;
@@ -1129,13 +1077,13 @@ static int Cyberfb_switch(int con, struct fb_info *info)
 {
         DPRINTK("ENTER\n");
 	/* Do we have to save the colormap? */
-	if (fb_display[currcon].cmap.len) {
-		fb_get_cmap(&fb_display[currcon].cmap, 1, Cyber_getcolreg,
+	if (fb_display[info->currcon].cmap.len) {
+		fb_get_cmap(&fb_display[info->currcon].cmap, 1, Cyber_getcolreg,
 			    info);
 	}
 
 	do_fb_set_var(&fb_display[con].var, 1);
-	currcon = con;
+	info->currcon = con;
 	/* Install new colormap */
 	do_install_cmap(con, info);
 	DPRINTK("EXIT\n");
@@ -1239,14 +1187,14 @@ static void fbcon_cyber8_revc(struct display *p, int xx, int yy)
 }
 
 static struct display_switch fbcon_cyber8 = {
-	setup:		fbcon_cfb8_setup,
-	bmove:		fbcon_cyber8_bmove,
-	clear:		fbcon_cyber8_clear,
-	putc:		fbcon_cyber8_putc,
-	putcs:		fbcon_cyber8_putcs,
-	revc:		fbcon_cyber8_revc,
-	clear_margins:	fbcon_cfb8_clear_margins,
-	fontwidthmask:	FONTWIDTH(8)
+	.setup =	fbcon_cfb8_setup,
+	.bmove =	fbcon_cyber8_bmove,
+	.clear =	fbcon_cyber8_clear,
+	.putc =		fbcon_cyber8_putc,
+	.putcs =	fbcon_cyber8_putcs,
+	.revc =		fbcon_cyber8_revc,
+	.clear_margins =fbcon_cfb8_clear_margins,
+	.fontwidthmask =FONTWIDTH(8)
 };
 #endif
 
