@@ -384,7 +384,7 @@ static int xxxfb_blank(int blank_mode, const struct fb_info *info)
  *	depending on the rastering operation with the value of color which
  *	is in the current color depth format.
  */
-void xxfb_fillrect(struct fb_info *p, struct fb_fillrect *region)
+void xxfb_fillrect(struct fb_info *p, const struct fb_fillrect *region)
 {
 /*	Meaning of struct fb_fillrect
  *
@@ -410,7 +410,7 @@ void xxfb_fillrect(struct fb_info *p, struct fb_fillrect *region)
  *      This drawing operation copies a rectangular area from one area of the
  *	screen to another area.
  */
-void xxxfb_copyarea(struct fb_info *p, struct fb_copyarea *area) 
+void xxxfb_copyarea(struct fb_info *p, const struct fb_copyarea *area) 
 {
 /*
  *      @dx: The x and y coordinates of the upper left hand corner of the
@@ -435,7 +435,7 @@ void xxxfb_copyarea(struct fb_info *p, struct fb_copyarea *area)
  *	mono image (needed for font handling) or a color image (needed for
  *	tux). 
  */
-void xxxfb_imageblit(struct fb_info *p, struct fb_image *image) 
+void xxxfb_imageblit(struct fb_info *p, const struct fb_image *image) 
 {
 /*
  *      @dx: The x and y coordinates of the upper left hand corner of the
@@ -536,7 +536,6 @@ int __init xxxfb_init(void)
      * space via ioremap. Consult ioport.h. 
      */
     info.screen_base = framebuffer_virtual_memory;	
-    info.node = NODEV;
     info.fbops = &xxxfb_ops;
     info.fix = xxxfb_fix;
     info.pseudo_palette = pseudo_palette;
@@ -566,7 +565,7 @@ int __init xxxfb_init(void)
 	
     if (register_framebuffer(&info) < 0)
 	return -EINVAL;
-    printk(KERN_INFO "fb%d: %s frame buffer device\n", minor(info.node),
+    printk(KERN_INFO "fb%d: %s frame buffer device\n", info.node,
 	   info.fix.id);
     return 0;
 }
