@@ -44,13 +44,13 @@ typedef struct {
 	char	name[8];
 } openprom_property;
 
-static openpromfs_node *nodes = NULL;
-static int alloced = 0;
-static u16 last_node = 0;
-static u16 first_prop = 0;
+static openpromfs_node *nodes;
+static int alloced;
+static u16 last_node;
+static u16 first_prop;
 static u16 options = 0xffff;
 static u16 aliases = 0xffff;
-static int aliases_nodes = 0;
+static int aliases_nodes;
 static char *alias_names [ALIASES_NNODES];
 
 #define OPENPROM_ROOT_INO	16
@@ -59,9 +59,9 @@ static char *alias_names [ALIASES_NNODES];
 #define NODE2INO(node) (node + OPENPROM_FIRST_INO)
 #define NODEP2INO(no) (no + OPENPROM_FIRST_INO + last_node)
 
-static int openpromfs_create (struct inode *, struct dentry *, int);
+static int openpromfs_create (struct inode *, struct dentry *, int, struct nameidata *);
 static int openpromfs_readdir(struct file *, void *, filldir_t);
-static struct dentry *openpromfs_lookup(struct inode *, struct dentry *dentry);
+static struct dentry *openpromfs_lookup(struct inode *, struct dentry *dentry, struct nameidata *nd);
 static int openpromfs_unlink (struct inode *, struct dentry *dentry);
 
 static ssize_t nodenum_read(struct file *file, char *buf,
@@ -639,7 +639,7 @@ static int lookup_children(u16 n, const char * name, int len)
 	return 0;
 }
 
-static struct dentry *openpromfs_lookup(struct inode * dir, struct dentry *dentry)
+static struct dentry *openpromfs_lookup(struct inode * dir, struct dentry *dentry, struct nameidata *nd)
 {
 	int ino = 0;
 #define OPFSL_DIR	0
@@ -854,7 +854,8 @@ out:
 	return 0;
 }
 
-static int openpromfs_create (struct inode *dir, struct dentry *dentry, int mode)
+static int openpromfs_create (struct inode *dir, struct dentry *dentry, int mode,
+		struct nameidata *nd)
 {
 	char *p;
 	struct inode *inode;
@@ -920,11 +921,7 @@ static int openpromfs_unlink (struct inode *dir, struct dentry *dentry)
 }
 
 /* {{{ init section */
-#ifndef MODULE
 static int __init check_space (u16 n)
-#else
-static int check_space (u16 n)
-#endif
 {
 	unsigned long pages;
 
@@ -944,11 +941,7 @@ static int check_space (u16 n)
 	return 0;
 }
 
-#ifndef MODULE
 static u16 __init get_nodes (u16 parent, u32 node)
-#else
-static u16 get_nodes (u16 parent, u32 node)
-#endif
 {
 	char *p;
 	u16 n = last_node++, i;
@@ -1025,15 +1018,23 @@ static void openprom_read_inode(struct inode * inode)
 	}
 }
 
+static int openprom_remount(struct super_block *sb, int *flags, char *data)
+{
+	*flags |= MS_NOATIME;
+	return 0;
+}
+
 static struct super_operations openprom_sops = { 
 	.read_inode	= openprom_read_inode,
 	.statfs		= simple_statfs,
+	.remount_fs	= openprom_remount,
 };
 
 static int openprom_fill_super(struct super_block *s, void *data, int silent)
 {
 	struct inode * root_inode;
 
+	s->s_flags |= MS_NOATIME;
 	s->s_blocksize = 1024;
 	s->s_blocksize_bits = 10;
 	s->s_magic = OPENPROM_SUPER_MAGIC;
@@ -1053,7 +1054,7 @@ out_no_root:
 }
 
 static struct super_block *openprom_get_sb(struct file_system_type *fs_type,
-	int flags, char *dev_name, void *data)
+	int flags, const char *dev_name, void *data)
 {
 	return get_sb_single(fs_type, flags, data, openprom_fill_super);
 }
